// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.logic;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.MetaOperator;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.rule.inst.SVInstantiations;


/** Meta operator for ASMs. This abstract class implements the
 * MetaOperator interface. The method calculate() is called each time
 * a taclet with a meta operator is applied. */

abstract class AsmMetaOperator extends AsmOperator implements MetaOperator {

    /** construct a new asm meta operator with the given name, the
     * given sort and the given subterms. */
    AsmMetaOperator(String name, Sort sort, Sort[] args) {
        super(name, sort, args);
    }

    public final Term calculate(Term term, SVInstantiations svInst, Services services) {
        return calculate(term, svInst);
    }

    /** This method is called each time a taclet with this operator is
     * applied. The top level operator of 'term' is this
     * operator. 'svInst' contains the variable instantiations of the
     * taclet application.  The term returned by this method replaced
     * the given term in the sequent. */
    protected abstract Term calculate(Term term, SVInstantiations svInst);

}
