// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.logic;


import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.Term;


/** Dynamic (non-rigid) function. The interpretation of a dynamic
 * (non-rigid) function may depend on the current state.  All dynamic
 * (non-rigid) functions must be intances of this class.  All other
 * operators are expected to be static (rigid).
 *
 * @author Hubert Schmid */

public class DynamicFunction extends Function {

    /** creates a new instance with the given name, sort and sorts of
     * arguements. See {@link Function#Function(Name, Sort,
     * Sort[])}. */
    public DynamicFunction(Name name, Sort sort, Sort[] argSorts) {
        super(name, sort, argSorts);
    }

    /**
     * @return true if the value of "term" having this operator as
     * top-level operator and may not be changed by modalities
     */
    public boolean isRigid (Term term) {
	return false;
    }

}
