// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.logic;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.logic.op.Operator;
import de.uka.ilkd.key.logic.op.Junctor;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.rule.inst.SVInstantiations;


/** Meta operators to remove the
 * short circuits junctor.
 * @author Hubert Schmid
 * @author Stanislas Nanchen
 */

public class MetaSCElimination extends AsmMetaOperator {
    
    /** Short reference to AsmTermFactory. */
    private static final AsmTermFactory tf = AsmTermFactory.ASMDEFAULT;

    /** creates a new meta operator. Only one instance is required, so
     * this class may be a singleton. */
    public MetaSCElimination() {
        super("#SC_ELIMINATION", Sort.FORMULA,
              new Sort[] { Sort.FORMULA });
    }

    /** Expects one formula with possible short circuit operators.
     * @return The equivalent formula without short circuit operators.
     */
    public Term calculate(Term term, SVInstantiations svInst) {
	return eliminate(term.sub(0));
    }

    /** Recursive function to eliminate the short circuit operators. */
    private Term eliminate(Term term) {
	Operator op;

	if(term.op() instanceof Junctor) {
	    if (term.arity() == 0) {
		return term;
	    } else if (term.arity() == 1) {
		return createSubstitutedOpTerm((Junctor) term.op(), term);
	    } else if (term.arity() == 2) {
		op = term.op();
		
		if (op == AsmJunctor.ANDALSO) {
		    op = Op.AND;
		} else if (op == AsmJunctor.ORELSE) {
		    op = Op.OR;
		} 
		return createSubstitutedOpTerm((Junctor) op, term);
	    } else {
		return createSubstitutedOpTerm((Junctor) term.op(), term);
	    }
	} else {
	    return term;
	}
    }

    private Term createSubstitutedOpTerm(Junctor op, Term term) {
	Term[] newterms;

	if (op.arity() == term.arity()) {
	    if (op.arity() == 0) {
		return tf.createJunctorTerm(op);
	    } else {
		newterms = new Term[op.arity()];
		for (int i = 0; i < op.arity(); i++) {
		    newterms[i] = eliminate(term.sub(i));
		}
		return tf.createJunctorTerm(op, newterms);
	    }
	} else {
	    throw new IllegalArgumentException("Arity of Operator " + op.name() +
					       " does not correspond to arity of term " + term.toString()
					       + ".");
	}
    }

}
