// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.logic;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.asmkey.logic.CollectionSort;

/** This class is used with the big operators. It is
 * used to generate either:
 * - n variables in the quantifier.
 * - a function call with these n variables as argument in
 *   the formula of a big operator.
 *
 * - its sorts is the universal one. i.e. it always match correctly
 *   as the real sorts will be checked after expansion.
 *
 * @author Stanislas Nanchen 
 */


public class NArityLogicVariable extends LogicVariable {

    public NArityLogicVariable(Name name) {
	super(name, AbstractMetaOperator.METASORT);
    }
    
    /** @return arity of the Variable as int */
    public int arity() {
	return 0;
    }

    /** @return the name of the function attached to this NArityLogicVariable */
    public Name functionName() {
	StringBuffer buf = new StringBuffer(name().toString());
	return new Name(buf.substring(1));
    }
    
    /** toString */
    public String toString() {
	return ""+name()+":"+sort();
    }

}
