// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.logic;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.asmkey.logic.*;
import de.uka.ilkd.asmkey.parser.env.DefaultEnvironment;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/** This class computes the strongly connected components
 * using the tarjan algorithm (depth-first). 
 */


class SCC {

    private HashMapFromAsmProcedureToAsmRule procs;
    private HashMapFromAsmRuleToInteger visitedToken;
    private HashMapFromAsmRuleToInteger lowLink;
    private ListOfAsmRule stack;
    private DefaultEnvironment env;
    private HashMapFromAsmRuleToListOfAsmRule sccs;
    private int c;

    public SCC (DefaultEnvironment env, HashMapFromAsmProcedureToAsmRule procs) {
	visitedToken = new HashMapFromAsmRuleToInteger();
	lowLink = new HashMapFromAsmRuleToInteger();
	stack = SLListOfAsmRule.EMPTY_LIST;
	sccs = new HashMapFromAsmRuleToListOfAsmRule();
	this.procs = procs;
	//sccs = SLListOfListOfAsmRule.EMPTY_LIST;
	this.env = env;
	c = 0;
    }
    
    public void doAnalysis() {
	List namedRules = env.getProcedures();
	
	Iterator it = namedRules.iterator();
	AsmProcedure namedRule;

	while (it.hasNext()) {
	    namedRule = (AsmProcedure) it.next();
	    if (! visitedToken.containsKey(procs.get(namedRule))) {
		c = 0;
		sc(procs.get(namedRule));
	    }
	}
	it = namedRules.iterator();
	while (it.hasNext()) {
	    namedRule = (AsmProcedure) it.next();
	    if(sccs.get(procs.get(namedRule)).size() > 1) {
		namedRule.setRec(true);
	    }
	}
    }

    private void sc(AsmRule v) {
	AsmRule w;
	Integer integer = new Integer(c);
	
	c++;
	
	visitedToken.put(v, integer);
	lowLink.put(v, integer);
	stack = stack.prepend(v);
	
	if (v.op() instanceof AsmProcedure.Operator) {
	    visit(v, procs.get(((AsmProcedure.Operator) v.op()).getProcedure()));
	} else {
	    for(int i = 0; i<v.arity(); i++) {
		if (v.sub(i) instanceof AsmRule) {
		    visit(v, (AsmRule) v.sub(i));
		}
	    }
	}

	if(lowLink.get(v).compareTo(visitedToken.get(v)) == 0) {
	    // we have a scc component. !
	    ListOfAsmRule component = SLListOfAsmRule.EMPTY_LIST;
	    w = null;
	    while (v != w) {
		w = stack.head();
		stack = stack.tail();
		component = component.prepend(w);
	    }
	    IteratorOfAsmRule it = component.iterator();
	    while(it.hasNext()) {
		sccs.put(it.next(), component);
	    }
	}
    }

    private void visit(AsmRule v, AsmRule w) {
	Integer vint, wint;

	if (visitedToken.containsKey(w)) {
	    if (! sccs.containsKey(w)) {
		vint = lowLink.get(v);
		wint = visitedToken.get(w);

	    
		lowLink.put(v, min(vint, wint));
	    }
	} else {
	    sc(w);
	    
	    vint = lowLink.get(v);
	    wint = lowLink.get(w);
	    lowLink.put(v, min(vint, wint));
	}
    }


    private static Integer min(Integer x, Integer y) {
	if (x.compareTo(y) < 0) {
	    return x;
	} else {
	    return y;
	}
    }
    
    IteratorOfListOfAsmRule getSSCs() {
	return sccs.values();
    }

}
