// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

header {
    package de.uka.ilkd.asmkey.parser.antlr;
}


class AsmKeyLexer extends Lexer;


options {
    // lookahead 3 is required for symbol '<-' versus '<->'.
    k = 3; 
    charVocabulary = '\1'..'\377';
}


tokens {
    /* quantifiers */
    FORALL = "all";
    EXISTS = "ex";

    /* literals */
    TRUE = "true";
    FALSE = "false";

    /* taclets */
    FIND = "find";
    IF = "if";
    CLOSE = "close";
    GOAL = "goal";
    REPLACEWITH = "replacewith";
    REPLACEBRANCH = "replacebranch";
    ADDRULES = "addrules";
    ADD = "add";
    BUILTIN = "builtin";
    /* try for asmkey */
    APPLY = "apply";
    TACLET = "taclet";
    /* cond */
    VARCOND = "varcond";
    COND = "cond";
    TYPEOF = "typeof";
    NEW = "new";
    OP = "op";
    NOT = "not";
    FREE = "free";
    PURE = "pure";
    DEPENDING = "depending";
    ON = "on";
    
    NONINTERACTIVE = "noninteractive";
    RECURSIVE = "recursive";
    DISPLAYNAME = "displayname";
    HELPTEXT = "helptext";

    /* proof */
    PROOF = "proof";
    DEPENDS = "depends";
    TERM = "term";
    BRANCH = "branch";
    INST = "inst";
    IFSEQFORMULA = "ifseqformula";
    INTERACTIVE = "interactive";
    HEUR = "heur";

    PROBLEM = "problem";
    PROCEDURES = "procedures";
    CALL = "call";
    IS = "is";

    /* declarations */
    SORTS = "sorts";
    SCHEMA = "schema";
    VARIABLES = "variables";
    CONSTS = "consts";
    PREDICATES = "predicates";
    FUNCTIONS = "functions";
    FORMULA = "formula";
    METARULES = "metarules";
    RULES = "rules";
    HYPERRULES = "hyperrules";
    HEURISTICS = "heuristics";
    LEMMAS = "lemmas";
    PACKAGES = "packages";
    IMPORTFILES = "importdecls";
    IMPORTLEMMAS = "importlemmas";
    FILE = "file";
    USE = "use";
    MAY = "may";
    ACCESS = "access";
    UPDATE = "update";
    DERIVED = "derived";

    RULE = "rule";

    ASM = "asm";
    DYNAMIC = "dynamic";
    STATIC = "static";
    STATICARGS = "staticargs";
    ATOMIC = "atomic";
    GENERIC = "generic";

    /* asm */
    ASMDEF = "def";
    ASMUPD = "upd";
    ASMCON = "Con";
    ASMINV = "inv";
    ASMSKIP = "skip";
    ASMEQUI = "equi";
    ASMEQUIS = "equiS";
    ASMEQUIA = "equiA";
    ASMACCT = "accT";
    ASMACCF = "accF";
    ASMACC = "acc";
    ASMTHEN = "then";
    ASMELSE = "else";
    ASMELSEIF = "elseif";
    ASMEND = "end";
    ASMIN = "in";
    ASMLET = "let";
    ASMFORALL = "forall";
    ASMWITH = "with";
    ASMDO = "do";
    ASMTRY = "try";
    ASMJOINABLE = "joinable";
    ASMAND = "and";
    ASMPAR = "par";
    ASMSEQ = "seq";

    /* big operator */

    BIGAND = "bigand";
    BIGOR = "bigor";
    FUNCTION = "function";
    
}


SEMI
    : ';' ;

COLON
    : ':' ;

ASSIGN
    : ":=" ;

DOT
    : '.' ;

COMMA
    : ',' ;

LPAREN
    : '(' ;

RPAREN
    : ')' ;

LBRACE
    : '{' ;

RBRACE
    : '}' ;

LSETBRACK
    : ":[";

LBRACK
    : '[' ;

RBRACK
    : ']' ;

LSEQBRACK
    : ":<" ;

DLBRACK
    : "[[" ;

DRBRACK
    : "]]";

AT
    : '@' ;

ORELSE
    : "||";

ANDALSO
    : "&&";

OR
    : '|' ;

AND
    : '&' ;

NOT
    : '!' ;

IMP
    : "->" ;

EQV
    : "<->" ;

EQUALS
    : '=' ;

NOTEQUALS
    : "!=" ;

LT
    : '<' ;

GT
    : '>' ;

SEQARROW
    : "==>" ;

PLUS
    : '+' ;

MINUS
    : '-' ;

ASTERIX
    : '*' ;

SLASH
    : '/';

UPDATE
    : "<-" ;

TIMED
    : '^' ;





protected
DIGIT
options {
    paraphrase = "a digit";
}
    : '0'..'9' ;


protected
LETTER
options {
    paraphrase = "a letter";
}
    : 'A'..'Z' | 'a'..'z' ;


protected
IDCHAR
options {
    paraphrase = "an admissible character for identifiers";
}
    : LETTER | DIGIT | '_' | '~';


IDENT
options {
    testLiterals = true;
    paraphrase = "an identifer or a timed identifier";
}
//    : ( IDCHAR | '#' ) ( IDCHAR )* (('^')? {$setType(IDENTTIMED);});
    :
        (IDCHAR '.')* ( IDCHAR )+ ('^' {$setType(IDENTTIMED);})?
    |
        '#' (IDCHAR)*
    ;

IDENTTIMED: "...";

IDENTNARITY
    :
        '%' (IDCHAR)*
    ;

/* whitespace */
WS
options {
    paraphrase = "white space";
}
    :
        (
            ' '
        |
            '\t'
        |
            '\n' { newline(); }
        |
            '\r'
        )
        {
            $setType(Token.SKIP);
        }
    ;


/* single line comment */
SL_COMMENT
options {
    paraphrase = "a comment";
}
    :
        "//"
        (~'\n')* '\n'
        {
            $setType(Token.SKIP);
            newline();
        }
    ;


/* multi line comment */
ML_COMMENT
options {
    paraphrase = "a comment";
}
	:
        "/*"
		(
			options {
				generateAmbigWarnings=false;
			}
		:
			{ LA(2) != '/' }? '*'
		|	'\n' { newline(); }
		|	~( '*' | '\n' )
		)*
		"*/"
        {
            $setType(Token.SKIP);
        }
	;


STRING_LITERAL
options {
    paraphrase = "a string in double quotes";
}
    : '"'! ( ESC | ~('"'|'\\') )* '"'! ;


protected
ESC
    :	'\\'
        (	'n'         { $setText("\n"); } 
        |	'r' { $setText("\r"); }
        |	't' { $setText("\t"); }
        |	'b' { $setText("\b"); }
        |	'f' { $setText("\f"); }
        |	'"' { $setText("\""); }
        |	'\'' { $setText("'"); }
        |	'\\' { $setText("\\"); }
        )
    ;


// Local variables:
// mode: Antlr
// End:
