// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.parser.ast;


/** Visitor for declarations.
 *
 * @see AstDeclaration
 * @author Hubert Schmid
 */

public interface AstDeclarationVisitor {

    /** Declaration of a new sort. */
    void visitSort(boolean isGenericSort);

    /** Declaration of a schema variable. */
    void visitSchemaVariable(AstSchemaType type);

    /** Declaration of a predicate symbol. */
    void visitPredicate(Identifier[] args);

    /** Declaration of a function symbol. */
    void visitFunction(Identifier[] args, Identifier ret, boolean dynamic);

    /** Declaration of a heuristic. */
    void visitHeuristic();

    /** Taclet declaration. */
    void visitTaclet(AstTaclet taclet);

    /** Named ASM rule (procedure) declaration. */
    void visitProcedure(AstParameter[] params, AstAsmRule rule);

    /** Lemma declaration (that a proof may use) */
    void visitLemma(AstTerm phi);

}
