// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.parser.env;


import de.uka.ilkd.asmkey.logic.AsmProcedure;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.Sort;


/** Implementation of TermEnvironment that forwards (delegates) all
 * queries to another TermEnvironment.
 *
 * @author Hubert Schmid
 */

class DelegateTermEnvironment implements TermEnvironment {

    /** The environment to which all queries are forwarded. */
    private final TermEnvironment env;


    DelegateTermEnvironment(TermEnvironment env) {
        this.env = env;
    }


    public boolean containsMetaOp(String id) {
        return env.containsMetaOp(id);
    }

    public MetaOperator getMetaOp(String id) throws EnvironmentException {
        return env.getMetaOp(id);
    }

    public Sort getSort(String id) throws EnvironmentException {
        return env.getSort(id);
    }

    public boolean containsSchemaVariable(String id) {
        return env.containsSchemaVariable(id);
    }

    public SchemaVariable getSchemaVariable(String id) throws EnvironmentException {
        return env.getSchemaVariable(id);
    }

    public Operator getOperator(String id) throws EnvironmentException {
        return env.getOperator(id);
    }

    public AsmProcedure getProcedure(String id) throws EnvironmentException {
        return env.getProcedure(id);
    }

    public Term getAbbreviatedTerm(String name) throws EnvironmentException {
        return env.getAbbreviatedTerm(name);
    }

}
