// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.pp;


import java.util.HashMap;
import java.util.Map;

import de.uka.ilkd.asmkey.logic.*;
import de.uka.ilkd.key.gui.Notation;
import de.uka.ilkd.key.gui.NotationInfo;
import de.uka.ilkd.key.logic.op.Operator;


/** This class extends {@link de.uka.ilkd.key.gui.NotationInfo} and adds {@link de.uka.ilkd.key.gui.Notation}s
 * for ASM terms and rules.
 *
 * @author Hubert Schmid 
 * @author Stanislas Nanchen  
 */

public final class AsmNotationInfo extends NotationInfo {

        /**
         * Creates a new Operator Notation table.
         * Every Operator gets assigned its prefix, infix(es), postfix and priority.
         */
        private static Map createNotation() {
                Map m = new HashMap();
		m.put(AsmOperator.EMPTY_SEQUENCE, AsmNotation.empty_sequence());
		m.put(AsmOperator.CONS, AsmNotation.sequence());
                m.put(AsmOperator.BOX, AsmNotation.box());
                m.put(AsmOperator.DIAMOND, AsmNotation.diamond());
                m.put(AsmOperator.DEF, AsmNotation.def());
                m.put(AsmOperator.UPD, AsmNotation.upd());
                m.put(AsmOperator.CON, AsmNotation.Con());
                m.put(AsmOperator.INV, AsmNotation.inv());
                m.put(AsmOperator.JOINABLE, AsmNotation.joinable());
                m.put(AsmOperator.EQUI, AsmNotation.equi());
                m.put(AsmOperator.EQUIS, AsmNotation.equiS());
		m.put(AsmOperator.EQUIA, AsmNotation.equiA());
		m.put(AsmOperator.ACCT, AsmNotation.acct());
		m.put(AsmOperator.ACCF, AsmNotation.accf());
		m.put(AsmOperator.ACC, AsmNotation.acc());
                m.put(AsmOperator.SEQ, AsmNotation.seq());
                m.put(AsmOperator.PAR, AsmNotation.par());
                m.put(AsmOperator.SKIP, AsmNotation.skip());
                m.put(AsmOperator.ASSIGN, AsmNotation.assign());
                m.put(AsmOperator.IF, AsmNotation.if_());
		m.put(AsmOperator.ELSEIF, AsmNotation.elseif());
                m.put(AsmOperator.LET, AsmNotation.let());
                m.put(AsmOperator.FORALL, AsmNotation.forall());
                m.put(AsmOperator.TRY, AsmNotation.try_());
		m.put(AsmJunctor.ANDALSO, AsmNotation.andalso());
		m.put(AsmJunctor.ORELSE, AsmNotation.orelse());
		m.put(AsmJunctor.TIMED, AsmNotation.timed());
                m.put(AsmProcedure.Operator.class, AsmNotation.named());
                return m;
        }


        /** Stores only the ASM notations. */
        private final Map map;


        public AsmNotationInfo() {
                map = createNotation();
        }


        public Notation getNotation(Operator op) {
                /* First check if a matching ASM notation exists. Else
                 * forward the call to the super class. */
                if (map.containsKey(op)) {
                        return (Notation) map.get(op);
                } else if (map.containsKey(op.getClass())) {
                        return (Notation) map.get(op.getClass());
                } else {
                        return super.getNotation(op);
                }
        }

}
