// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.rule;


import de.uka.ilkd.asmkey.logic.AsmOperator;
import de.uka.ilkd.asmkey.logic.AsmUtil;
import de.uka.ilkd.asmkey.logic.AsmLemma;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.rule.BuiltInRule;
import de.uka.ilkd.key.rule.RuleApp;
import de.uka.ilkd.key.util.Debug;



/** This class represent a lemma that a user can
 * apply. it is applicable only when it matches
 * a term that is at the left of 
 *
 * @author Hubert Schmid
 * @author Stanislas Nanchen
 */

public final class LemmaRule implements BuiltInRule {
    
    private AsmLemma lemma;

    public LemmaRule(AsmLemma lemma) {
	this.lemma = lemma;
    }

    /** If lemmas are defined, the lemma rule is always applicable. */
    public boolean isApplicable(Goal goal, PosInOccurrence pos, Constraint constraint) {
	//TODO match mechanism.
	return true;
    }

    /** Simply add the formula at the left of the sequent
     * TODO : intelligent term substitution
     */
    public ListOfGoal apply(Goal goal, Services services, RuleApp ruleApp) {
	ListOfGoal result = goal.split(1);
        Goal newGoal = result.head();
	newGoal.addFormula(new ConstrainedFormula(lemma.getFormula()),true,true);
	return SLListOfGoal.EMPTY_LIST.prepend(newGoal);
    }

    public Name name() {
	return new Name("lemma:" + lemma.getId());
    }

    public String displayName() {
	return "lemma:" + lemma.getId();
    }

    public AsmLemma lemma() {
	return lemma;
    }

}
