// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.rule;

import java.util.Collection;
import java.util.Iterator;
import de.uka.ilkd.asmkey.logic.*;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.rule.BuiltInRule;
import de.uka.ilkd.key.rule.RuleApp;
import de.uka.ilkd.key.util.Debug;


/** This class is a builtin rule it is the abstract class
 * for BigOperatorRule and FunctionOperatorRule and
 * contains the common functionnality. in particular,
 * the substitution mechanism for narity logic variables
 *
 * @author Stanislas Nanchen
 */

abstract public class OperatorRule implements BuiltInRule {

     /** Short refernce to TermFactory. */
    private static final TermFactory tf = TermFactory.DEFAULT;

    public abstract boolean isApplicable(Goal goal, PosInOccurrence pos, Constraint constraint);

    public ListOfGoal apply(Goal goal, Services services, RuleApp ruleApp) {
	Term term = ruleApp.posInOccurrence().constrainedFormula().formula();
	Term newTerm = calculate(term, services);

	ListOfGoal result = goal.split(1);
        Goal newGoal = result.head();
	newGoal.changeFormula(new ConstrainedFormula(newTerm), ruleApp.posInOccurrence());
	return result;
    }

     /** This method replaces a big operator by a suitable formula */
    abstract protected Term calculate(Term term, Services services);
    
    protected Term transform_and_subst(Term formula,
				       Name var,
				       LogicVariable[] xs,
				       Term fterm, Services services) {
	Term[] subterms = new Term[formula.op().arity()];
	Operator op = formula.op();
	Term result;

	if (op.name().equals(var)) {
	    /* We have found our NArityLogicVariable as term.
	     */
	    result = fterm;
	} else {
	    /* We are not at the end of the tree and must recursively
	     * go down.
	     */
	    for(int i=0; i<subterms.length; i++) {
		subterms[i] = transform_and_subst(formula.sub(i), var, xs, fterm, services);
	    }
	    
	    ArrayOfQuantifiableVariable vars = formula.varsBoundHere(0);
	    if ((op == Op.ALL || op == Op.EX) &&
		vars.size() == 1 &&
		vars.getQuantifiableVariable(0).name().equals(var)) {
		/* if we have our found a NArityLogicVariable var, we must
		 * make the remplacement.
		 */
		result = subterms[0];
		for (int i = xs.length - 1; i >= 0; i--) {
		    result = tf.createQuantifierTerm((Quantifier) op, xs[i], result);
		}
	    } else if ((op instanceof MetaOperator)) {
		/* we must provoke the subsisution */
		result = tf.createTerm(op,
				       subterms,
				       op.arity()==0?Term.EMPTY_VAR_LIST:formula.varsBoundHere(0),
				       formula.javaBlock(),
				       formula.ruleVar());
		result = ((MetaOperator)result.op()).calculate(result, null, services);
	    } else if ((op instanceof BigOperator) &&
		       ((BigOperator) op).getVariableName().equals(var)) {
		/* we check if we should subsitute further, VERY UGLY */
		result = formula;
	    } else {
		/* we can simply reconstitute */
		result = tf.createTerm(op,
				       subterms,
				       op.arity()==0?Term.EMPTY_VAR_LIST:formula.varsBoundHere(0),
				       formula.javaBlock(),
				       formula.ruleVar());
	    }
	}
	
	return result;
    }

    abstract public Name name();

    abstract public String displayName();

}
