// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.casetool.together;

import de.uka.ilkd.key.casetool.*;
import de.uka.ilkd.key.casetool.together.TogetherReprModelClass;
import tudresden.ocl.check.types.Type;
import java.io.IOException;
import java.util.Iterator;

import com.togethersoft.openapi.rwi.RwiModel;
import com.togethersoft.openapi.rwi.RwiNode;
import com.togethersoft.openapi.rwi.RwiModelAccess;
import com.togethersoft.openapi.rwi.RwiPackage;
import com.togethersoft.openapi.rwi.RwiProperty;
import com.togethersoft.openapi.rwi.enum.RwiPackageEnumeration;
import com.togethersoft.openapi.rwi.enum.RwiNodeEnumeration;
import com.togethersoft.openapi.sci.SciModel;
import com.togethersoft.openapi.sci.SciModelAccess;
import com.togethersoft.openapi.ide.project.*;
import com.togethersoft.openapi.ide.IdeAccess;

import recoder.abstraction.Method; 
import recoder.abstraction.Field; 
import recoder.abstraction.ClassType; 
import recoder.list.CompilationUnitList;
import recoder.list.MethodList;
import recoder.list.FieldList;
import recoder.list.StringArrayList;
import recoder.list.TypeList;
import recoder.list.ClassTypeList;
import recoder.java.declaration.TypeDeclaration; 
import recoder.io.SourceFileRepository;
import recoder.io.DefaultSourceFileRepository;
import recoder.util.FileCollector;
import recoder.java.CompilationUnit;
import recoder.service.DefaultSourceInfo;
import recoder.ServiceConfiguration;
import java.io.File;

public class UMLOCLTogetherModel extends UMLOCLJavaModel {
    //debug flag
    private static final boolean DEBUG = false;
    private static String pathSep = File.separator;


    public UMLOCLTogetherModel() {
	super(getTogetherProjectDir());
    }

    public static String getTogetherProjectDir() {
	IdeProject prj = IdeProjectManagerAccess
	    .getProjectManager().getActiveProject();
	if (prj != null) {
	    String tprFile = prj.getFileName();
	    return  tprFile.substring(0, tprFile.lastIndexOf(pathSep));
	} else
	    return "";
    }

    
    public HashMapOfClassifier getUMLOCLClassifiers() {
	      
	super.getUMLOCLClassifiers();
	// find associations
	RwiModel rwiModel = RwiModelAccess.getModel();		
	RwiPackageEnumeration rwiRoots=rwiModel.
	    rootPackages(RwiProperty.MODEL);
	while (rwiRoots.hasMoreElements()) {
	    RwiPackage rwiPackage = rwiRoots.nextRwiPackage();
	    processAssociations(rwiPackage);
	}

	if (DEBUG)
	    System.out.println("------------- found the following "+
			       "associations ------------------");

	Iterator cit = classifiers.values().iterator();
	UMLOCLClassifier c;
	while (cit.hasNext()) {
	    c = (UMLOCLClassifier)cit.next();
	    Iterator it = c.getAssociations().values().iterator();
	    UMLOCLAssociation assoc;
	    while (it.hasNext()) {
		assoc = (UMLOCLAssociation)it.next();
		if (DEBUG)
		    System.out.println("assoc from: "
				       +assoc.getSource().getName()+
				       " to: "+assoc.getTarget().getName());
	    }
	}
	return classifiers;
    }

    protected void processAssociations(RwiPackage rwiPackage){
	SciModel sciModel = SciModelAccess.getModel();
	RwiModel rwiModel = RwiModelAccess.getModel();

	RwiNodeEnumeration rwiNodesEnum = rwiPackage.nodes();
	RwiNode rwiNode;
	TogetherReprModelClass cls;
	while (rwiNodesEnum.hasMoreElements()){
	    rwiNode = rwiNodesEnum.nextRwiNode();
	    cls = new TogetherReprModelClass(rwiNode, rwiModel, null);
	    cls.getAssociations(classifiers);
	}	
	RwiPackageEnumeration subpackages = rwiPackage.subpackages();
	while (subpackages.hasMoreElements()) {
	    RwiPackage subpackage = subpackages.nextRwiPackage();
	    processAssociations(subpackage);
	}
    }


}
