// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.gui;


import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.util.pp.*;
import java.io.IOException;

public class PresentationFeatures {


    private PresentationFeatures() {}

    private static void putInfixNotation(NotationInfo notInfo, 
					 Namespace functions,
					 String op,
					 String token) {
	notInfo.createInfixNotation((Function) functions.lookup(new Name(op)), token);
    }

    private static void putPrefixNotation(NotationInfo notInfo, 
					  Namespace functions,
					  String op,
					  String token) {
	notInfo.createPrefixNotation((Function) functions.lookup(new Name(op)), token);
    }

    public static boolean ENABLED=false;

    public static void modifyNotationInfo(NotationInfo notInfo,
					  Namespace functions) {
	if (ENABLED) {
	    putInfixNotation(notInfo, functions, "lt", "< ");
	    putInfixNotation(notInfo, functions, "gt", "> ");
	    putInfixNotation(notInfo, functions, "jlt", "<");
	    putInfixNotation(notInfo, functions, "jgt", ">");
	    putInfixNotation(notInfo, functions, "leq", "<=");
	    putInfixNotation(notInfo, functions, "geq", ">=");
	    putInfixNotation(notInfo, functions, "jleq", "<=");
	    putInfixNotation(notInfo, functions, "=", "=");
	    putInfixNotation(notInfo, functions, "jgeq", ">=");
	    putInfixNotation(notInfo, functions, "+", "+");
	    putInfixNotation(notInfo, functions, "mul", "*");
	    putInfixNotation(notInfo, functions, "~d", "-");
	    putPrefixNotation(notInfo, functions, "~m", "-");
	    notInfo.createNumLitNotation
		((Function)functions.lookup(new Name("Z")));
	    putPrefixNotation(notInfo, functions, "neg", "-");
	    
	}
    }

    private static StringBuffer printlastfirst(Term t) {
	if (t.op().arity()==0) {
	    return new StringBuffer();
	} else {
	    return printlastfirst(t.sub(0)).
		append(t.op().name().toString());
	}
    }

    public static String printNumberTerm(Term t) 
    {
	StringBuffer sb=new StringBuffer();
	if (t.sub(0).op().name().toString().equals("neg")) {
	    return "-"+
		printlastfirst(t.sub(0).sub(0)).toString();
	} else {
	    return printlastfirst(t.sub(0)).toString();
	}
    }

    public static void initialize(Namespace funcns, NotationInfo ni, Proof p) {
	if (ENABLED) {
	    modifyNotationInfo(ni, funcns);
	    if (p!=null) p.updateProof();
	}	
    }

}
