// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.gui;

import de.uka.ilkd.key.proof.ListOfNode;
import de.uka.ilkd.key.proof.SLListOfNode;
import de.uka.ilkd.key.proof.proofevent.RuleAppInfo;


/** an object indicating that a proof event has happpend */
public class ProofEvent {
    
    private InteractiveProver source;

    private ListOfNode modifiedSubtrees;


    /** creates a new proof event with the interactive prover where
     * the event initially occured; this constructor is used for
     * <code>autoModeStopped</code>
     * @param source 
     * @param modifiedSubtree roots of subtrees containing all nodes
     * to which rules have been applied
     */
    public ProofEvent(InteractiveProver source,
		      ListOfNode modifiedSubtrees) {
	this.source = source;
	this.modifiedSubtrees = modifiedSubtrees;
    }
    
    /** creates a new proof event with the interactive prover where the
     * event initially occured
     * @param source 
     */
    public ProofEvent(InteractiveProver source) {
	this ( source, SLListOfNode.EMPTY_LIST );
    }
    
    
    /** creates a new proof event the interactive prover where the
     * event initially occured 
     * @param source 
     * @param message a String message
    public ProofEvent(InteractiveProver source, String message) {
	this.source = source;
	this.message = message;
    }
     */


    /** the interactive prover where the event initially occured
     * @return the interactive prover where the event initially
     * occured
     */
    public InteractiveProver getSource() {
	return source;
    }

    /**
     * Information for <code>autoModeStopped</code>
     * @return the root of a subtree containing all nodes to which
     * rules have been applied, or <code>null</code> if no rules have
     * been applied at all
     */
    public ListOfNode getModifiedSubtrees () {
	return modifiedSubtrees;
    }
    
/*
    public String getMessage() {
	return message;
    }
*/


    /**
     * This information should have its own event, but is currently
     * propagated via this one
     */
    public RuleAppInfo getRuleAppInfo () {
	return ruleAppInfo;
    }

    public ProofEvent(InteractiveProver source,
		      RuleAppInfo       rai) {
	this ( source );
	ruleAppInfo = rai;
    }

    private RuleAppInfo ruleAppInfo = null;

}
