// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import de.uka.ilkd.key.proof.RuleTreeModel;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.proof.mgt.MethodContract;

public class RuleView extends JPanel implements java.io.Serializable {

    protected RuleTreeModel ruleViewModel    = null;
    protected JTree ruleTree;

    protected KeYMediator mediator = null;

    /** Listener for proof changes */
    protected SelectionListener selectionListener = new SelectionListener ();


    public RuleView () {
	layoutPane();
	ruleTree.setCellRenderer(new RuleRenderer());
	setVisible ( true );
    }


    public void setMediator ( KeYMediator p_mediator ) {
	if ( mediator != null )
	    unregisterAtMediator ();
	
	mediator = p_mediator;
	registerAtMediator ();
	if ( mediator != null && mediator.getSelectedProof()!=null) {
	    setRuleTreeModel(new RuleTreeModel(mediator.getSelectedGoal()));
	}
    }


    protected void setRuleTreeModel(RuleTreeModel model) {

	ruleViewModel = model;

	if ( ruleViewModel != null ) {
	    ruleTree.setModel ( ruleViewModel );
	}
    }


    protected void registerAtMediator () {
	mediator.addKeYSelectionListener              ( selectionListener );
    }


    protected void unregisterAtMediator () {
	mediator.removeKeYSelectionListener           ( selectionListener );
    }


    protected void layoutPane() {
	setLayout(new BorderLayout());
	ruleTree = new JTree();
	add(ruleTree, BorderLayout.CENTER);
    }



    /**
     * Will be called when this dialog will be closed
     */
    protected void closeDlg(){
	//	mediator.freeModalAccess(this); 	
    }    


    private class SelectionListener implements KeYSelectionListener {

	/** focused node has changed */
	public void selectedNodeChanged(KeYSelectionEvent e) {
	    ruleViewModel.setSelectedGoal(e.getSource().getSelectedGoal());
	}

	/** the selected proof has changed (e.g. a new proof has been
	 * loaded) */ 
	public void selectedProofChanged(KeYSelectionEvent e) {
            Runnable action = new Runnable () {
		public void run () {
		    if ( mediator != null && mediator.getSelectedProof()!=null)
			setRuleTreeModel(new RuleTreeModel
					 (mediator.getSelectedGoal()));
		}
	    };
        
	    if (SwingUtilities.isEventDispatchThread())  
                action.run();
            else  SwingUtilities.invokeLater(action);
	}
	
    }

    class RuleRenderer extends DefaultTreeCellRenderer 
	implements TreeCellRenderer,
		   java.io.Serializable {

	JPanel panel;
	JPanel statusPanel;
	JTextArea text;	

	public RuleRenderer() {
	    FlowLayout lay = new FlowLayout();
	    lay.setAlignment(FlowLayout.LEFT);	   	    
	    panel = new JPanel(lay);
	    statusPanel = new JPanel(lay);
	    text = new JTextArea();
	    panel.add(statusPanel);
	    panel.add(text);	    
	    text.setEditable(false);
	    text.setCaretPosition(0);
	}

	public Component getTreeCellRendererComponent(JTree tree,
						      Object value,
						      boolean sel,
						      boolean expanded,
						      boolean leaf,
						      int row,
						      boolean hasFocus) {
	    Component comp = super.getTreeCellRendererComponent
		(tree, value, sel, expanded, leaf, row, hasFocus);

	    if (((DefaultMutableTreeNode)value).getUserObject() 
		instanceof MethodContract) {
		MethodContract mc = (MethodContract) 
		    ((DefaultMutableTreeNode)value).getUserObject();
		comp = MethodContractList.makeMethodContractDisplay
		    (mc, panel, text, statusPanel,
		     sel ? super.backgroundSelectionColor : 
		     super.backgroundNonSelectionColor);
	    }
	    return comp;


	}
    }


}
