// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

public class SimultaneousUpdateSimplifierConfiguration extends JDialog {

    private SimultaneousUpdateSimplifierSettings settingsFromConfigurationFile;

    private KeYMediator mediator = null;

    private boolean local_use_sim = false;
    private boolean local_implicit_execution = false;
    private boolean local_del_simple = false;
    private boolean local_del_unused = false;


    /** creates a new HeuristicSelector */
    public SimultaneousUpdateSimplifierConfiguration
	(KeYMediator mediator, 
	 SimultaneousUpdateSimplifierSettings settings) {  
	super(new JFrame(), "Update Simplification", true);

	settingsFromConfigurationFile = settings;

	local_use_sim = useSimultaneousUpdates();
	local_implicit_execution = implicitExecution();
	local_del_simple = deleteSimpleUpdates();
	local_del_unused = deleteUnusedUpdates();

	setMediator(mediator);

	layoutSimutanUpdatesimplifier(); 
	
	pack();
	setLocation(70,70);
	setVisible(true);
    }


    /** sets the mediator to correspond with other gui elements
     * @param mediator the KeYMediator
     */
    public void setMediator(KeYMediator mediator) {	
	this.mediator = mediator;
    }

    // setter
    private void setUseSimultaneousUpdates(boolean b) {
	local_use_sim = b;
    }

    private void setImplicitExecution(boolean b) {
	local_implicit_execution = b;	
	mediator.setUpdateSimplificationMode(b);
    }

    private void setDeleteSimpleUpdates(boolean b) {
	local_del_simple = b;
    }

    private void setDeleteUnusedUpdates(boolean b) {
	local_del_unused = b;
    }

    private void pushSettings() {
	settingsFromConfigurationFile.setUseSimultaneousUpdates(local_use_sim);
	settingsFromConfigurationFile.setDeleteSimpleUpdates(local_del_simple);
	settingsFromConfigurationFile.setDeleteUnusedUpdates(local_del_unused);
	settingsFromConfigurationFile.setImplicitExecution(local_implicit_execution);
	mediator.setSimplifier(settingsFromConfigurationFile.getSimplifier());
    }

    // getter
    public boolean useSimultaneousUpdates() {
	return settingsFromConfigurationFile.useSimultaneousUpdates();
    }

    public boolean implicitExecution() {
	return settingsFromConfigurationFile.implicitExecution();
    }

    public boolean deleteSimpleUpdates() {
	return settingsFromConfigurationFile.deleteSimpleUpdates();
    }

    public boolean deleteUnusedUpdates() {
	return settingsFromConfigurationFile.deleteUnusedUpdates();
    }


    /** lays out the configuration dialog */
    public void layoutSimutanUpdatesimplifier() {	
	JPanel checkBoxPanel = new JPanel(new GridLayout(4,1));
	checkBoxPanel.setBorder(new TitledBorder("Current Configuration"));

	
	JCheckBox use_sim_updates_cb = new JCheckBox("Use simultaneous updates",
						     useSimultaneousUpdates()); 	
	use_sim_updates_cb.addItemListener( new ItemListener() {
		public void itemStateChanged(ItemEvent ce) {
		    setUseSimultaneousUpdates(ce.getStateChange() == ItemEvent.SELECTED);
		}
	    });
	JCheckBox delete_simple_cb = new JCheckBox("Delete simple updates",
						   deleteSimpleUpdates()); 
	delete_simple_cb.addItemListener( new ItemListener() {
		public void itemStateChanged(ItemEvent ce) {
		    setDeleteSimpleUpdates(ce.getStateChange() == ItemEvent.SELECTED);
		}
	    });
	
	JCheckBox delete_unused_cb = new JCheckBox("Delete unused updates",
						   deleteUnusedUpdates());		
	delete_unused_cb.addItemListener( new ItemListener() {
		public void itemStateChanged(ItemEvent ce) {
		    setDeleteUnusedUpdates(ce.getStateChange() == ItemEvent.SELECTED);
		}
	    });

	JCheckBox implicit_execution_cb = new JCheckBox("Implicit Execution",
							implicitExecution()); 
	implicit_execution_cb.addItemListener( new ItemListener() {
		public void itemStateChanged(ItemEvent ce) {
		    setImplicitExecution(ce.getStateChange() == ItemEvent.SELECTED);
		}
	    });

	checkBoxPanel.add(use_sim_updates_cb);
	checkBoxPanel.add(delete_simple_cb);
	checkBoxPanel.add(delete_unused_cb);
	checkBoxPanel.add(implicit_execution_cb);

	JPanel buttonPanel = new JPanel(new GridLayout(1,2));
	JButton ok = new JButton("OK");
	ok.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    pushSettings();
		    dispose();
		}
	    });
	JButton cancel = new JButton("Cancel");
	cancel.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    dispose();
		}
	    });
	buttonPanel.add(ok);
	buttonPanel.add(cancel);

	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(checkBoxPanel, BorderLayout.CENTER);
	getContentPane().add(buttonPanel, BorderLayout.SOUTH);	
	
    }

    

}
