// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.gui;

import org.apache.log4j.Logger;

import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.proof.proofevent.RuleAppInfo;
import de.uka.ilkd.key.rule.*;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.SchemaVariable;

/**
 * Listen for rule applications, add abbreviations of skolem functions
 */
public class SkolemAbbreviator implements InteractiveProofListener {

    private KeYMediator mediator;

    public SkolemAbbreviator ( KeYMediator p_mediator ) {
	mediator = p_mediator;

	mediator.addInteractiveProofListener ( this );
    }

    /** invoked if automatic execution has started
     */
    public void autoModeStarted(ProofEvent e) {}

    /** invoked if automatic execution has stopped
     */
    public void autoModeStopped(ProofEvent e) {}
    
    /** invoked when a rule has been applied */
    public void ruleApplied(ProofEvent ev) {
	RuleAppInfo rai = ev.getRuleAppInfo ();

	if ( rai == null )
	    return;

	RuleApp ruleApp = rai.getRuleApp ();
	if ( ruleApp != null && ruleApp instanceof TacletApp ) {
	    TacletApp                app = (TacletApp)ruleApp;
	    IteratorOfNewDependingOn it  = app.taclet ().varsNewDependingOn ();
	    SchemaVariable           sv;
	    Term                     t;

	    while ( it.hasNext () ) {
		sv = it.next ().first ();
		t  = (Term)app.instantiations ().getInstantiation ( sv );

		Debug.assertTrue ( t != null,
				   "Instantiation missing, but should be there" );

		if ( t.op ().arity () > 0 ) addAbbreviation ( t );
	    }
	}
    }

    private void addAbbreviation (Term p_t) {
        try {            
            final String abbrev = p_t.op ().name ().toString ();
            mediator.getNotationInfo ().getAbbrevMap ().put ( p_t, abbrev, true );
        } catch ( AbbrevException e ) {
            final Logger logger = Logger.getLogger ( SkolemAbbreviator.class.getName () );
            logger.warn ( "Error occurred when trying to add "
                          + "abbreviation of skolem symbol:\n" + e );
        }
    }

}
