// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.util.Debug;

/** 
 * this dialog appears if a rule is selected to be applied and the rule has an
 * if sequent. The dialog offers the possibility to select the wanted match of
 * the if sequent or to enter the if-sequent instantiation manually
 */

public class TacletIfSelectionDialog extends JPanel{

    private JPanel ifPanel;
    private ApplyTacletDialogModel model;
    

    /** creates a new dialog
     * @param owner the JFrame that is the owner(invoker) of this dialog
     * @param ifModel the ifModel to be used
     * @param mediator the KeYMediator
     */
    public TacletIfSelectionDialog(ApplyTacletDialogModel model) { 

	this.model = model;
	layoutDialog();
	show();
    }
   
    private void layoutDialog() {
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	ifPanel = createIfPanel();
	add(ifPanel);	
    }

    /** creates the panel used to select the wanted if instantiation */
    private JPanel createIfPanel() {		      
	JPanel panel=new JPanel(); 
	panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
	panel.setBorder(new TitledBorder("If-sequent"));

	// If the if-sequent matches to diffferent parts of the sequent
	// the user is given the possibility to choose the right one
	// or to enter an instantiation manually in which case a cut
	// is performed if the manual entry does not match any other
	// part in the sequent. The list of possible the
	// if-instantiations is shown in a combo box model that is
	// created here.
	for (int i=0; i < model.ifChoiceModelCount(); i++) {
	    JPanel p = new JPanel();
	    p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
	    p.add(new JLabel(model.ifFma(i).toString()+"  "));
	    JComboBox ifChoice = new JComboBox(model.ifChoiceModel(i));
	    //	    ifChoice.setMinimumSize ( new Dimension ( 400, 10 ) );
	    IfComboRenderer rend = new IfComboRenderer();
	    ifChoice.setRenderer(rend);	    
	    ifChoice.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			JComboBox cb = (JComboBox)e.getSource();
			updateInputField(cb, (JTextField)cb.getParent()
					 .getComponent(2));
		    }
		});	    
	    p.add(ifChoice);
	    JTextField input = new JTextField(50);
	    p.add(input);
	    
	    updateInputField(ifChoice, input);
	    panel.add(p);
	}
	return panel;
    }

    /** the if selecton dialog is attached to exact one model */ 
    protected int current() {
	return 0;
    }

    /** 
     *  requests focus for the the field-th input field and sets the cursor at the
     * given position 
     */
    public void requestFocusAt(int field, int col) {
	ifPanel.setVisible(true);		   
	ifPanel.requestFocus();

	JTextField tf = (JTextField) 
	    ((JPanel)ifPanel.getComponent(field*2)).getComponent(2);
	Debug.outIfEqual("tacletifselectiondialog:: none existing field"
			 + " requested", null, tf);
	if ( tf != null && col >=0 && col < tf.getColumns() ) {
	    try {
		tf.setCaretPosition( col-1 );
	    } catch (IllegalArgumentException iae) {
		Debug.out("tacletifselectiondialog:: something is wrong with " +
			  "the caret position calculation.");		
	    } finally {
		tf.requestFocus();
		tf.validate();
	    }
	}
	ifPanel.validate();
    } 


    private void updateInputField(JComboBox ifChoice, JTextField inp) {
	IfChoiceModel icm = (IfChoiceModel)ifChoice.getModel();
	icm.setInput(inp.getText());
	if (ifChoice.getSelectedItem() != icm.manualText()) {
	    inp.setEnabled(false);
	} else {
	    inp.setEnabled(true);	    
	}
    }

    /** transfers input to the model */
    protected void pushAllInputToModel() {
	for (int i = 0; i < ifPanel.getComponentCount(); i++) {
	    JPanel pan = (JPanel) ifPanel.getComponent(i);	   
	    if (((JTextField)pan.getComponent(2)).getText() != null) {
		model.setManualInput(i, ((JTextField)pan.getComponent(2))
				     .getText());	    
	    } else {
		model.setManualInput(i, "");	    
	    }
	}
    }
    

    static class IfComboRenderer extends JLabel implements ListCellRenderer {
	public IfComboRenderer() {
	    setOpaque(true);
	}
	
	public Component getListCellRendererComponent(JList list,
						      Object value,
						      int index,
						      boolean isSelected,
						      boolean cellHasFocus)
	{	    
	    if (isSelected) {
		list.setToolTipText(value.toString());
	    }
	    int len = value.toString().length();
	    int max_len= len>40 ? 40 : len;
	    setText(value.toString().substring(0,max_len)+"...");
	    this.setBackground(isSelected ? Color.blue : Color.lightGray);
	    this.setForeground(isSelected ? Color.white : Color.black);
	    return this;
	}
    }

}


