package de.uka.ilkd.key.java;

import de.uka.ilkd.key.logic.ProgramElementName;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.recoderext.KeYCrossReferenceServiceConfiguration;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.declaration.modifier.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.reference.*;


/**
 * The KeYASTFactory helps building KeY Java AST structures.
 */
public abstract class KeYJavaASTFactory {

    /** 
     * creates an assignment <code> lhs:=rhs </code>
     */
    public static Statement assign(Expression lhs, Expression rhs) {
	return new CopyAssignment(lhs, rhs);
    }

    /**
     * creates an attribute access <code>prefix.field </code>
     */
    public static Expression attribute(ReferencePrefix prefix, 
				       ProgramVariable field) {
	return new FieldReference(field, prefix);
    }

    /**
     * creates a local variable declaration <code> typeRef name; </code>     
     */
    public static LocalVariableDeclaration declare
	(String name, TypeReference typeRef) {
	return new LocalVariableDeclaration
	    (typeRef, new VariableSpecification
	     (new ProgramVariable(new ProgramElementName(name), 
				  typeRef.getKeYJavaType())));
    }

    /**
     * create a local variable declaration <br></br>
     *   <code>type name = init; </code>
     */
    public static LocalVariableDeclaration declare
	(String name, Expression init, KeYJavaType type) {
	return new LocalVariableDeclaration
	    (new TypeRef(type), 
	     new VariableSpecification
		 (new ProgramVariable(new ProgramElementName(name), type), 
		  init, type));
    }


    /**
     * create a local variable declaration
     */
    public static LocalVariableDeclaration declare(String name, KeYJavaType type) {
	return new LocalVariableDeclaration
	    (new TypeRef(type), 
	     new VariableSpecification
		 (new ProgramVariable(new ProgramElementName(name), type)));
    }


}
