// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
package de.uka.ilkd.key.java.recoderext;

import de.uka.ilkd.key.util.Debug;
import recoder.*;
import recoder.abstraction.*;
import recoder.java.*;
import recoder.java.declaration.*;
import recoder.java.declaration.modifier.*;
import recoder.java.expression.operator.*;
import recoder.java.expression.literal.*;
import recoder.java.reference.*;
import recoder.java.statement.*;
import recoder.kit.TwoPassTransformation;
import recoder.list.ClassTypeList;
import recoder.list.FieldList;

import recoder.list.CompilationUnitMutableList;
import recoder.list.FieldSpecificationList;
import recoder.list.MemberDeclarationMutableList;
import recoder.list.ModifierMutableList;
import recoder.list.ModifierArrayList;
import recoder.list.ParameterDeclarationArrayList;
import recoder.list.StatementArrayList;
import recoder.list.StatementMutableList;
import java.util.*;

/**
 * If an allocation expression <code>new Class(...)</code> occurs, a new object
 * has to be created, in KeY this is quite similar to take it out of a list of
 * objects and setting the implicit flag <code> &lt;created&gt; </code> to
 * <code>true</code> as well as setting all fields of the object to their
 * default values. For the complete procedure, the method creates the 
 * implicit method <code>&lt;createObject$gt;</code> which on its part calls
 * another implicit method <code>lt;prepare&gt;</code> for setting the fields
 * default values.
 */
public class CreateObjectBuilder extends RecoderModelTransformer {

    public static final String 
	IMPLICIT_OBJECT_CREATE = "<createObject>";
    public static final String NEW_OBJECT_VAR_NAME = "__NEW__";


    public CreateObjectBuilder
	(CrossReferenceServiceConfiguration services, 
	 CompilationUnitMutableList units) {	
	super(services, units);
    }


    /** 
     * Creates the body of the static <code>&lt;createObject&gt;</code>
     * method.
     */
    private StatementBlock createBody(ClassDeclaration recoderClass) {
		
	StatementMutableList result = new StatementArrayList(10);
	LocalVariableDeclaration local = declare(NEW_OBJECT_VAR_NAME, recoderClass);
	

	result.add(local);
	result.add
	    (assign(new VariableReference
		    (new Identifier(NEW_OBJECT_VAR_NAME)), 
		    attribute(null, 
			      new ImplicitIdentifier
			      (ImplicitFieldAdder.IMPLICIT_NEXT_TO_CREATE))));


	result.add
	    (new MethodReference(new VariableReference
				 (new Identifier(NEW_OBJECT_VAR_NAME)), 
				 new ImplicitIdentifier
				 (CreateBuilder.IMPLICIT_CREATE)));

	result.add(new Return
		 (new VariableReference(new Identifier(NEW_OBJECT_VAR_NAME))));

	return new StatementBlock(result);
	
    }
    

    /**
     * creates the implicit static <code>&lt;createObject&gt;</code>
     * method that takes the object to be created out of the pool
     * @param type the TypeDeclaration for which the
     * <code>&lt;prepare&gt;</code> is created
     * @return the implicit <code>&lt;prepare&gt;</code> method
     */
    public MethodDeclaration createMethod(ClassDeclaration type) {
	ModifierMutableList modifiers = new ModifierArrayList(2);
	modifiers.add(new Public());
	modifiers.add(new Static());	
	MethodDeclaration md =  new MethodDeclaration
	    (modifiers, 
	     new TypeReference
	     ((Identifier)type.getIdentifier().deepClone()), 
	     new ImplicitIdentifier(IMPLICIT_OBJECT_CREATE), 
	     new ParameterDeclarationArrayList(0), 
	     null,
	     createBody(type));
	md.makeAllParentRolesValid();
	return md;
    }    


    /**
     * entry method for the constructor normalform builder
     * @param td the TypeDeclaration 
     */
    protected void makeExplicit(TypeDeclaration td) {
	if (td instanceof ClassDeclaration) {
	    attach(createMethod((ClassDeclaration)td), 
		   td, td.getMembers().size());
//  	    java.io.StringWriter sw = new java.io.StringWriter();
//  	    services.getProgramFactory().getPrettyPrinter(sw).visitClassDeclaration((ClassDeclaration)td);
//  	    System.out.println(sw.toString());
//  	    try { sw.close(); } catch (Exception e) {}		
	}
    }






}
