// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.java.recoderext;

import recoder.*;
import recoder.abstraction.*;
import recoder.abstraction.Package;
import recoder.convenience.*;
import recoder.io.*;
import recoder.java.*;
import recoder.java.statement.*;
import recoder.java.declaration.*;
import recoder.java.expression.*;
import recoder.java.expression.operator.*;
import recoder.java.reference.*;
import recoder.kit.*;
import recoder.list.*;
import recoder.util.*;
import recoder.service.*;

import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.java.KeYProgModelInfo;

public class KeYCrossReferenceServiceConfiguration 
    extends CrossReferenceServiceConfiguration{

    protected KeYProgModelInfo kpmi = null;

    public KeYCrossReferenceServiceConfiguration() {
	super(); // initialises servConf
	getProjectSettings().ensureSystemClassesAreInPath();
    }

    public KeYCrossReferenceServiceConfiguration(KeYProgModelInfo kpmi) {
	this();
	this.kpmi = kpmi;
    }

    protected void makeServices() { 
	super.makeServices();
    }

    protected void initServices() { 
	super.initServices();
	
    }

    protected ProgramFactory makeProgramFactory() {
	return ProofJavaProgramFactory.getInstance();
    }

    protected ChangeHistory makeChangeHistory() {
	return new ChangeHistory(this);
    }

    /**
       The cross reference source info is a subclass of the source info,
       so this class simply overrides the source info factory method.
     */
    protected SourceInfo makeSourceInfo() {
 	return new KeYCrossReferenceSourceInfo(this);
    }  

    protected ProjectSettings makeProjectSettings() {
	return new ProjectSettings(this);
    }

    protected SourceFileRepository makeSourceFileRepository() {
	return new DefaultSourceFileRepository(this);
    }
    
    protected ClassFileRepository makeClassFileRepository() {
	return new DefaultClassFileRepository(this);
    }

    protected ByteCodeInfo makeByteCodeInfo() {
	return new DefaultByteCodeInfo(this);
    }

    protected ImplicitElementInfo makeImplicitElementInfo() {
	return new DefaultImplicitElementInfo(this);
    }

    protected NameInfo makeNameInfo() {
	return new DefaultNameInfo(this);
    }

    protected ConstantEvaluator makeConstantEvaluator() {
	return new DefaultConstantEvaluator(this);
    }

    public KeYProgModelInfo getKeYProgModelInfo(){
	return kpmi;
    }
}
