// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.java.recoderext;

import recoder.service.*;
import recoder.ServiceConfiguration;
import recoder.abstraction.*;
import recoder.java.*;
import recoder.java.reference.*;
import recoder.java.declaration.VariableSpecification;

public class SchemaCrossReferenceSourceInfo extends KeYCrossReferenceSourceInfo {
    
    protected final PrimitiveType recoderTypeSVType;

    public SchemaCrossReferenceSourceInfo(ServiceConfiguration config) {
	super(config);
	recoderTypeSVType 
	    = new PrimitiveType("TypeSV", this);
    }    

    public void initialize(ServiceConfiguration cfg) {
	super.initialize(cfg);
	cfg.getChangeHistory().removeChangeHistoryListener(this);
    }

    public Type getType(TypeReference tr) {
	if (tr instanceof TypeSVWrapper) {
	    return recoderTypeSVType;
	} else {
	    return super.getType(tr);
	}
    }

    public Type getType(VariableSpecification vs) {
	if (vs.getExpressionCount()>0 
	    && vs.getExpressionAt(0) instanceof ProgramVariableSVWrapper) {
	    return recoderTypeSVType;
	} else {
	    return super.getType(vs);
	}
    }

    /** does not resolve the urq, just returns the argument
     */
    public Reference resolveURQ(UncollatedReferenceQualifier urq){
	return urq;
    }


}

