// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.java.statement;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.visitor.Visitor;

/**
 *  Throw.
 */

public class Throw extends ExpressionJumpStatement {

    /**
 *      Throw.
     */

    public Throw() {}

    /**
 *      Throw.
 *      @param expr an expression.
     */

    public Throw(Expression expr) {
        super(expr);
        if (expr == null) {
            throw new IllegalArgumentException("Throw requires one argument");
        }
    }

    /**
 *      Throw.
 *      @param ExtList .with all children
     */

    public Throw(ExtList children) {
        super(children);
    }

    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnThrow(this);
    }

    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printThrow(this);
    }
}
