// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.java.visitor;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.rule.metaconstruct.UnwindLoop;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

/** 
 * This visitor is used to collect all appearing SchemaVariables in a 
 * java program
 */
public class ProgramSVCollector extends JavaASTWalker {

    private ListOfSchemaVariable result = 
	SLListOfSchemaVariable.EMPTY_LIST; 

    /** the instantiations needed for unwind loop constructs */
    private SVInstantiations instantiations = 
	SVInstantiations.EMPTY_SVINSTANTIATIONS;

    /** create the ProgramSVCollector
     * @param root the ProgramElement where to begin
     * @param vars the ListOfSchemaVariable where to add the new found
     * ones
     */
    public ProgramSVCollector(ProgramElement root, 
			      ListOfSchemaVariable vars) {
	super(root);
	result = vars;
    }

    /** create the ProgramSVCollector
     * @param root the ProgramElement where to begin
     * @param vars the ListOfSchemaVariable where to add the new found
     * ones
     * @param svInst the SVInstantiations previously found in order to determine
     * the needed labels for the UnwindLoop construct. 
     */
    public ProgramSVCollector(ProgramElement root, 
			      ListOfSchemaVariable vars,
			      SVInstantiations svInst) {
	super(root);
	result = vars;
	instantiations = svInst;
    }

    /** starts the walker*/
    public void start() {
	walk(root());
    }

    public ListOfSchemaVariable getSchemaVariables() {
	return result;
    }

    /** the action that is performed just before leaving the node the
     * last time 
     */
    protected void doAction(ProgramElement node) {
	if (node instanceof SchemaVariable) {
	    result = result.prepend((SchemaVariable)node);
	} else if (node instanceof UnwindLoop) {	    
	    result = 
		result.prepend(((UnwindLoop)node).
			       neededInstantiations(((UnwindLoop)node).body(),	       
			       instantiations));
	}
    }
  
}
