// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.java.visitor;

import de.uka.ilkd.key.java.SourceElement;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Expression;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.rule.TacletForTests;
import java.util.Iterator;
import java.util.HashSet;
import de.uka.ilkd.key.java.ProgramElement;

import de.uka.ilkd.key.java.Services;

/** 
 * Walks through a java AST in depth-left-fist-order. 
 * This walker is used to collect all Types in a program.
 */
public class ProgramTypeCollector extends JavaASTVisitor {

    private HashSet result = new HashSet();
    private JavaInfo javaInfo;

    public ProgramTypeCollector(ProgramElement root, JavaInfo javainfo) {
	super(root);
        javaInfo = javainfo;
    }

    /** the action that is performed just before leaving the node the
     * last time 
     */
    protected void doAction(ProgramElement node) {
	node.visit(this);
    }

    /** starts the walker*/
    public void start() {	
	walk(root());	
    }

    public HashSet result() { 
	return result;
    }    

    public String toString() {
	return result.toString();
    }

    protected void doDefaultAction(SourceElement x) {
	//System.out.println("HERE:" +x+ "   "+ x.getClass());
        if (x instanceof Expression) {
	    //  System.out.println("HERE 1"+x);
	    if (javaInfo.getPrimitiveKeYJavaType( (Expression)x ) != null) {
		//System.out.println("HERE 2"+x);		
		result.add(javaInfo.getPrimitiveKeYJavaType( (Expression)x ));
	    }
	}
    }


}
