// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.java.visitor;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.rule.TacletForTests;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.ProgramVariable;

import junit.framework.*;
import java.util.HashSet;
import java.util.Iterator;

public class TestDeclarationProgramVariableCollector  extends TestCase {

    // some non sense java blocks with lots of statements and expressions
    private static String[] jblocks=new String[]{
	"{ int j1 = 0; int j2, j3, j4 = 0;}",
	"{ int j1; { int j2; } { int j3; } for (int j4; j4=0; j4++) {} int j5; }",
	"{ int j0; { { { { {  int j1; } int j2; } int j3;} int j4; } } }"
    };

    // names of variables expected to be collected in jblocks
    private static String[][] expectedVars = new String[][]{
	{"j1", "j2", "j3", "j4"},
	{"j1", "j5"},
	{"j0"}
    };


    static JavaBlock[] test_block = new JavaBlock[jblocks.length];
    private Recoder2KeY r2k;

    public TestDeclarationProgramVariableCollector(String name) {
	super(name);
	r2k=new Recoder2KeY(TacletForTests.services(), new NamespaceSet());
    }


    public void setUp() {
	for (int i = 0; i < jblocks.length; i++) {
	    test_block[i] = r2k.readBlockWithEmptyContext(jblocks[i]);
	}

    }
    
    private HashSet toNames(HashSet programVariables) {
	HashSet result = new HashSet();
	Iterator it = programVariables.iterator(); 
	while (it.hasNext()) {
	    String name = ""+((Named)it.next()).name();
	    if (result.contains(name)) {
		System.out.println("Warning: Program variables have same name."+
				   " Probably unsane test case");
	    }
	    result.add(name);
	}
	return result;
    }
    

    public void testVisitor() {
	DeclarationProgramVariableCollector dpvc;
	for (int i = 0; i < jblocks.length; i++) {
	    dpvc = new DeclarationProgramVariableCollector(test_block[i].program());
	    dpvc.start();
	    HashSet names = toNames(dpvc.result());


	    assertTrue("Too many variables collected. Collected:" + 
		       dpvc.result() + " in " + jblocks[i], 
		       dpvc.result().size() <= expectedVars[i].length);


	    for (int j = 0; j < expectedVars[i].length; j++) {
		assertTrue("Missing variable: "+expectedVars[i][j] + " of " + jblocks[i], 
			   names.contains(expectedVars[i][j]));
	    }	    
	}
    }

}
