// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.logic.op.Operator;
import de.uka.ilkd.key.logic.op.QuantifiableVariable;
import de.uka.ilkd.key.logic.op.ArrayOfQuantifiableVariable;
import de.uka.ilkd.key.logic.sort.Sort;

/**
 * A BinaryBindingTerm is an object that contains an Operator, two subterms, 
 * and one variable
 * Instances should never be accessed via 
 * this interface, use the interface of the superclass Term and construct 
 * instances only via
 * a TermFactory instead.
 */
class BinaryBindingTerm extends Term {

    /** iteration variable
     * @link aggregation
     * @supplierCardinality * 
     */
    private final ArrayOfQuantifiableVariable substVar;
    
    /** 
     * The two sub terms of this term, where subs[0] is a collection,
     * subs[1] is the expression that is evaluated for all elements
     * in the collection, and the expr. in which the variable is bound.
     */
    private final ArrayOfTerm subs;

    /** depth of the term */
    private final int depth;
  

    /** creates a term with two subterms
     * @param substVar the iteration variable (bound in the 2nd term)
     * @param subs the subterms
     */
    public BinaryBindingTerm(Operator op, 
			     QuantifiableVariable substVar, 
			     Term[] subs) {
	super(op, op.sort(subs));
	
	this.subs = new ArrayOfTerm(subs);
	this.substVar  = new ArrayOfQuantifiableVariable
	    (new QuantifiableVariable[] {substVar});
	this.depth = 1+(subs[0].depth() > subs[1].depth() ? 
			subs[0].depth() : subs[1].depth());

	fillCaches();
    }

    /** @return n-th subterm */    
    public Term sub(int n) {
	return subs.getTerm(n);
    }	

    /**@return depth of the term */
    public int depth() {
	return depth;
    }
   
    /** @return arity of term */
    public int arity() {
	return 2;
    } 

    /** @return the bound variables of the n-th subterm */
    public ArrayOfQuantifiableVariable varsBoundHere(int n) {
       return (n==1 ? substVar : EMPTY_VAR_LIST);
    }

}
