// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.logic.op.*;
/**
 * This class is used to collect all appearing variables that can
 * represent logic variables (LogicVariablesRepresentatives) in a
 * term. Duplicates are not removed becaues the use of persistent
 * datastructure and up to now we just have a SetAsList-implementaion
 * causing to have O(sqr(n)) if it would used.
 */

public class LVRCollector extends Visitor{
    /** collects all found variables */
    private ListOfLogicVariableRepresentative varList;

    /** creates the Variable collector */
    public LVRCollector() {
	varList = SLListOfLogicVariableRepresentative.EMPTY_LIST;
    }

    /** is called by the execPostOrder-method of a term 
     * @param Term to checked if it is a Variable and if true the
     * Variable is added to the list of found Variables
     */  
    public void visit(Term t) {
	if (t.op() instanceof LogicVariableRepresentative) {
	    varList=varList.prepend((LogicVariableRepresentative)t.op());
	} else if (t.op() instanceof Quantifier) {
	    for (int i=0; i<t.varsBoundHere(0).size();i++) {
		if (t.op() instanceof LogicVariableRepresentative) {
		    varList=varList.prepend
			(t.varsBoundHere(0).getQuantifiableVariable(i));
		}
	    }
	}
    }

        
    /** @return iterator of the found Variables */
    public IteratorOfLogicVariableRepresentative varIterator() {
	return varList.iterator();
    }

    /** @return true iff term contains the given variable */
    public boolean contains(LogicVariableRepresentative var) {
	return varList.contains(var);
    }

}
