// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic;

public class NamespaceSet {

    private Namespace varNS=new Namespace();
    private Namespace progVarNS=new Namespace();
    private Namespace funcNS=new Namespace();
    private Namespace heurNS=new Namespace();
    private Namespace sortNS=new Namespace();
    private Namespace choiceNS=new Namespace();
    
    public NamespaceSet() {
    }

    public NamespaceSet(Namespace varNS, Namespace funcNS, 
                        Namespace sortNS, Namespace heurNS,
			Namespace choiceNS, Namespace programVarNS) {
	this.varNS=varNS;
	this.progVarNS = programVarNS;
	this.funcNS=funcNS;
	this.sortNS=sortNS;
	this.heurNS=heurNS;
	this.choiceNS=choiceNS;
    }

    public Namespace variables() {
	return varNS;
    }

    public void setVariables(Namespace varNS) {
	this.varNS=varNS;
    }

    public Namespace programVariables() {
	return progVarNS;
    }

    public void setProgramVariables(Namespace progVarNS) {
	this.progVarNS=progVarNS;
    }

    public Namespace functions() {
	return funcNS;
    }

    public void setFunctions(Namespace funcNS) {
	this.funcNS=funcNS;
    }


    public Namespace heuristics() {
	return heurNS;
    }

    public void setHeuristics(Namespace heurNS) {
	this.heurNS=heurNS;
    }


    public Namespace sorts() {
	return sortNS;
    }

    public void setSorts(Namespace sortNS) {
	this.sortNS=sortNS;
    }

    public Namespace choices() {
	return choiceNS;
    }

    public void setChoices(Namespace choiceNS) {
	this.choiceNS=choiceNS;
    }

    public void add(NamespaceSet ns) {
	variables().add(ns.variables());
	programVariables().add(ns.programVariables());
	sorts().add(ns.sorts());
	heuristics().add(ns.heuristics());
	functions().add(ns.functions());
	choices().add(ns.choices());
    }

    public NamespaceSet copy() {
	NamespaceSet c = new NamespaceSet();
	c.setSorts(sorts().copy());
	c.setHeuristics(heuristics().copy());
	c.setFunctions(functions().copy());
	c.setVariables(variables().copy());
	c.setProgramVariables(programVariables().copy());
	c.setChoices(choices().copy());
	return c;
    }

    public String toString() {
	return "Sorts: "+sorts()+"\n"+
	    "Functions: "+functions()+"\n"+
	    "Variables: "+variables()+"\n"+
	    "ProgramVariables: "+programVariables()+"\n"+
	    "Heuristics: "+heuristics()+"\n"+
	    "Taclet Options: "+choices()+"\n";
    }

}
