// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.sort.Sort;

import java.util.HashMap;

public class ConjCond extends IfElse {

    private int arity;

    private static HashMap operatorHash = new HashMap(10);

    public static ConjCond createConjCond(int arity) {
	Integer arityInt = new Integer(arity);
	if (!operatorHash.containsKey(arityInt)) {
	    operatorHash.put(arityInt, new ConjCond(arity));
	}
	return (ConjCond)operatorHash.get(arityInt);
    }
    
    
    private ConjCond(int arity) {
	super(new Name("cond-conj-operator"));
	this.arity = arity;
    }

    public boolean validTopLevel(Term term) {
	return true;
    }

    /**
     * @return the sort
     */
    public Sort sort(Term[] term) {
	return term[arity()-1].sort();
    }

    /** @return arity of the operator as int */
    public int arity() {
	return arity;
    }
}
