// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.logic.ProgramConstruct;
import de.uka.ilkd.key.logic.Name;

class ProgramSV extends SortedSchemaVariable {
    
    /** 
     * creates a new SchemaVariable used as a placeholder for program
     * constructs
     * @param name the Name of the SchemaVariable
     * @param listSV a boolean which is true iff the schemavariable is
     * allowed to match a list of program constructs
     */    
    ProgramSV(Name name, ProgramSVSort s, boolean listSV) {
	super(name, ProgramConstruct.class, s, listSV);	
    }
    
    /** returns true iff this SchemaVariable is used to match
     * part of a program
     * @return true iff this SchemaVariable is used to match
     * part of a program
     */
    public boolean isProgramSV() {
	return true;
    }
    
    /** @returns comments if the schemavariable stands for programm
     * construct and has comments attached to it (not supported yet)
     */
    public Comment[] getComments() {
	return new Comment[0];
    }
    
    public SourceElement getFirstElement(){
	return this;
    }
    
    public SourceElement getLastElement(){
	return this;
    }

    /**
     * Returns the start position of the primary token of this element.
     * To get the start position of the syntactical first token,
     * call the corresponding method of <CODE>getFirstElement()</CODE>.
     * @return the start position of the primary token.
     */
    public Position getStartPosition(){
	return Position.UNDEFINED;
    }
    
    /**
     * Returns the end position of the primary token of this element.
     * To get the end position of the syntactical first token,
     * call the corresponding method of <CODE>getLastElement()</CODE>.
     * @return the end position of the primary token.
     */
    public Position getEndPosition(){
	return Position.UNDEFINED;
    }

    /**
     * Returns the relative position (number of blank heading lines and 
     * columns) of the primary token of this element.
     * To get the relative position of the syntactical first token,
     * call the corresponding method of <CODE>getFirstElement()</CODE>.
     * @return the relative position of the primary token.
     */
    public Position getRelativePosition(){
	return  Position.UNDEFINED;
    }

    /** toString */
    public String toString() {
	return toString("program "+sort().name());
    }	
}

