// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/* Generated by Together */

package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.ClashFreeSubst;

public class SubstOp extends Op {
    
    SubstOp(Name name) {
	super(name);
    }

    /**
     * @return true iff 
     *    (the arity of term is 2, the sort of subterm 0
     *     has the same sort as or a subsort of the variable 
     *     to be substituted, and the number of 
     *     variables bound there is 0 for the 0th subterm 
     *     and 1 for the 1st subterm)
     *     or
     *     (the arity of term is 3, the sort of subterm 0
     *     and subterm 1 respectively
     *     has the same sorts as or subsorts of the variables 
     *     to be substituted, and the number of 
     *     variables bound there is 0 for the 0th and 1st  
     *     subterm and 2 for the 2nd subterm)
     */
    public boolean validTopLevel(Term term){
	if (term.arity() != 2 && term.arity() != 3) return false;
	if (term.arity() == 2) {
	    if (term.varsBoundHere(1).size() != 1) return false;
	    if (term.varsBoundHere(0).size() != 0) return false;
	    Sort substSort=term.sub(0).sort();
	    Sort varSort=term.varsBoundHere(1).getQuantifiableVariable(0).sort();
	    return substSort.extendsTrans(varSort);
	}
	//For OCL simplification (createTernarySubstTerm(...))
	else {
	    if (term.varsBoundHere(2).size() != 2) return false;
	    if (term.varsBoundHere(1).size() != 0) return false;
	    if (term.varsBoundHere(0).size() != 0) return false; 
	    return true;
	}
	//
    }

    /**
     * @return sort of the second subterm (arity == 2),
     *         sort of the third subterm (arity == 3), or throws an
     * IllegalArgumentException if term has another arity
     */
    public Sort sort(Term[] term) {
	if (term.length==2)
	    return term[1].sort();
	else if (term.length==3)
	    return term[2].sort();
	else throw new IllegalArgumentException("Cannot determine sort of "+
						"invalid term (Wrong arity).");
    }

    /** @return arity of the Substitution operator as int */
    public int arity() {
	return 2;
    }

    /**
     * Apply this substitution operator to <code>term</code>, which
     * has this operator as top-level operator
     */
    public Term apply ( Term term ) {
	//The "normal" case
	if (term.varsBoundHere(1).size() != 0) {
	    QuantifiableVariable v=term.varsBoundHere(1).getQuantifiableVariable(0);
	    ClashFreeSubst cfSubst = new ClashFreeSubst(v, term.sub(0));
	    Term res = cfSubst.apply(term.sub(1));
	    return res;
	}
	//For OCL simplification
	else {
	    QuantifiableVariable v1=term.varsBoundHere(2).getQuantifiableVariable(0);
	    QuantifiableVariable v2=term.varsBoundHere(2).getQuantifiableVariable(1);
	    ClashFreeSubst cfSubst = new ClashFreeSubst(v1, term.sub(0));
	    Term res = cfSubst.apply(term.sub(2));
	    ClashFreeSubst cfSubst2 = new ClashFreeSubst(v2, term.sub(1));
	    res = cfSubst2.apply(res);
	    return res;
	}
    }
}
