// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
//
//

package de.uka.ilkd.key.logic.sort;

import de.uka.ilkd.key.logic.Namespace;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.op.SortDependingSymbol;
import de.uka.ilkd.key.logic.op.InstanceofSymbol;
import de.uka.ilkd.key.logic.op.ListOfSortDependingSymbol;
import de.uka.ilkd.key.logic.op.SLListOfSortDependingSymbol;
import de.uka.ilkd.key.logic.op.Function;

public class ClassInstanceSortImpl 
    extends de.uka.ilkd.key.logic.sort.AbstractNonCollectionSort 
    implements ObjectSort{


    SetOfSort ext;

    /** creates an ClassInstanceSort*/
    public ClassInstanceSortImpl(Name name, SetOfSort ext) {
	super(name);
	this.ext=ext;
    }

    /** creates an ClassInstanceSort*/
    public ClassInstanceSortImpl(Name name, de.uka.ilkd.key.logic.sort.Sort ext) {
	this(name, SetAsListOfSort.EMPTY_SET.add(ext));
    }


    /** creates an ClassInstanceSort*/
    public ClassInstanceSortImpl(Name name) {
	this(name, SetAsListOfSort.EMPTY_SET);
    }

   
    /**
     * @return the sorts of the predecessors of this sort
     */
    public SetOfSort extendsSorts() {
	return ext;
    }

    /**
     * Create the symbols defined by this sort, insert them into the
     * namespace "p_func_ns"
     * @param p_func_ns Namespace to which functions and predicates
     * should be added to
     */
    public void createSymbols ( Namespace     p_func_ns, Namespace sort_ns ) {
	super.createSymbols ( p_func_ns, sort_ns );
	if ( !symbolsCreated ) {
	    ListOfSortDependingSymbol list =
		SLListOfSortDependingSymbol.EMPTY_LIST;

	    Sort booleanSort=(Sort)sort_ns.lookup(new Name("boolean"));
	    if (booleanSort==null) {
		return; //create symbols later
	    }
	    list = list.prepend ( new InstanceofSymbol (this, booleanSort));
	    addSymbols ( list, p_func_ns );
	    symbolsCreated = true;
	}
    }

    private boolean symbolsCreated = false;

}
