// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
//
//


package de.uka.ilkd.key.ocl;

import java.io.*;

import de.uka.ilkd.key.casetool.ReprModelClass;
import de.uka.ilkd.key.casetool.ReprModelMethod;

import java.util.Vector;
import javax.swing.*;


public class OCLExport extends JFrame {

    private File file;
    private ReprModelClass cls[];
    private FileWriter out;

    private final static String SEP = "------------------\n";

    public OCLExport(ReprModelClass cls, FileWriter out){
	this.cls = new ReprModelClass[1];
	this.cls[0] = cls;
	this.out = out;
    }

    public OCLExport(ReprModelClass[] cls, FileWriter out){
	this.cls = cls;
	this.out = out;
    }


    public void export() throws IOException{
	String inv;
	Vector ops;
	for (int j=0; j<cls.length; j++){
	    inv = cls[j].getMyInv();
	    ops = cls[j].getOps();
	    out.write("Class "+cls[j].getFullClassName()
		      +" has the following OCL-specification:\n\n");
	    if (!inv.equals("")) {
		out.write("INVARIANT:\n"+
			  "  "+inv+"\n");
	    }
	    out.write(SEP);
	    String[] prepost = new String[ops.size()];
	    String pre, post;
	    for (int i=0; i < ops.size(); i++){
		ReprModelMethod meth = (ReprModelMethod)(ops.elementAt(i));	   
		pre = meth.getMyPreCond().equals("") ? 
		    "none" : meth.getMyPreCond();
		post = meth.getMyPostCond().equals("") ? 
		    "none" : meth.getMyPostCond();
		if (!pre.equals("none") || !post.equals("none")){
		    out.write("Method "+meth.getName()+":\n\n");
		    out.write("  PRE-CONDITION:\n"+
			      "    "+pre+"\n\n");
		    out.write("  POST-CONDITION:\n"+
			      "    "+post+"\n");
		    if (i<ops.size()-1)
			out.write(SEP);
		}
	    }
	}
    }

}
