// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator;

import de.uka.ilkd.key.logic.*;
import tudresden.ocl.check.KeyTypeChecker;

/**
 * A class that is used in order to modifiy a string
 * that represents an OCL constraint to be translated
 * and that does nor conform to the parsers grammer 
 * (altough the constraint should be correct), so that
 * it can be parsed by the parser.
 * The modification will be resolved later on in the
 * tree representation of the parsed constraint.
 */
public class ConstraintModifier  {
    

    private int skipWS(String text, int startIdx) {
	for (int i = startIdx; i<text.length(); i++) {
	    if (!Character.isWhitespace(text.charAt(i))) {
		return i;
	    }
	}
	return text.length();
    }

    // this method has as precondition that a constraint never ends with
    // a situation similar to ...:TypeName
    /** returns the index where the type decl. starts or -1 */
    private int indexOfType(String constraintText, 
			    String typeName, 
			    int startIdx) {
	for (int i = startIdx; i<constraintText.length(); i++) {
	    if (constraintText.charAt(i) == ':') {
		int j = skipWS(constraintText, i+1);
		if (i + typeName.length() <= constraintText.length()) {
		    if (typeName.equals(constraintText.substring
					(i, i+typeName.length()))) {		    		    
			char nextChar = 
			    constraintText.charAt(i+typeName.length());
			if (Character.isWhitespace(nextChar) || 
			    nextChar == ',' || nextChar == ';' || 
			    nextChar == ')' || nextChar == '=') {
			    return j;
			}
		    }
		} else {
		    i = j;
		}
	    }
	}
	return -1;
    }
    

    /**
     * Creates a parsable version of the given constraint
     * text if possible.
     * 
     * Currently only constructor method names (starting with an
     * uppercase letter) will be prepended by some default 
     * prefix (starting with a lowercase letter).
     * This prefix will be removed later in by the KeyTypeChecker.
     */
    public String modifyConstraintText(String constraintText){
	String result;

	de.uka.ilkd.key.util.Debug.out("Modifying constraint:\n ", constraintText);

	// replace primitive integral Java type by Integer in the context of the
	// constraint, e.g. in the signature of a method, because the
	// parser wants the context in ocl-notation.
	String[] typesToReplace = new String[]{"byte", "short", "char", "int", "long"};
	for (int i=0; i<typesToReplace.length; i++) {
	    int index = indexOfType(constraintText, typesToReplace[i], 0);
	    while (index != -1){
		constraintText = 
		    constraintText.substring(0, index) +
		    "Integer" +
		    constraintText.substring(index + typesToReplace[i].length(),
					     constraintText.length());    
		index = indexOfType(constraintText, 
				    typesToReplace[i],
				    index + 8);
		// 8 is "Integer".length() +1
	    }
	}
	int invTokenIndex = constraintText.indexOf(" inv");
	int openParenIndex = constraintText.indexOf("(");
	if (invTokenIndex == -1 || (openParenIndex != -1 && openParenIndex < invTokenIndex)) {
	    // We have an operation constraint ...

	    // Check whether we have a constructor specification, that
	    // means that the name of the method and the name of
	    // the corresponding context class coincides ...

	    String startString =  constraintText.substring(0, openParenIndex);
	    String restString = constraintText.substring(openParenIndex);
	    int dcolonIndex =  startString.lastIndexOf("::");
	    String methodName = startString.substring(dcolonIndex+2).trim();
	    String prefixString = startString.substring(0, dcolonIndex);
	    int classNameStartIndex;
	    int i = prefixString.lastIndexOf("::");
	    if (i != -1){
		classNameStartIndex = i+2;
	    } else {
		classNameStartIndex = prefixString.lastIndexOf(" ")+1;
	    }
	    String className = prefixString.substring(classNameStartIndex).trim();
	    if (className.equals(methodName)){
		// Add a default prefix string (that starts with a lowercase letter) 
		// in front of the constructor name ...
		System.err.println("ADDING PREFIX for CONSTRUCTOR NAME ...");
		
		result = prefixString + "::" + KeyTypeChecker.KEY_PREFIX_LOWERCASE 
		    + methodName + restString;
	    } else {
		result = constraintText;
	    }
	} else {
	    result = constraintText;
	}

	de.uka.ilkd.key.util.Debug.out("Modified constraint:\n ", result);

	return result;
	
    }

}








