// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator;

/**
 * A class representing the translation of an OCL expression
 * into a logical term in the JavaCard-DL as used in the
 * KeY-project
 */

import de.uka.ilkd.key.logic.*;
import java.util.*;
import de.uka.ilkd.key.logic.op.LogicVariable;

public class KeyTranslationResult implements TranslationResult {
    
    public static final int NORMAL_TRANSLATION = 0;
    public static final int PREDICATIVE_TRANSLATION = 1; 
    
    /** The logical term representing the translation result */
    Term translationTerm;
    
    /** the type of the translation: predicative or normal */
    private int translationType;
    
    /** 
     * Some more logical formulas representing additional constraints
     * for the translation
     */
    
    Set translationAxioms;

    LogicVariable variableOfPredicate;


    /**
     * Creates an translation result with the given term t as 
     * the result of the translation and an empty set of axioms.
     */
    public KeyTranslationResult(Term t){
	translationTerm = t;
	translationAxioms = new HashSet();
	translationType =  NORMAL_TRANSLATION;
	variableOfPredicate = null;
    }

    /**
     * Creates an translation result with no term as 
     * the result of the translation and an empty set of axioms.
     */
    public KeyTranslationResult(){
	translationTerm = null;
	translationAxioms = new HashSet();
	translationType =  NORMAL_TRANSLATION;
	variableOfPredicate = null;
    }
    
    public Term getTerm(){
	return translationTerm;
    }
    
    public Set getAxioms(){
	return translationAxioms;
    }


    public void setTerm(Term t){
	translationTerm = t;
    }

    /**
     * add the given set of axioms (logical formulas)
     * to the axioms in this translation result.
     */
    public void addAxioms(Set axioms){
	Term currentAxiom;
	for (Iterator it=axioms.iterator(); it.hasNext(); ){
	    currentAxiom = (Term) it.next();
	    if (!translationAxioms.contains(currentAxiom)){
		// Omit duplicates ...
		translationAxioms.addAll(axioms);
	    }
	}

    }
    public void addAxiom(Term axiom){
	if (!translationAxioms.contains(axiom)){
	    translationAxioms.add(axiom);
	}
    }
    
    public void setTranslationType(int translationType){
	this.translationType = translationType;
    }

    public int getTranslationType(){
	return(this.translationType);
    }

    public boolean isPredicativeTranslation(){
	return(this.translationType == PREDICATIVE_TRANSLATION);
    }

    public LogicVariable getPredicateVariable(){
	return(variableOfPredicate);
    }
    
    public void setPredicateVariable(LogicVariable lv){
	variableOfPredicate = lv;;
    }

    public String toString(){
	String result = "Term : \n";
	result += getTerm().toString() + "\n";
	result += "\nAdditional Axioms:\n";
	Set axioms = getAxioms();
	for(Iterator it = axioms.iterator(); it.hasNext();){
	    Term ax = (Term) it.next();
	    result += "Axiom : " + ax.toString() + "\n";
	}
	
	if (isPredicativeTranslation()){
	    //  result += "Predicate variable: " + variableOfPredicate + "\n";
	}
	
	return result;
	
    }

    

}










