// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.control;

import tudresden.ocl.*;
import tudresden.ocl.parser.analysis.*;
import java.util.Set;
import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Vector;
import tudresden.ocl.parser.node.*;
import tudresden.ocl.check.types.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.heuristics.*;

/**
 * An optimized version of the most rudimenary translation control 
 * that can be used for
 * all logical target languages that are extensions of some 
 * first-order predicate logic.
 * 
 * An instance of this class automatically uses the following
 * translation heuristics:
 *  - Functions and Methods as non-rigid Functionsymbols - Heuristic
 *  - Boolean expressions as formulas Heuristic
 *  - Collections as Sets Heuristic
 *  - If-then-else Optimization Heuristic
 */

public class OptimizedGeneralTranslationControl extends GeneralTranslationControl {
  
    public OptimizedGeneralTranslationControl(){
	super();
	addHeuristic(new CollectionsAsSetsHeuristic());
	addHeuristic(new IfThenElseHeuristic());
    }
    

}

















