// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.control;

import java.util.*;
import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.heuristics.*;

/**
 * A class that manages all the different marks some node in the
 * tree representation of the constraint may be marked with.
 *
 * Especially there are functions for adding and removing marks,
 * as well as methods to get all the mark of the current node or
 * test whether the corresponding node has been marked with a given
 * mark.
 */

public class TranslationType {

 
    Set marks;
    
    /** Creates a description respresenting an emty mark set */
    public TranslationType(){
	this.marks = new HashSet();
    }

    /** Adds a new mark to the translation type if it isn't already present */
    public TranslationType addMark(TranslationMark tm){
	if (!marks.contains(tm)){
	    marks.add(tm);
	}
	return this;
    }

    /** Removes a mark form the set of marks of this translation type instance */
    public TranslationType removeMark(TranslationMark tm){
	if (marks.contains(tm)){
	    marks.remove(tm);
	}
	return this;
    }
    

    /** @return  the set of all marks represented by this translation type instance */
    public Set getMarks(){
	return marks;
    }

    
    /** @return true iff the the set of marks of this translation type instance contains
     *                   the given mark.
     */
    public boolean containsMark(TranslationMark tm){
	return (marks.contains(tm));
    }


    /** @return a string representation of the translation type */
    public String toString(){
	String s;
	
	if (marks.size() == 0){
	    s = "NO MARKS SPECIFIED!";
	} else {
	    s = "";
	    for (Iterator it = marks.iterator(); it.hasNext();  ){
		s += (it.next().toString() + "; ");
	    }
	}

	return s;
    }
    
}








