// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.detailedocl;

public class XMIFactory {

    private XMIFactory() {}

    private static int counter=0;
    private static final String OP="Foundation.Core.Operation";
    private static final String ATTRIB="Foundation.Core.Attribute";
    public static final String ME_NAME="Foundation.Core.ModelElement.name";
    private static final String ME_VISIBILITY="Foundation.Core.ModelElement.visibility";
    private static final String BFEAT_ISQUERY="Foundation.Core.BehavioralFeature.isQuery";
    private static final String FEAT_OWNERSCOPE="Foundation.Core.Feature.ownerScope";
    private static final String OP_SPEC="Foundation.Core.Operation.specification";
    private static final String OP_CONCURRENCY="Foundation.Core.Operation.concurrency";
    private static final String OP_ISPOLYMORPHIC="Foundation.Core.Operation.isPolymorphic";
    private static final String BFEAT_PARAM="Foundation.Core.BehavioralFeature.parameter";
    private static final String PARAM="Foundation.Core.Parameter";
    private static final String PARAM_KIND="Foundation.Core.Parameter.kind";
    private static final String PARAM_TYPE="Foundation.Core.Parameter.type";
    private static final String DATATYPE="Foundation.Core.DataType";
    private static final String SFEAT_MULTIPLICITY="Foundation.Core.StructuralFeature.multiplicity";
    private static final String SFEAT_CHANGEABLE="Foundation.Core.StructuralFeature.changeable";
    private static final String SFEAT_TARGETSCOPE="Foundation.Core.StructuralFeature.targetScope";
    private static final String SFEAT_TYPE="Foundation.Core.StructuralFeature.type";
    private static final String GEN_ISROOT="Foundation.Core.GeneralizableElement.isRoot";
    private static final String GEN_ISLEAF="Foundation.Core.GeneralizableElement.isLeaf";
    private static final String GEN_ISABSTRACT="Foundation.Core.GeneralizableElement.isAbstract";
    private static final String GEN_ISACTIVE="Foundation.Core.Class.isActive";
    public static final String CLASSIF_FEATURE="Foundation.Core.Classifier.feature";
    public static final String CLASS="Foundation.Core.Class";

    public static String startTag(String name, String attr) {
	return "<"+name+" "+attr+">";
    }

    public static String startTagId(String name, String id, String suffix) {
	return startTag(name, "xmi.id = '"+id+suffix+"'");
    }


    public static String endTag(String name, String attr) {
	return "</"+name+" "+attr+">";
    }

    public static String encloseInTags(String tagName, String between) {
	return startTag(tagName,"")+between+endTag(tagName,"");
    }

    public static String attrTag(String tagName, String attr) {
	return "<"+tagName+" "+attr+"/>";
    }

    public static String attrTagValue(String tagName, String attr) {
	return attrTag(tagName, "xmi.value = "+attr);
    }


    public static String attrTagRef(String tagName, String attr) {
	return attrTag(tagName, "xmi.idref = "+attr);
    }

    public static String attrTagId(String tagName, String id, String suffix) {
	return attrTag(tagName, "xmi.id = '"+id+suffix+"'");
    }

    public static String attrTagId(String tagName, String suffix) {
	return attrTagId(tagName, getId(), suffix);
    }


    public static String getId() {
	return "keyxmiid"+(counter++);
    }

    public static String getOp(String name, String returnType) {
	return getOpParam(name, returnType, null, null);
    }

    public static String getParam(String name, String id, String type, String kind) {
	return startTagId(PARAM, id, "")
	    + "\n" + encloseInTags(ME_NAME, name)
	    + "\n" + attrTagValue(ME_VISIBILITY, "'private'")
	    + "\n" + attrTagValue(PARAM_KIND, "'"+kind+"'")
	    + "\n" + encloseInTags(PARAM_TYPE, attrTagRef
				   (DATATYPE, "'"+ type+"'"))
	    + "\n" + endTag(PARAM, "");
    }

    public static String getOpParam(String name, String returntype, String param, String paramtype) {
	String opid=getId();
	String start= startTagId(OP, opid, ".O")
	    + "\n" + encloseInTags(ME_NAME, name)
	    + "\n" + attrTagValue(ME_VISIBILITY, "'public'")
	    + "\n" + attrTagValue(FEAT_OWNERSCOPE, "'instance'")
	    + "\n" + attrTagValue(BFEAT_ISQUERY, "'false'")
	    + "\n" + encloseInTags(OP_SPEC, "")
	    + "\n" + attrTagValue(OP_ISPOLYMORPHIC, "'false'")
	    + "\n" + attrTagValue(OP_CONCURRENCY, "'sequential'")
	    + "\n" + startTag(BFEAT_PARAM,"");
	String p = "";
	if (returntype!=null) {
	    p=p+"\n" + getParam(name+".Return",opid+".Return",returntype,"return");
	}
	if (param!=null) {
	    p=p+ "\n" + getParam(param,opid+".1",paramtype,"inout");
	}
	String end="\n" + endTag(BFEAT_PARAM,"")+ "\n" + endTag(OP,"");
	return start+p+end;
    }

    public static String getAssoc(String name, String aend1, String aend2, 
				  String id1, String id2) {
	return "";
    }


    public static String getAttribute(String name, String classId) {
	String attrid=getId();
	return startTagId(ATTRIB, attrid, ".a")
	    + "\n" + encloseInTags(ME_NAME, name)
	    + "\n" + attrTagValue(ME_VISIBILITY, "'private'")
	    + "\n" + attrTagValue(FEAT_OWNERSCOPE, "'classifier'")	 
	    + "\n" + encloseInTags(SFEAT_MULTIPLICITY, "1..1")
	    + "\n" + attrTagValue(SFEAT_CHANGEABLE, "'public'")
	    + "\n" + attrTagValue(SFEAT_TARGETSCOPE, "'classifier'")
	    + "\n" + encloseInTags(SFEAT_TYPE, attrTagRef(DATATYPE, classId))
	    + "\n" + endTag(ATTRIB,"");	  
    }

    public static String getClassTag(String name, String classId, String featureInsert) {
	return startTagId(CLASS, classId, "")
	    + "\n" + encloseInTags(ME_NAME, name)
	    + "\n" + attrTagValue(ME_VISIBILITY, "'public'")
	    + "\n" + attrTagValue(GEN_ISROOT, "'false'")	 
	    + "\n" + attrTagValue(GEN_ISLEAF, "'false'")
	    + "\n" + attrTagValue(GEN_ISABSTRACT, "'false'")
	    + "\n" + attrTagValue(GEN_ISACTIVE, "'false'")
	    + "\n" + encloseInTags(CLASSIF_FEATURE, featureInsert)
	    + "\n" +endTag(CLASS,"");
	
    }

    public static String getClassFeature(String featureInsert) {
	return encloseInTags(CLASSIF_FEATURE, featureInsert);
    }

}
