// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.heuristics;

/**
 * A base class for all classes implementing a heuristic for
 * a suitable control of the translation of an arbitrary OCL expression
 * within the KeY-System.
 *
 * It defines especially the TranslationTypes availables for the KeY-System.
 * and changes the behaviour of the getTranslationType() method in order to
 * get a DEFAULT type when the corresponding node was not marked at all.
 */
import tudresden.ocl.*;
import tudresden.ocl.parser.analysis.*;
import java.util.Set;
import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Vector;
import tudresden.ocl.parser.node.*;
import tudresden.ocl.check.types.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.control.*;

/**
 * An abstract base class for translation heuristics within the KeY-System.
 * Especially adds logging functionallity.
 */

public abstract class KeyTranslationHeuristic extends BasicTranslationHeuristic {

   
    boolean logEnabled = true; //false;

    // basic operations for the handling of translation type for nodes ...
    
    protected TranslationType getTranslationTypeForNode(Node n){
	
	if(n == null){ 
	    error("Trying to get a translation type for a 'null' node !");
	}
	
	if (!nodeTranslationTypeMap.containsKey(n)) {
	    return ((new TranslationType()).addMark(TranslationMark.DEFAULT));
	    
	}
	return (TranslationType) nodeTranslationTypeMap.get(n); 
    }

    
 
    protected TranslationType getTranslationTypeForVariable(String varName, String typeName){
	String varKey = varName + "$$$" + typeName;

	if (!varMap.containsKey(varKey)) {
	    return ((new TranslationType()).addMark(TranslationMark.DEFAULT));
	}
	
	return (TranslationType) varMap.get(varKey); 
    }
    

    
    public void setLoggingEnabled(boolean log){
	logEnabled = log;
    }


    protected void log(String text){
	if (logEnabled) {
	    super.log(text);
	}
    }

}

















