// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.normalize;

import tudresden.ocl.check.types.*;
import tudresden.ocl.parser.node.*;
import tudresden.ocl.normalize.*;
import tudresden.ocl.parser.*;
import tudresden.ocl.OclTree;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import java.util.List;

import de.uka.ilkd.key.ocl.ocldltranslator.*;


/** 
 * This normalizer resolves: 'implicit' usages of navigations over associations with
 * multiplicity <= 1 (results in a single or eventually no instance of a
 * UML model type) as sets will be made explicit, e.g
 *                    context Company in:
 *                      self.manager->size = 1
 *
 * will be normalized to ...
 *
 *                    context Company in:
 *                      Set{self.manager}->size = 1
 */
public class InstanceAsSetNormalizer implements NodeNormalizer {

  public void normalize(Node n, OclTree tree) {
    if (n instanceof APostfixExpression) {
      APostfixExpression pe = (APostfixExpression) n;
      
      //  System.err.println("))))))))))))))))))))))))))))))");
//        System.err.println("Current PostFixExpression == " + pe);
//        System.err.println("Current PostFixExpression's primary expression == " + pe.getPrimaryExpression() );
//        System.err.println("))))))))))))))))))))))))))))))");

      Iterator iter=pe.getPostfixExpressionTail().iterator();
      Type lastType=tree.getNodeType( pe.getPrimaryExpression() );
      boolean finished = false;
      while (!finished && iter.hasNext()) {
        APostfixExpressionTail pet=(APostfixExpressionTail) iter.next();
        PPostfixExpressionTailBegin petb = pet.getPostfixExpressionTailBegin();
        if ( petb instanceof AArrowPostfixExpressionTailBegin &&
             isUMLModelType(lastType) ) {
	    makeSetUsageExplicit(pe, pet, tree);
	    // Call this method again for the new PostfixExpression to handle 
	    // further occurences of implicit set usage ...
	    
	    normalize(pe,tree);
	   
	    // Do not iterate any more (since the postfix expression we iterate in/over has changed !!!!
	    finished = true;
        }
        lastType=tree.getNodeType(pet);
      }
    }
  }

  protected void makeSetUsageExplicit(APostfixExpression pe, APostfixExpressionTail pet, OclTree tree) {
    

    ALitColPrimaryExpression newPrimaryExpression;
    AExpression newElementExpression;
    APostfixExpression elementPostfixExpression;
    
    // System.err.println("old pe ==  " + pe.toString() );
    // System.err.println("old PrimaryExpressions ==  " + pe.getPrimaryExpression().toString() );
    // System.err.println("old PostfixExpressionTail ==  " +pe.getPostfixExpressionTail().toString() );
      

    
    // Generate the new Set literal and its expression ...
    
    // Generate the new elementPostfixExpression ...
    List newElementPostfixExpressionTail = new LinkedList();
    
    Iterator iter=pe.getPostfixExpressionTail().iterator();
    boolean stopCreating = false;
    
    while (iter.hasNext()) {
        APostfixExpressionTail currentPET=(APostfixExpressionTail) iter.next();
	if (currentPET == pet) {
	    stopCreating = true;
	}
        if (!stopCreating) {
	    newElementPostfixExpressionTail.add(currentPET);
	    
	    // System.err.println("Added to newElementPostfixExpressionTail : " + currentPET.toString() );
        }
        
    }
    
    elementPostfixExpression = new APostfixExpression( pe.getPrimaryExpression(),
						       newElementPostfixExpressionTail ); 

    // System.err.println("elementPostfixExpression ==  " + elementPostfixExpression.toString() );

    newElementExpression = new AExpression(new LinkedList(),
					   new ALogicalExpression( 
						new ARelationalExpression( 
						    new AAdditiveExpression( 
							  new AMultiplicativeExpression( 
							       new APostfixUnaryExpression( elementPostfixExpression ),
							       new LinkedList() ),
							  new LinkedList()), 
						    null),
						new LinkedList())
					       );


    // System.err.println("newElementExpression ==  " + newElementExpression.toString() );


    //  Generate new Set literal : Set { newElementExpression }

    ALiteralCollection newLiteralCollection =
	new ALiteralCollection(new ASetCollectionKind(new TTSet("Set")),
			       new TLBrace(),
			       new AExpressionListOrRange(newElementExpression, null),
			       new TRBrace());

    // System.err.println("newLiteralCollection ==  " + newLiteralCollection.toString() );



    // Generate the new postfixExpressionTails ...
    
    iter=pe.getPostfixExpressionTail().iterator();
    boolean startAdding = false;
    List newPostfixExpressionTails = new LinkedList();
 
    while (iter.hasNext()) {
        APostfixExpressionTail currentPET=(APostfixExpressionTail) iter.next();
	if (currentPET == pet) {
	    startAdding = true;
	}
        if ( startAdding ) {
	    newPostfixExpressionTails.add(currentPET);

	    // System.err.println("Added to newPostfixExpressionTail : " + currentPET.toString() );
        }
        
    }
    

    newPrimaryExpression = new ALitColPrimaryExpression(newLiteralCollection);
    
      // System.err.println("newPriamaryExpression ==  " + newPrimaryExpression.toString() );
      // System.err.println("newPostfixExpressionTails ==  " + newPostfixExpressionTails.toString() );
      // System.err.println("newPostfixExpressionTails size ==  " + newPostfixExpressionTails.size() );
      


    // change existing pet to new collect()
    pe.setPrimaryExpression( newPrimaryExpression );

    // This is necessary because of an 'error' in APostfixExpression.setPostfixExpressionTail !!!! :-(
    List oldPET = pe.getPostfixExpressionTail();
    oldPET.clear();

    pe.setPostfixExpressionTail(newPostfixExpressionTails);

      // System.err.println("new pe ==  " + pe.toString() );
      
    tree.changeNotify(pe);
    
  }

    protected boolean isUMLModelType(Type t){
	boolean result = (!t.equals(Basic.INTEGER) &&
			  !t.equals(Basic.REAL) &&
			  !t.equals(Basic.BOOLEAN) &&
			  !t.equals(Basic.STRING) &&
			  !t.equals(Basic.ENUMERATION) &&
			  !(t instanceof Collection) &&
			  !(t instanceof OclType) &&
			  !(t instanceof OclState) );
	return result;
	
    }

    

  public void open(OclTree tree, NormalizerPass pass) {
    
  }

  public void close(OclTree tree, NormalizerPass pass) {
  }

} 






