// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.parser;

import junit.framework.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.java.*;
import java.io.*;
import de.uka.ilkd.key.gui.AbbrevMap;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.LogicVariable;
import de.uka.ilkd.key.rule.TacletForTests;
import de.uka.ilkd.key.rule.SetAsListOfTaclet;


public class TestTermParser extends TestCase {
    
    private TermFactory tf = TermFactory.DEFAULT;

    NamespaceSet nss;

    Choice defaultChoice;

    Services serv;

    Recoder2KeY r2k;

    Sort elem,list,int_sort;

    Function head,tail,nil,cons,isempty,p; 

    LogicVariable x,y,z,xs,ys,zs;

    Term t_x,t_y,t_z,t_xs,t_ys,t_zs;
    Term t_headxs,t_tailxs,t_tailys,t_nil;

    public TestTermParser(String name) {
	super(name);
    }


    public void setUp() {
	defaultChoice = new Choice(new Name("Default"));
	nss = new NamespaceSet();
	serv = TacletForTests.services ();
	r2k = new Recoder2KeY(serv, nss);
	r2k.parseSpecialClasses();	
	parseDecls("sorts { elem; list; int; int_sort; numbers;  }\n" +
		   "functions {\n" +
		   "  elem head(list);\n" +
		   "  list tail(list);\n" +
		   "  list nil;\n" +
		   "  list cons(elem,list);\n"  +
		   "numbers #;\n"+
		   "numbers 0 (numbers);\n"+
		   "numbers 1 (numbers);\n"+
		   "numbers 2 (numbers);\n"+
		   "numbers 3 (numbers);\n"+
		   "numbers 4 (numbers);\n"+
		   "numbers 5 (numbers);\n"+
		   "numbers 6 (numbers);\n"+
		   "numbers 7 (numbers);\n"+
		   "numbers 8 (numbers);\n"+
		   "numbers 9 (numbers);\n"+
		   "numbers neg (numbers);\n"+
		   "int Z (numbers);\n"+
		   "int ~m (int);\n"+
		   "}\n" +
		   "predicates {\n" +
		   "  isempty(list);\n" +
		   "  p(elem,list);\n" +
		   "}\n"

		   );

        int_sort = lookup_sort("int");
	elem = lookup_sort("elem");
	list = lookup_sort("list");
//    int_sort = lookup_sort("int");

	head = lookup_func("head"); tail = lookup_func("tail");
	nil = lookup_func("nil"); cons = lookup_func("cons");
	isempty = lookup_func("isempty"); p = lookup_func("p");
	
	// The declaration parser cannot parse LogicVariables; these
	// are normally declared in quantifiers, so we introduce them
	// ourselves!
	x = declareVar("x",elem);   t_x = tf.createVariableTerm(x);  
	y = declareVar("y",elem);   t_y = tf.createVariableTerm(y);  
	z = declareVar("z",elem);   t_z = tf.createVariableTerm(z);  
	xs = declareVar("xs",list); t_xs = tf.createVariableTerm(xs);
	ys = declareVar("ys",list); t_ys = tf.createVariableTerm(ys);
	zs = declareVar("zs",list); t_zs = tf.createVariableTerm(zs);
	
	t_headxs = tf.createFunctionTerm(head,t_xs);
	t_tailxs = tf.createFunctionTerm(tail,t_xs);
	t_tailys = tf.createFunctionTerm(tail,t_ys);
	t_nil = tf.createFunctionTerm(nil);
    }

    Sort lookup_sort(String name) {
	return (Sort)nss.sorts().lookup(new Name(name));
    }
    
    Function lookup_func(String name) {
	return (Function)defaultChoice.funcNS().lookup(new Name(name));
    }

    LogicVariable declareVar(String name,Sort sort) {
	LogicVariable v = new LogicVariable(new Name(name),sort);
	nss.variables().add(v);
	return v;
    }
    

    private DeclParser stringDeclParser(String s) {
	Services serv = TacletForTests.services ();
	Recoder2KeY r2k = new Recoder2KeY(serv, nss);
	r2k.parseSpecialClasses();
	return new DeclParser(new KeYLexer(new StringReader(s)),
			      "No file. Call of parser from parser/TestTermParser.java",
			      serv, nss, defaultChoice);
    }

    public void parseDecls(String s) {
	try {
	    DeclParser p = stringDeclParser(s);
	    p.decls();
	} catch (Exception e) {
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw);
	    e.printStackTrace(pw);
	    throw new RuntimeException("Exc while Parsing:\n" + sw );
	}
    }

    public Term parseProblem(String s) {
	try {
	    Services serv = TacletForTests.services ();
	    Recoder2KeY r2k = new Recoder2KeY(serv, nss);
	    r2k.parseSpecialClasses();
	    ProblemParser p = new ProblemParser
		(new KeYLexer(new StringReader(s)),
		 "No file. Call of parser from parser/TestTermParser.java",
		 new ParserConfig(serv, nss, defaultChoice),
		 new ParserConfig(serv, nss, defaultChoice),
		 null, SetAsListOfTaclet.EMPTY_SET);
	    return p.problem();
	} catch (Exception e) {
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw);
	    e.printStackTrace(pw);
	    throw new RuntimeException("Exc while Parsing:\n" + sw );
	}	
    }

    private TermParser stringTermParser(String s) {
	TermParser p =  new TermParser
	    (new KeYLexer(new StringReader(s)), 
	     "No file. Call of parser from parser/TestTermParser.java",
	     tf, 
	     r2k=new Recoder2KeY(TacletForTests.services(), nss),
	     TacletForTests.services(), nss, 
	     new AbbrevMap());
	return p;
    }

    public Term parseTerm(String s) {
	try {
	    TermParser p = stringTermParser(s);
	    return p.term();
	} catch (Exception e) {
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw);
	    e.printStackTrace(pw);
	    throw new RuntimeException("Exc while Parsing:\n" + sw );
	}
    }


    public Term parseFma(String s) {
	try {
	    TermParser p = stringTermParser(s);
	    
	    return p.formula();
	} catch (Exception e) {
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw);
	    e.printStackTrace(pw);
	    throw new RuntimeException("Exc while Parsing:\n" + sw );
	}
    }

    public void test1() {
	assertEquals("parse x",t_x,parseTerm("x"));
    }

    public void test1a() {
	boolean parsed = false;
	try { parseTerm("x()"); parsed = true;} catch (Exception e) {}
	assertFalse("parse x() should fail", parsed);
    }

    public void test2() {
	assertEquals("parse head(xs)",t_headxs,parseTerm("head(xs)"));
    }

    public void test3() {
	Term t = tf.createFunctionTerm(cons,t_headxs,t_tailys);
	
	assertEquals("parse cons(head(xs),tail(ys))",
		     t,parseTerm("cons(head(xs),tail(ys))"));
    }

    public void test5() {
	Term t = tf.createEqualityTerm
	    (tf.createFunctionTerm
	     (head,
	      tf.createFunctionTerm(cons, t_x, t_xs)),
	     tf.createFunctionTerm
	     (head,
	      tf.createFunctionTerm (cons,t_x,t_nil)));
	     
	assertEquals("parse head(cons(x,xs))=head(cons(x,nil))",
		     t,parseFma("head(cons(x,xs))=head(cons(x,nil))"));
	assertEquals("parse head(cons(x,xs))=head(cons(x,nil))",
		     t,parseFma("head(cons(x,xs))=head(cons(x,nil()))"));
    }

    public void test6() {
	Term t = tf.createJunctorTerm
	    (Op.EQV,
	     tf.createJunctorTerm
	     (Op.IMP,
	      tf.createJunctorTerm
	      (Op.OR,
	       tf.createEqualityTerm(t_x,t_x),
	       tf.createEqualityTerm(t_y,t_y)),
	      tf.createJunctorTerm
	      (Op.AND,
	       tf.createEqualityTerm(t_z,t_z),
	       tf.createEqualityTerm(t_xs,t_xs))),
	     tf.createJunctorTerm
	     (Op.NOT,
	      tf.createEqualityTerm(t_ys,t_ys)));

	     
	assertEquals("parse x=x | y=y -> z=z & xs =xs <-> ! ys = ys",
		     t,parseFma("x=x|y=y->z=z&xs=xs<->!ys=ys"));
    }

    public void test7() {
	/** Bound variables are newly created by the parser,
	 * so we have to parse first, then extract the used variables,
	 * then build the formulae. */
	
	String s = "all x:list. all l1:list. ! x = l1";
	Term t = parseFma(s);
	
	LogicVariable thisx = (LogicVariable) t.varsBoundHere(0)
	    .getQuantifiableVariable(0);
	LogicVariable l1 = (LogicVariable) t.sub(0).varsBoundHere(0)
	    .getQuantifiableVariable(0);

	Term t1 = tf.createQuantifierTerm
	    (Op.ALL,thisx,
	     tf.createQuantifierTerm
	     (Op.ALL,l1,
	      tf.createJunctorTerm
	      (Op.NOT,
	       tf.createEqualityTerm(tf.createVariableTerm(thisx),
				     tf.createVariableTerm(l1)))));
	
	assertTrue("new variable in quantifier", thisx != x);
	assertEquals("parse all x:list. all l1:list. ! x = l1", t1,t);
	
    }

    public void test8() {
	/** A bit like test7, but for a substitution term */

	{
	    String s = "{xs:elem head(xs)} cons(xs,ys)";
	    Term t = parseTerm(s);

	    LogicVariable thisxs = (LogicVariable) t.varsBoundHere(1)
		.getQuantifiableVariable(0);
	
	    Term t1 = tf.createSubstitutionTerm
		(Op.SUBST,
		 thisxs, t_headxs,
		 tf.createFunctionTerm
		 (cons, 
		  tf.createVariableTerm(thisxs), 
		  t_ys));

	    assertTrue("new variable in subst term", thisxs != xs);
	    assertEquals("parse {xs:elem head(xs)} cons(xs,ys)",t1,t);
	}

	{
	    String s = "{xs:elem head(xs) waryEx} p(xs,ys)";
	    Term t = parseTerm(s);

	    LogicVariable thisxs = (LogicVariable) t.varsBoundHere(1)
		.getQuantifiableVariable(0);
	
	    Term t1 = tf.createSubstitutionTerm
		(Op.WARY_SUBST_EX,
		 thisxs, t_headxs,
		 tf.createFunctionTerm
		 (p, 
		  tf.createVariableTerm(thisxs), 
		  t_ys));

	    assertTrue("new variable in subst term", thisxs != xs);
	    assertEquals("parse {xs:elem head(xs) waryEx} p(xs,ys)",t1,t);
	}

	{
	    String s = "{xs:elem head(xs) waryAll} p(xs,ys)";
	    Term t = parseTerm(s);

	    LogicVariable thisxs = (LogicVariable) t.varsBoundHere(1)
		.getQuantifiableVariable(0);
	
	    Term t1 = tf.createSubstitutionTerm
		(Op.WARY_SUBST_ALL,
		 thisxs, t_headxs,
		 tf.createFunctionTerm
		 (p, 
		  tf.createVariableTerm(thisxs), 
		  t_ys));

	    assertTrue("new variable in subst term", thisxs != xs);
	    assertEquals("parse {xs:elem head(xs) waryAll} p(xs,ys)",t1,t);
	}
    }

    public void test9() {
	/* Try something with a prediate */
	
	String s = "ex x:list. !isempty(x)";
	Term t = parseFma(s);
	
	LogicVariable thisx = (LogicVariable) t.varsBoundHere(0)
	    .getQuantifiableVariable(0);

	Term t1 = tf.createQuantifierTerm
	    (Op.EX,thisx,
	     tf.createJunctorTerm
	     (Op.NOT,
	      tf.createFunctionTerm(isempty,tf.createVariableTerm(thisx))));
	      
	assertTrue("new variable in quantifier", thisx != x);
	assertEquals("parse all x:list. all l1:list. ! x = l1", t1,t);
	
    }

    public void test10() {
	// Unquoted, this is
	// <{ int x = 2; {String x = "\"}";} }> true
	//	String s = "< { int x = 1; {String s = \"\\\"}\";} } > true";
	String s = "< { int x = 1; {int s = 2;} } > x=x";
	Term t = parseTerm(s);
	
	// for now, just check that the parser doesn't crash
	
	//	 System.out.println(t);

	// Same with a box
	s = "[{ int x = 2; {String s = \"\\\"}\";} }] true";
	//s = "< { int x = 1; {int s = 2;} } > true";
	t = parseTerm(s);
	//System.out.println(t);
    }

    public void test10a() {
	// Unquoted, this is
	// <{ int x = 2; {String x = "\"}";} }> true
	//	String s = "< { int x = 1; {String s = \"\\\"}\";} } > true";
	String s = "< { int x = 1; int y=2; int z=3; int w=4;} > (x ?= y).z /->w";
	//Term t = parseTerm(s);
	
	//	System.out.println(t);
    }


    public void test12() {
	    String s="<{int i; i=0;}> <{ while (i>0) ;}>true";
	    Term t=parseTerm(s);
    }

    public void test13(){
	    Term t1 = parseTerm("ex x:elem . all ys:list. all xs:list.( xs ="
		                    			+" cons(x,ys))");
	    Term t2 = parseTerm("ex y:elem . all xs:list. all ys:list. ( ys ="
                                        +" cons(y,xs))");
        Term t3 = parseTerm("ex bi:int_sort.(< { int x = 1;"
                            +" {int s = 2;} } >"
                            +" x=x ->"
                            +"< { int x = 1;boolean y=2<1;"
                            +" while(y){ int s=3 ;} } >"
                            +" x=x)");
        Term t4 = parseTerm("ex ci:int_sort.(< { int y = 1;"
                            +" {int s = 2;} } >"
                            +" y=y ->"
                            +"< { int y = 1;boolean x = 2<1;"
                            +"while(x){ int s=3 ;} } >"
                            +" y=y)");
        assertTrue("Terms should be equalModRenaming", t3.equalsModRenaming(t4));
     }

    public void test14() {
	    String s="<{int[] i;i=new int[5];}>i[3]=i[2]";
	    Term t=parseTerm(s);
	    s="<{int[] i;}><{}>true";
	    t=parseTerm(s);
    }

    public void testBindingUpdateTerm() {
	    String s="<{int i,j;}> {i:=j} i = j";
	    Term t = parseTerm(s);
	    assertTrue("expected {i:=j}(i=j) but is ({i:=j}i)=j)", 
		       t.sub(0).op() instanceof SimultaneousUpdateOp);
    }

    public void testBindingUpdateConditionalArray() {
	    String s="<{int z; int[] i,j;}> {i:=j} (i?=j) i[0] : j[0] = z";
	    Term t = parseTerm(s);
	    assertTrue("expected {i:=j} ( ((i?=j) i : j) = j)", 
		       t.sub(0).op() instanceof SimultaneousUpdateOp &&
		       t.sub(0).sub(2).op() instanceof Equality &&
		       t.sub(0).sub(2).sub(0).op() instanceof IfElse &&
		       t.sub(0).sub(2).sub(0).sub(3).op() instanceof ArrayOp);
    }

    public void testBindingUpdateTermCombinationConditionalTerm2() {
	String s="<{int i,j;}> {i:=j} ((i?=j) i : j) = j";
	Term t = parseTerm(s);
	assertTrue("expected {i:=j} ( ((i?=j) i : j) = j)", 
		   t.sub(0).op() instanceof SimultaneousUpdateOp &&
		   t.sub(0).sub(2).op() instanceof Equality &&
		   t.sub(0).sub(2).sub(0).op() instanceof IfElse);
    }

    public void testParsingArray() {
	    String s="<{int[][] i; int j;}> i[j][j] = j";
	    Term t = parseTerm(s);
    }

    // fails because of lexer matching blocks with braces
    public void xtestParsingArrayWithSpaces() {
	String s="<{int[][] i; int j;}> i[ j ][ j ] = j";
	Term t = parseTerm(s);
    }

    public void testParsingArrayCombination() {
	    String s="<{int[][] i; int j;}> i [i[i[j][j]][i[j][j]]][i[j][i[j][j]]] = j";
	    Term t = parseTerm(s);
    }

    
    public void testAmbigiousFuncVarPred() {
	// tests bug id 216
	String s = "functions {} predicates{gt(int, int);}"+
	    "\nproblem {all x:int.gt(x, 0)}\n proof {\n"+
	    "(branch \"dummy ID\""+
	    "(opengoal \"  ==> true  -> true \") ) }";
	try {
	    Term result = parseProblem(s);
	} catch (RuntimeException re) {
	    System.out.println(re);
	    assertTrue("Fixed bug 216 occured again. The original bug "+
		       "was due to ambigious rules using semantic "+
		       "predicates in a 'wrong' way", false);
	}
    }

    public void testParseQueriesAndAttributes() {
	TacletForTests.getJavaInfo().readJavaBlock("{}");
	r2k.readCompilationUnit("public class T extends "
				+"java.lang.Object{ "
				+"private T a;"
				+"private static T b;"
				+"T c;"
				+"static T d;"
				+"public T e;"
				+"public static T f;"
				+"protected T g;"
				+"protected T h;"
				+"public T query(){} "
				+"public static T staticQ(T p){} "
				+"public static T staticQ() {}}");
	String s = "<{T t;}> (t.T::query()=t & T::staticQ()=t "
	    +"& T::staticQ(t)=t & T.T::b=t.T::a & T.d=t.c & t.e=T.f & t.g=t.h)";
	Term t =parseTerm(s);
    }

    public void testProgramVariables() {
	TacletForTests.getJavaInfo().readJavaBlock("{}");
	r2k.readCompilationUnit("public class T0 extends "
				+"java.lang.Object{} ");
	String s = "<{T0 t;}> t(1,2) = t()";
	boolean parsed=false;
	try {
	    Term t =parseTerm(s);
	    parsed = true;
	} catch (Exception e) {}
	assertFalse("Program variables should not have arguments", parsed);
    }


    public void testNegativeLiteralParsing() {
	String s1 = "-1234";
	Term t = null;
	try {
	    t = parseTerm(s1);
	} catch (Exception e) {fail();}
	assertTrue("Failed parsing negative literal", 
		   (""+t.op().name()).equals("Z") && 
		   (""+t.sub(0).op().name()).equals("neg"));


	String s2 = "-(1) ";
	try {
	    t = parseTerm(s2);
	} catch (Exception e) {fail();}
	assertTrue("Failed parsing negative complex term", 
		   (""+t.op().name()).equals("~m") && 
		   (""+t.sub(0).op().name()).equals("Z"));

	String s3 = "all i:int.-i = 0 ";
	try {
	    t = parseTerm(s3);
	} catch (Exception e) {fail();}
	assertTrue("Failed parsing negative variable", 
		   (""+t.sub(0).sub(0).op().name()).equals("~m"));

	
    }

}
