// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/* -*-antlr-*- */
header {

package de.uka.ilkd.key.parser;

import java.io.*;
import antlr.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.rule.*;
import de.uka.ilkd.key.proof.RuleSource;
import de.uka.ilkd.key.proof.init.Includes;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.visitor.*;
import java.util.List;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.HashSet;



}

/** 
 * A parser for signature declarations,
 * i.e. sorts, schema variabls, predicates, functions and heuristics.
 */  
class DeclParser extends Parser;
options {
    importVocab=KeYLexer;
    k = 1;
    defaultErrorHandler=false;
}

{
    static final Sort[] AN_ARRAY_OF_SORTS = new Sort[0];
    
    static HashMap name2sort = new HashMap(50);
    
    static {
          name2sort.put("statement",ProgramSVSort.STATEMENT);
	  name2sort.put("expression",ProgramSVSort.EXPRESSION);
	  name2sort.put("javaboolean expression",ProgramSVSort.JAVABOOLEANEXPRESSION);
	  name2sort.put("executioncontext",ProgramSVSort.EXECUTIONCONTEXT);
	  name2sort.put("multiplevardecl",ProgramSVSort.MULTIPLEVARDECL);
	  name2sort.put("simple expression",ProgramSVSort.SIMPLEEXPRESSION);
	  name2sort.put("simple javabyte expression",ProgramSVSort.SIMPLEJAVABYTEEXPRESSION);
	  name2sort.put("simple javachar expression",ProgramSVSort.SIMPLEJAVACHAREXPRESSION);
	  name2sort.put("simple javashort expression",ProgramSVSort.SIMPLEJAVASHORTEXPRESSION);
	  name2sort.put("simple javaint expression",ProgramSVSort.SIMPLEJAVAINTEXPRESSION);
	  name2sort.put("simple javalong expression",ProgramSVSort.SIMPLEJAVALONGEXPRESSION);
	  name2sort.put("simple javabyteshortint expression",ProgramSVSort.SIMPLEJAVABYTESHORTINTEXPRESSION);
	  name2sort.put("simple anyjavatype expression",ProgramSVSort.SIMPLEANYJAVATYPEEXPRESSION);
	  name2sort.put("nonsimple expression",ProgramSVSort.NONSIMPLEEXPRESSION);
	  name2sort.put("nonsimple methodreference",ProgramSVSort.NONSIMPLEMETHODREFERENCE);
	  name2sort.put("loopinit",ProgramSVSort.LOOPINIT);
	  name2sort.put("guard",ProgramSVSort.GUARD);
	  name2sort.put("forupdates",ProgramSVSort.FORUPDATES);
	  name2sort.put("label",ProgramSVSort.LABEL);
	  name2sort.put("type",ProgramSVSort.TYPE);
	  name2sort.put("notprimitive type",ProgramSVSort.TYPENOTPRIMITIVE);
	  name2sort.put("explicit variable",ProgramSVSort.EXPLICITVARIABLE);
          name2sort.put("static variable",ProgramSVSort.STATICVARIABLE);
	  name2sort.put("variable",ProgramSVSort.VARIABLE);
	  name2sort.put("constant variable",ProgramSVSort.CONSTANTVARIABLE);
	  name2sort.put("variableinitializer",ProgramSVSort.VARIABLEINIT);
	  name2sort.put("implicit variable",ProgramSVSort.IMPLICITVARIABLE);
	  name2sort.put("implicit reference field",ProgramSVSort.IMPLICITREFERENCE);
	  name2sort.put("implicit next", ProgramSVSort.IMPLICITNEXT);
          name2sort.put("implicit nextToCreate", ProgramSVSort.IMPLICITNEXTTOCREATE);
          name2sort.put("implicit classInitialized", ProgramSVSort.IMPLICITCLINIT);
          name2sort.put("implicit classInitializationInProgress",ProgramSVSort.IMPLICITINITINPROGRESS);
          name2sort.put("implicit classErroneous", ProgramSVSort.IMPLICITERRONEOUS);
          name2sort.put("implicit classPrepared", ProgramSVSort.IMPLICITPREPARED);
	  name2sort.put("begin transaction",ProgramSVSort.JCBEGINTRANSACTION);
	  name2sort.put("commit transaction",ProgramSVSort.JCCOMMITTRANSACTION);
	  name2sort.put("abort transaction",ProgramSVSort.JCABORTTRANSACTION);
	  name2sort.put("transient booleanarray",ProgramSVSort.JCMAKETRANSIENTBOOLEANARRAY);
	  name2sort.put("transient bytearray",ProgramSVSort.JCMAKETRANSIENTBYTEARRAY);
	  name2sort.put("transient shortarray",ProgramSVSort.JCMAKETRANSIENTSHORTARRAY);
	  name2sort.put("transient objectarray",ProgramSVSort.JCMAKETRANSIENTOBJECTARRAY);
	  name2sort.put("transient istransient",ProgramSVSort.JCISTRANSIENT);
	  name2sort.put("array initializer",ProgramSVSort.ARRAYINITIALIZER);
	  name2sort.put("array length",ProgramSVSort.ARRAYLENGTH);
	  name2sort.put("array creation expression",ProgramSVSort.NEWARRAY);
	  name2sort.put("array post declaration",ProgramSVSort.ARRAYPOSTDECL);
	  name2sort.put("instance creation expression",ProgramSVSort.NEW);
	  name2sort.put("switch",ProgramSVSort.SWITCH);
	  name2sort.put("catch",ProgramSVSort.CATCH);
	  name2sort.put("methodname",ProgramSVSort.METHODNAME);
	  name2sort.put("special constructorreference",ProgramSVSort.SPECIALCONSTRUCTORREFERENCE);
	  name2sort.put("methodbody",ProgramSVSort.METHODBODY);
	  name2sort.put("lefthandside",ProgramSVSort.LEFTHANDSIDE);
    }
    
    private Prioritization.StaticPrioritization heur_prioritization
        = new Prioritization.StaticPrioritization ();
    
    private NamespaceSet nss;
    private Choice defaultChoice = null;
    protected HashMap category2Default = new HashMap();
    protected boolean onlyWith=false;
    protected SetOfChoice activatedChoices = SetAsListOfChoice.EMPTY_SET;
    protected HashSet usedChoiceCategories = new HashSet();

    private Services services;

    // these variables are set if a file is read in step by
    // step. This used when reading in LDTs because of cyclic
    // dependencies.
    protected boolean skip_schemavariables;
    protected boolean skip_functions;
    protected boolean skip_predicates;
    protected boolean skip_sorts;
    protected boolean skip_heuristics;

    private boolean schemaMode = false;
    protected boolean parse_includes = false;
    protected Includes includes = new Includes();

    public DeclParser(TokenStream lexer,
                      String filename,
                      Services services,
                      NamespaceSet nss,
                      Choice defaultChoice) {
        this(lexer);
        setFilename(filename);
        this.nss = nss;
        this.services = services;
        setSkips(false, false, false, false, false);
        switchToNormalMode();
        this.defaultChoice = defaultChoice;
    }

    public DeclParser(TokenStream lexer,
                      String filename,
                      Services services,
                      NamespaceSet nss) {
        this(lexer, filename, services, nss, null);
    }
    
    public JavaInfo getJavaInfo() {
        return getServices().getJavaInfo();
    }

    public Services getServices() {
        return services;
    }

    public Namespace sorts() {
        return nss.sorts();
    }

    public Namespace functions() {
        return nss.functions();
    }

    public Namespace heuristics() {
        return nss.heuristics();
    }

    public Namespace variables() {
        return nss.variables();
    }

    public Namespace programVariables() {
        return nss.programVariables();
    }

    public Namespace choices(){
        return nss.choices();
    }

    public NamespaceSet namespaces() {
        return nss;
    }
    
    public HashMap getCategory2Default(){
        return category2Default;
    }

    private void switchToSchemaMode() {
        schemaMode = true;      
    }

    private boolean inSchemaMode() {
        return schemaMode;
    }

    private void switchToNormalMode() {
        schemaMode = false;  
    }

    public Prioritization getHeuristicPrioritization () {
        return heur_prioritization;
    }

    protected int getLine() {
        int line = -1;
        try {
            line = LT(0).getLine();
        } 
        catch (TokenStreamException e) {
            System.err.println("No further token in stream");
        }
        return line;
    }   

    protected int getColumn() {
        int col = -1;
        try {
            col = LT(0).getColumn();
        } 
        catch (TokenStreamException e) {
            System.err.println("No further token in stream");
        }
        return col;
    }   

    protected void setSkips(boolean svars, boolean funcs, 
                            boolean preds, boolean sorts,
                            boolean heurs) {
        skip_schemavariables = svars;
        skip_functions       = funcs;
        skip_predicates      = preds;
        skip_sorts       = sorts;
        skip_heuristics      = heurs;
    }

    public Named lookup(Name n) {
	final Namespace[] lookups = {
		functions(), variables(), 
		programVariables()
	};
	for (int i = 0; i<lookups.length; i++) {
	    if (lookups[i].lookup(n) != null) {
		return lookups[i].lookup(n);
            }
	}
	return null;	
    }

    public void parseVariables()    
        throws antlr.RecognitionException, antlr.TokenStreamException
        {
            setSkips(false, true, true, true, true); 
            top();
            setSkips(false, false, false, false, false);                
        }

    protected void addInclude(String filename, boolean relativePath, boolean ldt){
        InputStream input=null;
        RuleSource source=null;
        if (relativePath) {
            int end = getFilename().lastIndexOf(File.separator);
            int start = 0;
            filename = filename.replace('/', File.separatorChar);
            filename = filename.replace('\\', File.separatorChar);
            if(getFilename().startsWith("file:")){
                start = 5;
            }
            File path=new File(getFilename().substring(start,end+1)+filename);
            try{ 
                source = RuleSource.initRuleFile(path.toURL()); 
            }catch(java.net.MalformedURLException e){
                System.err.println("Exception due to malformed URL of file "+
                                   filename+"\n " +e);
            }
        } else {
            source = RuleSource.initRuleFile(filename+".key"); 
        }
        if (ldt) {
            includes.putLDT(filename, source);
        } else {
            includes.put(filename, source);
        }
    }  

    public void parseFunctions()
        throws antlr.RecognitionException, antlr.TokenStreamException

        {
            setSkips(true, false, true, true, true); 
            top();
            setSkips(false, false, false, false, false);
        }

    public void parsePredicates()
        throws antlr.RecognitionException, antlr.TokenStreamException
        {
            setSkips(true, true, false, true, true); 
            top();
            setSkips(false, false, false, false, false);        
        }

    public void parseSorts() 
        throws antlr.RecognitionException, antlr.TokenStreamException
        {
            setSkips(true, true, true, false, true); 
            top();
            setSkips(false, false, false, false, false);
        }

    public void parseHeuristics()
        throws antlr.RecognitionException, antlr.TokenStreamException
        {
            setSkips(true, true, true, true, false); 
            top();
            setSkips(false, false, false, false, false);        
        }

    public void parseFuncAndPred()
        throws antlr.RecognitionException, antlr.TokenStreamException
        {
            setSkips(true, false, false, true, true); 
            top();
            setSkips(false, false, false, false, false);
        }
    

    public void schema_var_decl(String name,Sort s,
                                    boolean makeVariableSV,
                                    boolean makeDependingSV,
                                    boolean rigid,
                                    boolean listModifier)
	{
              if (!skip_schemavariables) {
                  SchemaVariable v;
                  if ( s == Sort.FORMULA ) {
                      v = SchemaVariableFactory.createFormulaSV
                          (new Name(name), listModifier, rigid);
                      } else if ( s instanceof ProgramSVSort ) {
                          v = SchemaVariableFactory.createProgramSV
                         (new ProgramElementName(name),(ProgramSVSort) s, listModifier);
                      } else {
                        if ( makeVariableSV ) {
                             v=SchemaVariableFactory.createVariableSV
                             (new Name(name), s, listModifier);
                        } else if ( makeDependingSV ) {
                               v = SchemaVariableFactory.createDependingSV
                             (new Name(name), s, listModifier);
                        } else { v = SchemaVariableFactory.createTermSV
                              (new Name(name), s, listModifier, rigid);
                        }
                     }
                     if (inSchemaMode()) {
                         variables().add(v);
                     }
               }

         }
     
}

top : decls;

decls : 

        (INCLUDE 
            ( LDTS COLON LBRACE include[true] (COMMA include[true])* RBRACE
            |
                include[false]
            )
            SEMI)*
        {
            if(parse_includes) return;
            activatedChoices = SetAsListOfChoice.EMPTY_SET;  
        }
        ("with" LPAREN activated_choice (COMMA activated_choice)* RPAREN)?
        {if(onlyWith) return;}
        (
            option_decls
        |    
            sort_decls
        |
            prog_var_decls
        |
            schema_var_decls
        |
            pred_decls
        |
            func_decls
        |
            heur_decls

        ) *
    ;

include [boolean ldt]:
        (absfile:IDENT{ 
                if(parse_includes){
                    addInclude(absfile.getText(),false,ldt);
                }
            }
        | relfile:STRING_LITERAL{ 
                if(parse_includes){
                    addInclude(relfile.getText(),true,ldt);
                }
            })
    ;

activated_choice{
    String name;
    Choice c;
}:
        cat:IDENT COLON choice:IDENT
        {if(usedChoiceCategories.contains(cat.getText())){
            throw new IllegalArgumentException("You have already chosen a different option for "+cat.getText());
        }
        usedChoiceCategories.add(cat.getText());
        name = cat.getText()+":"+choice.getText();
        c = (Choice) choices().lookup(new Name(name));
        if(c==null){
            throw new NotDeclException("Option", choice,
                                       getFilename());
        }else{
            activatedChoices=activatedChoices.add(c);
            namespaces().setFunctions(functions().extended(c.funcNS().allElements()));
        }
        }
    ;

option_decls{
}:
        "options" LBRACE choice (SEMI choice)* RBRACE 
    ;

choice{
    String cat=null;
}:
        category:IDENT {cat=category.getText();} (COLON LBRACE choice_option[cat] (COMMA choice_option[cat])* RBRACE)? 
        {
            if(!category2Default.containsKey(cat)){
                choices().add(new Choice("On",cat));
                choices().add(new Choice("Off",cat)); 
                category2Default.put(cat, cat+":On");               
            }
        }
    ;

choice_option[String cat]{
    String name;
}:
        choice:IDENT { name=cat+":"+choice.getText();
        Choice c = (Choice) choices().lookup(new Name(name));
        if(c==null){
            c = new Choice(choice.getText(),cat);
            choices().add(c);
        }
        if(!category2Default.containsKey(cat)){
            category2Default.put(cat, name);
        }
        }
    ;

sort_decls {
    boolean isObjectSort=false;
    boolean isGenericSort=false;

    //For OCL simplification
    boolean isOclSort = false;

    Sort[] sortExt=new Sort [0];
    Sort[] sortOneOf=new Sort [0];
    ListOfName sortIds;
}:
        "sorts" 
        LBRACE ( 
            { 
                isObjectSort=false; 
                isGenericSort=false; 
                isOclSort = false;
                sortExt = new Sort[0];
                sortOneOf = new Sort[0];
            }
            ( "object" {isObjectSort=true;} sortIds = sortIdentifiers
            | "generic" {isGenericSort=true;} sortIds = sortIdentifiers
                ( "oneof" sortOneOf = oneof_sorts )? 
                ("extends" sortExt = extends_sorts )?

            | "ocl" {isOclSort=true;} sortIds = sortIdentifiers

            | sortIds = sortIdentifiers

            )
            SEMI
            {   
                if (!skip_sorts) {
                    Name sort_name;
                    IteratorOfName it = sortIds.iterator ();
                    while ( it.hasNext () ) {
                        sort_name = it.next ();

                        if (sorts().lookup(sort_name) == null) {
                            Sort s;
                            if (isObjectSort) {
                                if ("Object".equals(sort_name)) {
                                    if (sorts().lookup(new Name("Object")) == null) {
                                        s = new ClassInstanceSortImpl(sort_name);
                                    }
                                    s=(Sort)sorts().lookup(new Name("Object"));
                                } else {
                                    s = new ClassInstanceSortImpl(sort_name, 
                                                                  (Sort)sorts().lookup
                                                                  (new Name("Object")));
                                }	
                            }                            
                            //Just constant for now
                            else if (isOclSort) {
                               s = Sort.OCL_ANY;
                            }
                            //
                            else if (isGenericSort) {
                                int        i;
                                SetOfSort  ext   = SetAsListOfSort.EMPTY_SET;
                                SetOfSort  oneOf = SetAsListOfSort.EMPTY_SET;

                                for ( i = 0; i != sortExt.length; ++i )
                                    ext = ext.add ( sortExt[i] );

                                for ( i = 0; i != sortOneOf.length; ++i )
                                    oneOf = oneOf.add ( sortOneOf[i] );

                                try {
                                    s = new GenericSort ( sort_name,
                                                          ext,
                                                          oneOf );
                                } catch ( GenericSupersortException e ) {
                                    throw new GenericSortException ( "sort",
                                                                     "Illegal sort given", e.getIllegalSort (),
                                                                     getFilename (), getLine (), getColumn () );
                                }
                            } else {
                                s = new PrimitiveSort(sort_name);
                            }
                            sorts().add(s);
                            if (s instanceof NonCollectionSort) {
                                NonCollectionSort ns
                                    = (NonCollectionSort)s;
                                sorts().add(ns.getSetSort());
                                sorts().add(ns.getSequenceSort());
                                sorts().add(ns.getBagSort());
                            }
                            if ( s instanceof SortDefiningSymbols )
                                ((SortDefiningSymbols)s).createSymbols ( defaultChoice.funcNS(),
                                                                         sorts() );
                        }

                    }
                }
            }
            
        )* 
        RBRACE
    ;

sortIdentifiers returns [ListOfName ids = SLListOfName.EMPTY_LIST]
    :
        id:IDENT { ids = ids.append ( new Name(id.getText()) ); }
        (
            COMMA
            id2:IDENT { ids = ids.append ( new Name(id2.getText()) ); }
        ) *
    ;

extends_sorts returns [Sort[] extendsSorts = null] 
{
    List args = new LinkedList();
    Sort s;
}
    :
        s = any_sortId_check[true] { args.add(s); }
        (
            COMMA s = any_sortId_check[true] {args.add(s);}
        ) *
        {
            extendsSorts = (Sort[])args.toArray(AN_ARRAY_OF_SORTS);
        }
    ;

oneof_sorts returns [Sort[] oneOfSorts = null] 
{
    List args = new LinkedList();
    Sort s;
}
    :
        LBRACE
        s = sortId_check[true] { args.add(s); }
        (
            COMMA s = sortId_check[true] {args.add(s);}
        ) *
        RBRACE
        {
            oneOfSorts = (Sort[])args.toArray(AN_ARRAY_OF_SORTS);
        }
    ;

prog_var_decls 
{
    String var_name;
    String type = null;
}
    :
        { switchToNormalMode(); }
        "program" "variables"
        LBRACE 
        (
            ((v0:IDENT {type= "" + v0.getText();} (DOT v1:IDENT {type= type + "." + v1.getText();})*)
              |
             (pid:PIDENT {type = pid.getText();} )
            )
            (LBRACKET RBRACKET {
               type = type + "[]";
            })?
            //sortId_check[true]
            var_name = id
            {
                KeYJavaType kjt = getJavaInfo().getKeYJavaType(type);
		ProgramElementName pvName = new ProgramElementName(var_name);
                if (kjt == null) {
                    throw new antlr.SemanticException
                        ("Program Variable " + var_name + 
                         " declared with unknown type " + 
                         type);

                } else if (lookup(pvName)!=null) {
		  // commented out as pv do not have unique name (at the moment)
		    //throw new AmbigiousDeclException
		//	(var_name, getFilename(), getLine(), getColumn()); 
		}
            namespaces().programVariables().add(new ProgramVariable
                (new ProgramElementName(var_name), kjt));

            }
            SEMI
        ) *
        RBRACE
    ;

schema_var_decls
{
    Sort s = null;
    boolean makeVariableSV  = false;
    boolean makeDependingSV = false; // skolem functions
    boolean rigid           = false;
    boolean listModifier    = false;
    String type = "";    
}
    :
        "schema" "variables"
        LBRACE  { switchToSchemaMode(); }
        (
            ( "modal" "operator" one_schema_modal_decl (COMMA one_schema_modal_decl)* SEMI )
        |
            {makeVariableSV = false; makeDependingSV = false; rigid = false; listModifier=false;}
            ( ( "program" ("svlist" {listModifier=true;})? {type = "";}
                ((IDENT IDENT) => id1:IDENT{type += " "+id1.getText();}
                 | id2:IDENT{ type = type.substring(1,type.length());
                              s = (Sort) DeclParser.name2sort.get(type);
                              if (s == null) {
                                  throw new antlr.SemanticException
                                  ("SchemaVariable of sort "+type+ " not found.");
                              }
                              schema_var_decl(id2.getText(),s,makeVariableSV,makeDependingSV,rigid,listModifier);} 
                )+
                (COMMA id3:IDENT{schema_var_decl(id3.getText(),s,makeVariableSV,makeDependingSV,rigid,listModifier);})* SEMI
               )
               | "formula"
                  ( "rigid" { rigid = true; } ) ? {s = Sort.FORMULA;type = ""; }
		  (id5:IDENT {schema_var_decl(id5.getText(),s,makeVariableSV,makeDependingSV,rigid,listModifier);})
		  ( COMMA (id6:IDENT {schema_var_decl(id6.getText(),s,makeVariableSV,makeDependingSV,rigid,listModifier);}))* SEMI
               | s = any_sortId_check[true]
                  ( "variables" { makeVariableSV = true; }
                    | "depending" { makeDependingSV = true; }
                    | "rigid" { rigid = true; } ) ? {type = "";}
                  (id7:IDENT {schema_var_decl(id7.getText(),s,makeVariableSV,makeDependingSV,rigid,listModifier);})
	          ( COMMA (id8:IDENT {schema_var_decl(id8.getText(),s,makeVariableSV,makeDependingSV,rigid,listModifier);}))* SEMI
             )
        ) *
        { switchToNormalMode(); }
        RBRACE
    ;



one_schema_modal_decl
{
    Modality[] mods;
} 
    :
        id:IDENT EQUALS mods=modalities_decl {
            SchemaVariable msv = (SchemaVariable)variables().lookup(new Name(id.getText()));
            if(msv != null)
            throw new antlr.SemanticException("Schema variable "+id.getText()+ " already defined");
            int arity = mods[0].arity();
            for(int i=1;i<mods.length;i++)
            if(arity != mods[i].arity())
            throw new antlr.SemanticException("Arity mismatch for schema variable "+id.getText());
            msv = SchemaVariableFactory.createModalOperatorSV(new Name(id.getText()), arity, mods);
            if (inSchemaMode()) {
                variables().add(msv);
            }
        }
    ;

modalities_decl returns [Modality[] modalities = null] 
{
    List args = new LinkedList();
}
    : (LBRACE 
            s1:IDENT {
                if(s1.getText().charAt(0) == '#') {
                    ModalOperatorSV msv = (ModalOperatorSV)variables().lookup(new Name(s1.getText()));
                    if(msv == null)
                        throw new antlr.SemanticException("Schema variable "+s1.getText()+ " not defined");
                    for(int i=0; i<msv.modalities().length; i++)
                        args.add(msv.modalities()[i]);
                }else{
                    Modality op = Op.getModality(s1.getText());
                    if(op == null)
                        throw new antlr.SemanticException("Unrecognised modal operator "+s1.getText());
                    args.add(op);
                }
            } (COMMA s2:IDENT {
                    if(s2.getText().charAt(0) == '#') {
                        ModalOperatorSV msv = (ModalOperatorSV)variables().lookup(new Name(s2.getText()));
                        if(msv == null)
                            throw new antlr.SemanticException("Schema variable "+s2.getText()+ " not defined");
                        for(int i=0; i<msv.modalities().length; i++)
                            args.add(msv.modalities()[i]);
                    } else {
                        Modality op = Op.getModality(s2.getText());
                        if(op == null)
                            throw new antlr.SemanticException("Unrecognised modal operator "+s2.getText());
                        args.add(op);
                    }
                })* RBRACE
        ) {
            modalities = (Modality[])args.toArray(new Modality[args.size()]);
        }
    ;

pred_decls 
{
    Sort[] argSorts;
    String pred_name;
    Namespace oldFNS=null;
    String ctext = null;
    Choice c=null;
}
    :
        "predicates" (LPAREN cat:IDENT COLON choice:IDENT RPAREN)?
        { if(cat!=null){
            ctext = cat.getText()+":"+choice.getText();
            c = (Choice) choices().lookup(new Name(ctext));
            if(c==null){
                throw new NotDeclException("Option", choice,
                                           getFilename());
            }else{
                namespaces().setFunctions(c.funcNS());
            }
        }else{
            namespaces().setFunctions(defaultChoice.funcNS());
        }
        cat=null;
        }
        LBRACE
        (
            pred_name = id
            argSorts=arg_sorts[!skip_predicates]
            {
                if (!skip_predicates) {
                    Name predicate = new Name(pred_name);		    		    
                    if (lookup(predicate) != null) {
                        throw new AmbigiousDeclException
                        (pred_name, getFilename(), getLine(), getColumn());
                    } else {
                        Function p = new Function(predicate,
                                                  Sort.FORMULA, 
                                                  argSorts);
                        functions().add(p);         
                    }
                } 
            }
            SEMI
        ) *
        RBRACE
    ;

func_decls 
{
    Sort[] argSorts;
    Sort retSort;
    String func_name;
    Namespace oldFNS=null;
    String ctext = null;
    Choice c=null;
}
    :
        "functions" (LPAREN cat:IDENT COLON choice:IDENT RPAREN)?
        { 
            if(cat!=null){
                ctext = cat.getText()+":"+choice.getText();
                c = (Choice) choices().lookup(new Name(ctext));
                if(c==null){
                    throw new NotDeclException("Option", choice,
                                               getFilename());
                }
            }else{
                c=defaultChoice;
            }
            namespaces().setFunctions(c.funcNS());
            cat=null;
        }
        LBRACE
        (
            retSort = sortId_check[!skip_functions]
            func_name = id 
            argSorts = arg_sorts[!skip_functions]
            {
                if (!skip_functions) {
                    Name fct_name = new Name(func_name);
                    if (lookup(fct_name) != null) {
                        throw new AmbigiousDeclException
                        (func_name, getFilename(), getLine(), getColumn());
                    } else {
                        functions().add(new Function(fct_name,
     	                                             retSort, 
        	                                     argSorts));
                    }
                } 
            }
            SEMI
        ) *
        RBRACE
    ;

arg_sorts[boolean checkSort] returns [Sort[] argSorts = null] 
{
    List args = new LinkedList();
    Sort s;
}
    :
        (
            LPAREN
            s = sortId_check[checkSort] { args.add(s); }
            (
                COMMA s = sortId_check[checkSort] {args.add(s);}
            ) *
            RPAREN
        ) ?
        {
            argSorts = (Sort[])args.toArray(AN_ARRAY_OF_SORTS);
        }
    ;


integer_num returns [ int num = 0 ]
options {defaultErrorHandler=false;}
    :
        {
            try {
                Token t = LT(1);
                num = Integer.decode ( t.getText () ).intValue ();
                consume ();
            } catch ( TokenStreamException e ) {
                throw new antlr.SemanticException ( "Expected a number",
                                                    getFilename (), getLine (), getColumn () );
            } catch ( NumberFormatException e ) {
                throw new antlr.SemanticException ( "Expected a number",
                                                    getFilename (), getLine (), getColumn () );
            } catch ( NullPointerException e ) {
                throw new antlr.SemanticException ( "Expected a number",
                                                    getFilename (), getLine (), getColumn () );
            }
        }
    ;

heur_decls
{
    int     t_priority = 0;
    Integer priority   = null;
}
    :
        "heuristics"
        LBRACE
        (
            { priority = null; }
            id:IDENT
            (
                LPAREN t_priority = integer_num RPAREN 
                { priority = new Integer ( t_priority ); }
            ) ?
            SEMI
            { 
                if (!skip_heuristics) {
                    Heuristic h = new Heuristic(new Name(id.getText()));
                    if(heuristics().lookup(new Name(id.getText()))==null){
                        heuristics().add(h);
                    }
                    if ( priority != null )
                        heur_prioritization.setPriority ( h, priority.intValue () );
                }
            }
        ) *
        RBRACE
    ;

sortId  returns [Sort s = null]
    :
        s = sortId_check[true]
    ;           

// Non-generic sorts, array sorts allowed
sortId_check [boolean checkSort] returns [Sort s = null]                
{
    Sort t;
}
    :
        t = sortId_check_help[checkSort]
        s = array_decls[t]
    ;

// Generic and non-generic sorts, array sorts allowed
any_sortId_check [boolean checkSort] returns [Sort s = null]                
{
    Sort t;
}
    :
        t = any_sortId_check_help[checkSort]
        s = array_decls[t]
    ;

// also allow generic sorts
any_sortId_check_help [boolean checkSort] returns [Sort s = null]
{
    String name;
}
    :
        name = id
        {   s= (Sort) sorts().lookup(new Name(name));
            if (checkSort && s == null) {
                throw new NotDeclException("sort", name, 
                                           getFilename(), 
                                           getLine(), getColumn());
            }
        }
    ;

sortId_check_help [boolean checkSort] returns [Sort s = null]
    :
        s = any_sortId_check_help[checkSort]
        {
            // don't allow generic sorts or collection sorts of
            // generic sorts at this point
            Sort t = s;
            while ( t instanceof CollectionSort )
            t = ((CollectionSort)t).elementSort ();
            if ( t instanceof GenericSort )
            throw new GenericSortException ( "sort",
                                             "Non-generic sort expected", t,
                                             getFilename (), getLine (), getColumn () );
        }
    ;

array_decls [Sort p] returns [Sort s = null]                
{
    int  n=0;
}

    :
        (LBRACKET RBRACKET {n++;})*

        { 
            if(n != 0){
                s = ArraySortImpl.getArraySortForDim(
                    p, n, getJavaInfo().getObjectSort(),
                    getJavaInfo().getCloneableSort());
            }else{
                s = p;
            }
        }
            
    ;

id_declaration returns [ IdDeclaration idd = null ]
{
    Sort s = null;
}
    :
        id:IDENT
        ( COLON s = sortId_check[true] ) ?
        {
            idd = new IdDeclaration ( id.getText (), s );
        }
    ;

id returns [String name = null]
    :
        (prefix:IDENT 
	{ name = prefix.getText(); }
        | pprefix : PIDENT { name = pprefix.getText(); })
        ((COLON COLON)=>COLON COLON nameID:IDENT 
            { 
                name += "::" + nameID.getText();
            }
        )?
    ;

