// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/* -*-antlr-*- */
header {

    package de.uka.ilkd.key.parser;

    import de.uka.ilkd.key.logic.*;
    import de.uka.ilkd.key.logic.op.*;
    import de.uka.ilkd.key.java.JavaInfo;
    import de.uka.ilkd.key.java.abstraction.KeYJavaType;
    import de.uka.ilkd.key.java.*;
    import de.uka.ilkd.key.java.reference.*;
    import java.util.HashSet;

}

/** 
 * A parser for modifies clauses,
 * i.e. FieldReferences.
 */  

class ModifiesParser extends Parser;
options {
	importVocab=KeYLexer;
	k = 3;
	defaultErrorHandler=false;
}

{
	private Namespace ns;
	private HashSet fields = new HashSet();
	private JavaInfo javaInfo;
	private boolean isStatic = false;
	private String packagePrefix = new String();

	public ModifiesParser(TokenStream lexer,
			  Namespace ns,
			  JavaInfo javaInfo) {
	    this(lexer);
	    this.ns = ns;
	    this.javaInfo = javaInfo;
	}
		
	public Namespace namespace() {
	    return ns;
        }

	public JavaInfo getJavaInfo() {
	    return javaInfo;
	}

	public HashSet modifiesElements() {
	    return fields;
	}

}



top {ProgramElement f;} : f = fieldref {fields.add(f);} 
			   (COMMA f = fieldref {fields.add(f);})* ;

fieldref returns [ProgramElement pe = null]  {String y = null;
					      KeYJavaType last = null;
					     } :
		(
			((y = programvar_or_typeref)
			{ pe = (ProgramVariable) ns.lookup(new Name(y));
			  if (pe == null) {
                       		last = javaInfo.getKeYJavaType(y);
				if (last == null) {
					throw new 
			   		 RuntimeException("Program Variable or Typeref not found: "+y);
				}
				pe = new TypeRef(last);
				isStatic = true;
       			  } else
			  last = ((ProgramVariable)pe).getKeYJavaType();
			} 
            		| LPAREN (id:IDENT DOT 
                    	{packagePrefix = packagePrefix+id.getText()+".";
                    	})* (y = programvar_or_typeref)
                	{
                    	  last = javaInfo.getKeYJavaType(packagePrefix+y);
			  if (last == null) {
				throw new 
			   		 RuntimeException("Program Variable or Typeref not found: "+y);
			  }
			  pe = new TypeRef(last);
			  isStatic = true;
                	}
             		  RPAREN )
			(DOT y = programvar_or_typeref
			{
			  ProgramVariable x = javaInfo.getAttribute(y, last);
              if (x==null) { // for private attributes, though OCL has
			                 // no visibilities
		          x=javaInfo.getAttribute
		                (last.getSort().toString()+"::"+y, 
		                 last);
	          }
			  if (x == null) {
			    throw new
			    RuntimeException("Attribute "+y+" not found in "
						+last+"   "+pe);
			  }
			  if (isStatic) {
				if (!x.isStatic()) {
					throw new
			    		RuntimeException("ProgramVariable "+x+" is not static but "+
							  "a static field was expected.");
				}
				isStatic = false;
			  }
			  pe = new FieldReference(x, (ReferencePrefix) pe);
			  last = ((ProgramVariable)x).getKeYJavaType();
			})*
		) ;

programvar_or_typeref returns [String pv = null]:
   id:IDENT 
	{pv = (String) id.getText(); 
	} ;

