// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.proof;

import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.logic.ldt.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.gui.Main;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.rule.*;
import java.io.*;

public class CountingBufferedInputStream extends BufferedInputStream {

    private int chars;
    private long step=0;
    private Main main=null;

    public CountingBufferedInputStream(InputStream in) {
	super(in);
	chars=0;
    }

    public CountingBufferedInputStream(InputStream in, Main main, int step) {
	this(in);
	this.step=step;
	this.main=main;
	chars=0;
    }

    public CountingBufferedInputStream(InputStream in, Main main, 
				       int step, int alreadyRead) {
	this(in, main, step);
	chars=alreadyRead;
    }

    public CountingBufferedInputStream(InputStream in, int size, int step) {
	super(in, size);
	this.step=step;
	chars=0;
    }
  
    public int read() throws IOException{
	try{
	    chars++;
	    if (chars % step==0) {
		main.setProgress(chars);
	    }
	} catch (ArithmeticException aex) {
	    Debug.out("Exception thrown by class CountingBufferedInputStream at setProgress()");
	    } finally {
	    return super.read();
	}
    }

    public int getNumberOfParsedChars(){
	return chars;
    }

}
