// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.

//
//


package de.uka.ilkd.key.proof;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import de.uka.ilkd.key.logic.*;

import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.*;


public class MethodSpecTransformation{
    
    private MethodSpecTransformation() {}

    public static Term getTransformedHoare(Term hoare){
	//check for hoare-term here

	Term precondition = hoare.sub(0);
	Term postcondition = hoare.sub(1).sub(0);
	JavaBlock diamond = (hoare.sub(1)).javaBlock();
	HashMap preFunctions = new HashMap();
	HashMap postFunctions = new HashMap();
	HashMap logicVariableTerms = new HashMap();
	HashMap operators = new HashMap();
	HashMap neededTermsLeft = new HashMap();
	HashMap neededTermsRight = new HashMap();

	Vector postsubterms = getSubTerms(postcondition, new Vector(), preFunctions, postFunctions, logicVariableTerms);
	Term purePrecondition = purifyPrecondition(precondition, operators);
	Term prefix = getConjunction(preFunctions, postFunctions, postsubterms, 
				     neededTermsLeft, logicVariableTerms, 
				     operators);
	Term suffix = getConjunction(postFunctions, preFunctions, postsubterms, 
				     neededTermsRight, logicVariableTerms, 
				     operators);
	Term substPostcondition = substitute(postcondition, postsubterms, 
					     postFunctions, preFunctions,  
					     neededTermsRight, 
					     logicVariableTerms, operators);
	Vector remove = new Vector();
	Iterator cleanUp = postFunctions.keySet().iterator();
	while (cleanUp.hasNext()) {
	    Object next=cleanUp.next();
	    Object test = logicVariableTerms.get(postFunctions.get(next));
	    if (!neededTermsLeft.containsKey(test)){
		remove.add(next);
	    }
	}
	cleanUp = preFunctions.keySet().iterator();
	while (cleanUp.hasNext()) {
	    Object next=cleanUp.next();
	    Object test = logicVariableTerms.get(preFunctions.get(next));
	    if (!neededTermsRight.containsKey(test)){
		remove.add(next);
	    }
	}
	cleanUp = remove.iterator();
	while (cleanUp.hasNext()){
	    Object next = cleanUp.next();
	    postFunctions.remove(next);
	    preFunctions.remove(next);
	}
	QuantifiableVariable[] qvars = new QuantifiableVariable[postFunctions.size()+preFunctions.size()];
	int index=0;
	cleanUp = preFunctions.keySet().iterator();
	while (cleanUp.hasNext()){
	    qvars[index++]=(QuantifiableVariable)preFunctions.get(cleanUp.next());
	}
	cleanUp = postFunctions.keySet().iterator();
	while (cleanUp.hasNext()){
	    qvars[index++]=(QuantifiableVariable)postFunctions.get(cleanUp.next());
	}
	prefix = getConjunction(preFunctions, postFunctions, postsubterms, neededTermsLeft, logicVariableTerms, operators);
	prefix = removePres(prefix, operators);
	suffix = getConjunction(postFunctions, preFunctions, postsubterms, neededTermsRight, logicVariableTerms, operators);
	substPostcondition = substitute(postcondition, postsubterms, postFunctions, preFunctions,  neededTermsRight, logicVariableTerms, operators);


	prefix = TermFactory.DEFAULT.createJunctorTerm(Op.AND, purePrecondition, prefix);
        Term postImplication = TermFactory.DEFAULT.createJunctorTermAndSimplify(Op.AND, suffix, substPostcondition);
	hoare = TermFactory.DEFAULT.createJunctorTerm(Op.IMP, prefix, 
						      TermFactory.DEFAULT.createDiamondTerm(diamond , postImplication));
	
	if (qvars.length>0){
	    hoare = TermFactory.DEFAULT.createQuantifierTerm(Op.ALL, qvars, hoare);
	}
	return hoare;
    }
    
    static Term removePres(Term t, java.util.Map preMap) {
	Term[] subs=new Term[t.arity()];
	boolean changed = false;
	for (int i=0; i<subs.length; i++) {
	    subs[i]=removePres(t.sub(i), preMap);
	    if (subs[i]!=t.sub(i)) changed=true;
	}
	if (preMap.containsKey(t.op())) {
	    t = (Term)preMap.get(t.op());
	    if (t.op() instanceof AttributeOp) {
		subs = new Term[]{subs[0]};	    
	    }
	    changed = true;
	}
	return changed ? TermFactory.DEFAULT.createTerm(t.op(), subs,
							t.varsBoundHere(0), //%%%
							t.javaBlock(),
							null) : t;
    }
    
    static Term substitute(Term from, Vector canonicSubterms, HashMap mapA, 
			   HashMap mapB, HashMap notifyUsage, 
			   HashMap logicVariableTerms, 
			   HashMap ops){
	if (canonicSubterms.contains(from)) {
	    Term canonicFrom = (Term)canonicSubterms.get
		(canonicSubterms.indexOf(from));
	    if (mapA.containsKey(canonicFrom)) {
		notifyUsage.put(logicVariableTerms.get(mapA.get(canonicFrom)), 
				canonicFrom);
		return (Term)logicVariableTerms.get(mapA.get(canonicFrom));
	    } else if (mapB.containsKey(canonicFrom)) {
		notifyUsage.put(logicVariableTerms.get(mapB.get(canonicFrom)), 
				canonicFrom);
		return  (Term)logicVariableTerms.get(mapB.get(canonicFrom));
	    }
	}
	boolean changed=false;
	int arity = from.arity();
	Term subTerms[] = new Term[arity];
	ArrayList vars = new ArrayList();
	for ( int i = 0; i<arity; i++ ) {
	    subTerms[i] = substitute(from.sub(i), canonicSubterms, mapA, 
				     mapB, notifyUsage, logicVariableTerms, ops);
	    if (subTerms[i]!=from.sub(i)){
	        changed=true;
	    }
	}
	Operator top = from.op();
	if (ops.containsKey(top)) {
	    top = ((Term)ops.get(top)).op();
	    changed = true;
	}
	
	QuantifiableVariable v1[] = new QuantifiableVariable[0];
	v1 = (QuantifiableVariable[])(vars.toArray(v1));
	if (changed) { 
	    return TermFactory.DEFAULT.createTerm(top,subTerms,v1,
						  from.javaBlock(),null);
	}
	return from;

    }

    static Term getConjunction(HashMap rightSideFunctions, HashMap otherTerms, 
			       Vector allterms, HashMap needed, HashMap lvt, 
			       HashMap opMap){	
	Term result = null;

	if (rightSideFunctions.size()==0){
	    return TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
	}

	Iterator rsvIt = rightSideFunctions.keySet().iterator();
	Term rst = (Term)rsvIt.next();
	LogicVariable lv = (LogicVariable)rightSideFunctions.get(rst);
	needed.put(lvt.get(lv), null);
	Term[] subTerms = new Term[rst.arity()];
	for (int i=0;i<rst.arity();i++){
	    Term replace = substitute(rst.sub(i), allterms, rightSideFunctions, 
				      otherTerms, needed, lvt, opMap);
	    subTerms[i]=replace;
        }
	Operator opmapped = rst.op();
	Term changedMap 
	    = TermFactory.DEFAULT.createTerm(opmapped,subTerms, 
					     new QuantifiableVariable[0], 
					     rst.javaBlock(),null);
	result = TermFactory.DEFAULT.createEqualityTerm((Term)lvt.get(lv), 
							changedMap);
	while (rsvIt.hasNext()){
	    rst = (Term)rsvIt.next();
	    lv = (LogicVariable)rightSideFunctions.get(rst);
	    needed.put(lvt.get(lv), null);
	    subTerms = new Term[rst.arity()];
	    for (int i=0;i<rst.arity();i++){
	        Term replace = substitute(rst.sub(i), allterms, 
					  rightSideFunctions, otherTerms, 
					  needed, lvt, opMap);
		subTerms[i]=replace;
	    }
	    opmapped = rst.op();
	    changedMap = TermFactory.DEFAULT.createTerm(opmapped,subTerms, new QuantifiableVariable[0], rst.javaBlock(),null);
	    result = TermFactory.DEFAULT.createJunctorTerm(Op.AND, result, 
							   TermFactory.DEFAULT.createEqualityTerm((Term)lvt.get(lv), changedMap));
	}
	
	return result;
    }


    static Vector getSubTerms(Term t, Vector res, HashMap pre, HashMap post, HashMap lvt){
	if (!res.contains(t)){
  	    res.add(t);
	    
	    if (t.sort()!=Sort.FORMULA){
		LogicVariable subst = new LogicVariable(new Name("_tauV"+res.size()), t.sort());
		lvt.put(subst, TermFactory.DEFAULT.createVariableTerm(subst));
		
		if (t.op() instanceof Function){
		    if (((Function)t.op()).name().toString().indexOf("@pre")>-1){
			pre.put(t, subst);
		    } else {
			post.put(t, subst);
		    }  
		} else{
		    post.put(t, subst);
		}
	    }
	    
	    for (int i=0; i<t.arity(); i++){
	        res=getSubTerms(t.sub(i), res, pre, post, lvt);
	    }

	}
	return res;
    }

    static Term purifyPrecondition(Term pre, HashMap op){
	Term[] subs;
	if (pre.op().name().toString().equals("all")){
	    if (pre.sub(0).op() instanceof de.uka.ilkd.key.logic.op.Equality){
		if (pre.sub(0).sub(0).op() instanceof Function){
	            if (((Function)pre.sub(0).sub(0).op()).name().toString().indexOf("@pre")>-1){
		        op.put(pre.sub(0).sub(0).op(), pre.sub(0).sub(1));
			return TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
		    }
		}
	    }
	}
	
	boolean changed=false;
	int arity = pre.arity();
	Term subTerms[] = new Term[arity];
	// Stupid hack because of another stupid hack in TermFactory
	ArrayList vars = new ArrayList();
	for ( int i = 0; i<arity; i++ ) {
	    //ArrayOfQuantifiableVariable vbh = pre.varsBoundHere(i);
	    //for( int j=0; j<vbh.size(); j++ ) {
		// THERE MUST NOT OCCUR SCHEMAVARIABLES!
		//	vars.add(map((SchemaVariable)
		//	     (vbh.getQuantifiableVariable(j))));
	    //    }
	    subTerms[i] = purifyPrecondition(pre.sub(i), op);
	    if (subTerms[i]!=pre.sub(i)){
	        changed=true;
	    }
	}
	QuantifiableVariable v1[] = new QuantifiableVariable[0];
	v1 = (QuantifiableVariable[])(vars.toArray(v1));
	if (changed) { 
	    if (pre.op().name().toString().equals("and")) {
		if (subTerms[1].op()==Op.TRUE) {
		    return subTerms[0];
		}
	    }
	    return TermFactory.DEFAULT.createTerm(pre.op(),subTerms,v1,
						  pre.javaBlock(),null);
	}
	return pre;
    }
}
