// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.proof;

import de.uka.ilkd.key.proof.mgt.ProofEnvironment;
import de.uka.ilkd.key.proof.mgt.ProofStatus;

public class ProofList {

    private Proof[] proofs;
    private String name;
    private ProofStatus proofStatus = null;

    public ProofList(Proof[] proofs, String name) {
	this.proofs=proofs;
	this.name=name;
    }

    public ProofList(Proof p) {
	this.proofs = new Proof[]{p};
	this.name="";
    }

    public ProofList(Proof p, String name) {
	this.proofs = new Proof[]{p};
	this.name=name;
    }

    public Proof[] getProofs() {
	return proofs;
    }

    public void setProofEnv(ProofEnvironment env) {
	for (int i=0; i<proofs.length; i++) {
	    proofs[i].setProofEnv(env);
	}
    }

    public Proof get(int i) {
	return proofs[i];
    }

    public int size() {
	return proofs.length;
    }

    public String description() {
	return name;
    }

    public String toString() {	
	return name;
    }

    public void updateProofStatus() {
	Proof p = null;
	boolean open = false;
	boolean closedButLemmasLeft = false;
	for (int i=0; i<proofs.length; i++) {
	    p =  proofs[i];
	    p.mgt().updateProofStatus();
	    ProofStatus ps = p.mgt().getStatus();
	    if (ps.getProofOpen()) open = true;
	    if (ps.getProofClosedButLemmasLeft()) closedButLemmasLeft = true;
	}
	proofStatus = new ProofStatus(!open && !closedButLemmasLeft, closedButLemmasLeft && !open, open);
    }
    
    
    public ProofStatus getStatus() {
	if (proofStatus==null) updateProofStatus();
	return proofStatus;	
    }

    public boolean equals (Object o) {
	if (!(o instanceof ProofList)) return false;
	ProofList cmp = (ProofList) o;
	for (int i=0; i<cmp.size(); i++) {
	    if (cmp.get(i)!=get(i)) return false;
	}
	return true;
    }
    
    public int hashCode(){
    	int result = 17;
    	for (int i=0; i < size(); i++){
    		result = 37 * result + get(i).hashCode();
    	}
    	return result;
    }

}
