// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.proof;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.rule.*;
import de.uka.ilkd.key.proof.mgt.*;
import de.uka.ilkd.key.casetool.ReprModelMethod;
import de.uka.ilkd.key.java.Statement;

public class RuleTreeModel extends DefaultTreeModel {
    
    protected Goal goal;
    protected MutableTreeNode axiomTacletRoot 
	= new DefaultMutableTreeNode("Axiom Taclets");
    protected MutableTreeNode builtInRoot 
	= new DefaultMutableTreeNode("Built-In");
    protected MutableTreeNode lemmaRoot = new DefaultMutableTreeNode("Lemmas");
    protected MutableTreeNode methcontrRoot 
	= new DefaultMutableTreeNode("Method Contracts");
    protected MutableTreeNode contrRoot 
	= new DefaultMutableTreeNode("DL Contracts");
    protected MutableTreeNode proveableTacletsRoot 
	= new DefaultMutableTreeNode("Non-Axiom Taclets");

    public RuleTreeModel(Goal g) {
	super(new DefaultMutableTreeNode("Rule Base"));
	this.goal = g;
	insertNodeInto(builtInRoot, (MutableTreeNode) getRoot(), 0);
	insertNodeInto(axiomTacletRoot, (MutableTreeNode) getRoot(), 0);
	insertNodeInto(lemmaRoot, (MutableTreeNode) getRoot(), 0);
	insertNodeInto(proveableTacletsRoot, lemmaRoot, 0);
	insertNodeInto(methcontrRoot, lemmaRoot, 0);
	insertNodeInto(contrRoot, lemmaRoot, 0);
	if (g!=null) rulesForGoal(g);
    }

    private void rulesForGoal(Goal g) {
	IteratorOfBuiltInRule rit = getBuiltInIndex().rules().iterator();
	while (rit.hasNext()) {
	    BuiltInRule br = rit.next();
	    insertNodeInto(new DefaultMutableTreeNode(br), builtInRoot, 0);
	}
	SetOfNoPosTacletApp apps = getTacletIndex().allNoPosTacletApps();
	IteratorOfNoPosTacletApp it = apps.iterator();
	while (it.hasNext()) {
	    NoPosTacletApp app = it.next();
	    RuleJustification just = mgt().getJustification(app);
	    if (just==null) continue; // do not break system because of this
	    if (just.isAxiomJustification()) {
		insertNodeInto(new DefaultMutableTreeNode
			       (app.taclet().displayName()), 
			       axiomTacletRoot, 0);
	    } else if (just instanceof RuleJustificationByMethodSpec) {
		RuleJustificationByMethodSpec specJust 
		    = (RuleJustificationByMethodSpec) just;
		Contract ct = specJust.getMethodContract();
		if (ct.getObjectOfContract() instanceof ReprModelMethod) {
		    insertMethodNodeInto
			((ReprModelMethod)ct.getObjectOfContract(), ct);
		} else if (ct.getObjectOfContract() instanceof NoPosTacletApp) {
		    insertNodeInto
			(new DefaultMutableTreeNode(app.taclet()),
			 proveableTacletsRoot, 0);
		} else {
		    insertNodeInto
			(new DefaultMutableTreeNode(ct), contrRoot, 0);
		}
	    }
	}
    }

    private void insertMethodNodeInto(ReprModelMethod meth, Contract ct) {
	MutableTreeNode methNode = null;
	for (int i=0; i<methcontrRoot.getChildCount(); i++) {
	    if (((DefaultMutableTreeNode)
		 methcontrRoot.getChildAt(i)).getUserObject()==meth) {
		methNode = (MutableTreeNode) methcontrRoot.getChildAt(i);
	    }
	}
	if (methNode==null) {
	    methNode = new DefaultMutableTreeNode(meth);
	    insertNodeInto(methNode, methcontrRoot,0);
	}
	insertNodeInto(new DefaultMutableTreeNode(ct), methNode, 0);
    }

    private TacletIndex getTacletIndex() {
	return goal.ruleAppIndex().tacletIndex();
    }

    private BuiltInRuleIndex getBuiltInIndex() {
	RuleAppIndex ri =  goal.ruleAppIndex();
	return ri.builtInRuleAppIndex().builtInRuleIndex();
    }

    public ProofCorrectnessMgt mgt() {
	return goal.node().proof().mgt();
    }

    public void setSelectedGoal(Goal g) {
	goal=g;
    }
    

}
