// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.proof;

import de.uka.ilkd.key.logic.*;
import java.util.HashMap;
import java.util.Map;
import java.util.HashSet;
import java.util.Set;
import java.util.ArrayList;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.visitor.ProgVarReplaceVisitor;
import de.uka.ilkd.key.java.declaration.LocalVariableDeclaration;
import de.uka.ilkd.key.java.declaration.ArrayOfVariableSpecification;

public class TermSVReplacer {

    Map schemaVariables;
    Set locallyDeclaredVars = new HashSet();
    Term from, to;

    TermFactory fact;

    public TermSVReplacer(Map map) {
	fact = TermFactory.DEFAULT;
	schemaVariables = map;
    }

    private void createSV(ProgramVariable pv) {

	SortedSchemaVariable ssv = (SortedSchemaVariable)
	    SchemaVariableFactory.createProgramSV
	    (new ProgramElementName("#"+pv),
	     ProgramSVSort.VARIABLE, false);
	schemaVariables.put(pv, ssv);
    }

    public Set getLocallyDeclaredVars() {
	return locallyDeclaredVars;
    }
    
    public Term inst(Term t) {
	Term result = null;
	if (t.op() instanceof ProgramVariable 
	    && !((ProgramVariable)t.op()).isMember()){ 
	    result = t;
	    if (!schemaVariables.containsKey(t.op())) {
		createSV((ProgramVariable)t.op());
	    } 
	    result = TermFactory.DEFAULT.createVariableTerm
		((SchemaVariable) schemaVariables.get(t.op()));

	} else {
	    Term subTerms[] = new Term[t.arity()];
	    ArrayList vars = new ArrayList();
	    for ( int i = 0; i<t.arity(); i++ ) {
		ArrayOfQuantifiableVariable vbh = t.varsBoundHere(i);
		for( int j=0; j<vbh.size(); j++ ) {
		    vars.add(vbh.getQuantifiableVariable(j));
		}
		subTerms[i] = inst(t.sub(i));
	    }
	    QuantifiableVariable v1[] = new QuantifiableVariable[0];
	    v1 = (QuantifiableVariable[])(vars.toArray(v1));
	    JavaBlock jb = t.javaBlock();
	    if (!jb.isEmpty()) {
		jb = JavaBlock.createJavaBlock
		    ((StatementBlock)inst((Statement)t.javaBlock().program()));
		if (jb.equals(t.javaBlock())) jb = t.javaBlock();
	    }
	    result = fact.createTerm(t.op(),subTerms,v1,jb,null);	   
	}
	return result;
    }

    public Statement inst(Statement sta) {
	ProgVarReplaceVisitor pvrepl = new ProgVarReplaceVisitor
	    (sta, schemaVariables) {

		protected void walk(ProgramElement node) {
		    if (node instanceof LocalVariableDeclaration 
			&& replaceallbynew) {
			LocalVariableDeclaration vd
			    = (LocalVariableDeclaration)node;
			ArrayOfVariableSpecification vspecs
			    =vd.getVariableSpecifications();
			for (int i=0; i<vspecs.size(); i++) {
			    ProgramVariable pv 
				= (ProgramVariable) 
				vspecs.getVariableSpecification(i)
				.getProgramVariable();
			    if (!replaceMap.containsKey(pv)){
				createSV(pv);
			    }
			    locallyDeclaredVars.add(pv);
			}
		    }
		    super.walk(node);
		}

		public void performActionOnProgramVariable(ProgramVariable pv) {
		    if (!pv.isMember() && !replaceMap.containsKey(pv)) {
			createSV(pv);
		    }
		    super.performActionOnProgramVariable(pv);
		}		
	    };

	pvrepl.start();
	return (Statement) pvrepl.result();
    }

}

