// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/** tests the TacletIndex class.*/
package de.uka.ilkd.key.proof;


import junit.framework.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.rule.*;


public class TestTacletIndex extends TestCase{   

    Taclet ruleRewriteNonH1H2; 
    Taclet ruleNoFindNonH1H2H3;
    Taclet ruleAntecH1;
    Taclet ruleSucc;
    Taclet ruleMisMatch;
    Taclet notfreeconflict;
    
    Heuristic h1;
    Heuristic h2;
    Heuristic h3;

    TacletIndex variante_one;
    

    public TestTacletIndex(String name) {
	super(name);
    }

    private Taclet taclet(String name) {
	return TacletForTests.getTaclet(name).taclet();
    }
    
    public void setUp() {
	TacletForTests.parse(System.getProperty("key.home")+
           java.io.File.separator+"system"+java.io.File.separator+
           "de/uka/ilkd/key/proof/ruleForTestTacletIndex.taclet");

	h1 = (Heuristic)TacletForTests.getHeuristics().lookup(new Name("h1"));
	h2 = (Heuristic)TacletForTests.getHeuristics().lookup(new Name("h2"));
	h3 = (Heuristic)TacletForTests.getHeuristics().lookup(new Name("h3"));

	ruleRewriteNonH1H2 = taclet("rewrite_noninteractive_h1_h2");
	ruleNoFindNonH1H2H3 = taclet("nofind_noninteractive_h1_h2_h3");
	ruleAntecH1 = taclet("rule_antec_h1");
	ruleSucc = taclet("rule_succ");
	ruleMisMatch = taclet ("antec_mismatch");
	notfreeconflict = taclet("not_free_conflict");
	

	variante_one = new TacletIndex();
	variante_one.add(ruleRewriteNonH1H2);
	variante_one.add(ruleNoFindNonH1H2H3);
	variante_one.add(ruleAntecH1);
	variante_one.add(ruleSucc);


    }


    private boolean isRuleIn(ListOfNoPosTacletApp l, Taclet rule) {
	IteratorOfNoPosTacletApp it=l.iterator();
	while(it.hasNext()) {
	    if (it.next().taclet()==rule) return true;
	}
	return false;
    }

    private boolean isRuleIn(ListOfTacletApp l, Taclet rule) {
	IteratorOfTacletApp it=l.iterator();
	while(it.hasNext()) {
	    if (it.next().taclet()==rule) return true;
	}
	return false;
    }


    /**
     * test disabled. Since 0.632 "noninteractive" is disabled
     */
    public void disabled_testNonInteractiveIsShownOnlyIfHeuristicIsMissed() {
	Term term_p1 = TacletForTests.parseTerm("p(one, zero)");	
	ListOfHeuristic listofHeuristic=SLListOfHeuristic.EMPTY_LIST;
        listofHeuristic=listofHeuristic.prepend(h3);
  	assertTrue("Noninteractive antecrule is not in list, but none of its"+
		   "heuristics is active.",
		   isRuleIn(variante_one.getAntecedentTaclet(term_p1, Constraint.BOTTOM,
			    new IHTacletFilter (true, listofHeuristic),
   			    null, Constraint.BOTTOM),ruleRewriteNonH1H2)); 

 	assertTrue("Noninteractive antecrule is in list, but one of its "+
		   "heuristics is active.",
		   !isRuleIn(variante_one.getAntecedentTaclet(term_p1, Constraint.BOTTOM,
		             new IHTacletFilter (true, listofHeuristic.prepend(h1)),
			     null, Constraint.BOTTOM),ruleRewriteNonH1H2));

  

  	assertTrue("Noninteractive nofindrule is not in list, but none of its "+
		   "heuristics is active.",
		   isRuleIn(variante_one.getNoFindTaclet(new IHTacletFilter (true, SLListOfHeuristic.EMPTY_LIST),
   			    null, Constraint.BOTTOM),ruleNoFindNonH1H2H3));  

 	assertTrue("Noninteractive nofindrule is in list, but one of its "+
		   "heuristics is active.",
		   !isRuleIn(variante_one.getNoFindTaclet(new IHTacletFilter (true, listofHeuristic), null, Constraint.BOTTOM),ruleNoFindNonH1H2H3));
	
    }


    public void testShownIfHeuristicFits() {
	ListOfHeuristic listofHeuristic=SLListOfHeuristic.EMPTY_LIST;
	listofHeuristic=listofHeuristic.prepend(h3).prepend(h2);

	Term term_p1 = TacletForTests.parseTerm("p(one, zero)");	

  	assertTrue("ruleSucc has no heuristics, but is"+
		   " not in succ list.",
		   isRuleIn(variante_one.getSuccedentTaclet(term_p1, Constraint.BOTTOM,
		            new IHTacletFilter (true, listofHeuristic),
		            null, Constraint.BOTTOM), ruleSucc)); 

	PosInOccurrence pos = new PosInOccurrence
	    ( new ConstrainedFormula ( term_p1, Constraint.BOTTOM ),
	      PosInTerm.TOP_LEVEL, (Sequent)null );
  	assertTrue("ruleSucc has no heuristics, but is"+
		   " in rewrite list.",
		   !isRuleIn(variante_one.getRewriteTaclet(pos, Constraint.BOTTOM,
		             new IHTacletFilter (true, listofHeuristic),
		             null, Constraint.BOTTOM),ruleSucc)); 


  	assertTrue("ruleSucc has no heuristics, but is"+
		   " in heuristic succ list.",
		   !isRuleIn(variante_one.getSuccedentTaclet(term_p1, Constraint.BOTTOM,
		             new IHTacletFilter (false, listofHeuristic),
   			     null, Constraint.BOTTOM),ruleSucc)); 

  	assertTrue("ruleSucc has no heuristics, but is"+
		   " in heuristic of nofind list.",
		   !isRuleIn(variante_one.getNoFindTaclet(new IHTacletFilter (false,
   			     listofHeuristic), null, Constraint.BOTTOM),ruleSucc));
    }

    public void testNoMatchingFindRule() {
	ListOfHeuristic listofHeuristic=SLListOfHeuristic.EMPTY_LIST;

	Term term_p2 = TacletForTests.parseTerm("all z:nat.p(z, one)").sub(0);
	
 	assertTrue("rule matched, but no match possible",
		   !isRuleIn(variante_one.getAntecedentTaclet(term_p2, Constraint.BOTTOM,
		                                              new IHTacletFilter (true, listofHeuristic),
							      null, Constraint.BOTTOM), 
			     ruleRewriteNonH1H2));
 

	listofHeuristic=listofHeuristic.prepend(h3).prepend(h2).prepend(h1);

 	assertTrue("ruleSucc matched but matching not possible",
		   !isRuleIn(variante_one.getSuccedentTaclet(term_p2, Constraint.BOTTOM,
		             new IHTacletFilter (true, listofHeuristic),
			     null, Constraint.BOTTOM),ruleSucc));		
    }

    public void testMatchConflictOccurs() {
	TacletIndex ruleIdx=new TacletIndex();
	ruleIdx.add(ruleRewriteNonH1H2);
	ruleIdx.add(ruleNoFindNonH1H2H3);
	ruleIdx.add(ruleAntecH1);
	ruleIdx.add(ruleSucc);
	ruleIdx.add(ruleMisMatch);

	Term term_p4 = TacletForTests.parseTerm("p(zero, one)");

	ListOfHeuristic listofHeuristic=SLListOfHeuristic.EMPTY_LIST;
	
 	assertTrue("rule matched, but no match possible",
		   !isRuleIn(ruleIdx.getAntecedentTaclet(term_p4, Constraint.BOTTOM,
		             new IHTacletFilter (true, listofHeuristic),
   			     null, Constraint.BOTTOM),ruleMisMatch));
 
    }

    public void testNotFreeInYConflict() {
	TacletIndex ruleIdx=new TacletIndex();
	ruleIdx.add(notfreeconflict);

	Term term_p5 = TacletForTests.parseTerm("all z:nat.p(f(z), z)");
	ConstrainedFormula cfma_p5 = new ConstrainedFormula(term_p5);
	Sequent seq_p5 = Sequent.createAnteSequent
	    (Semisequent.EMPTY_SEMISEQUENT.insertFirst(cfma_p5).semisequent());
	PosInOccurrence pio_p5 = new PosInOccurrence
	    (cfma_p5, PosInTerm.TOP_LEVEL, seq_p5);
        RuleAppIndex appIdx = createGoalFor(seq_p5, ruleIdx);
		  
	assertTrue("No rule should match",
		   !isRuleIn(appIdx.getTacletAppAt
			     (TacletFilter.TRUE, pio_p5,
			     null, Constraint.BOTTOM),
			     notfreeconflict));

	Term term_p6 = TacletForTests.
	    parseTerm("all z:nat.p(zero, z)");

	ConstrainedFormula cfma_p6 = new ConstrainedFormula(term_p6);
	Sequent seq_p6 = Sequent.createAnteSequent
	    (Semisequent.EMPTY_SEMISEQUENT.insertFirst(cfma_p6).semisequent());
	PosInOccurrence pio_p6 = new PosInOccurrence
	    (cfma_p6, PosInTerm.TOP_LEVEL, seq_p6);
	appIdx = createGoalFor(seq_p6, ruleIdx);

	assertTrue("One rule should match", isRuleIn
		   (appIdx.
		    getTacletAppAt(TacletFilter.TRUE, pio_p6,
		                   null, Constraint.BOTTOM),
		    notfreeconflict));
	    
    }

    private RuleAppIndex createGoalFor(Sequent seq_p5, TacletIndex ruleIdx) {
        final Node node_p5 =
            new Node (new Proof (), seq_p5);
        final BuiltInRuleAppIndex builtinIdx =
            new BuiltInRuleAppIndex (new BuiltInRuleIndex ());
        final Goal goal_p5 =
            new Goal (node_p5, new RuleAppIndex (ruleIdx, builtinIdx));
	return goal_p5.ruleAppIndex ();
    }


}
