// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.visitor.CreatingASTVisitor;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.JavaProgramElement;
import de.uka.ilkd.key.java.SourceElement;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.java.expression.Assignment;
import de.uka.ilkd.key.proof.Node;
import de.uka.ilkd.key.proof.IteratorOfNode;
import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.proof.ProofList;
import de.uka.ilkd.key.proof.mgt.Contract;
import de.uka.ilkd.key.proof.mgt.ProofEnvironment;
import de.uka.ilkd.key.logic.IteratorOfConstrainedFormula;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.Sequent;
import de.uka.ilkd.key.logic.Semisequent;
import de.uka.ilkd.key.logic.JavaBlock;
import de.uka.ilkd.key.collection.SetOfString;

import java.util.Vector;
import java.util.List;
import java.util.Enumeration;


public class NonInterferencePO implements ProofOblInput {


    ProofList po;
    InitConfig initConfig;
    Proof proof1,proof2;
    
    public NonInterferencePO(ProofEnvironment env, Proof p1, Proof p2) {
        initConfig = env.getInitConfig();
        if(p1 == null || p2 == null)throw new IllegalStateException(
            "Proofs are not initialised");
        proof1 = p1;
        proof2 = p2;
    }
    
    

    /** returns the proof obligation term as result of the proof obligation
     * input. If there is still no input available because nothing has
     * been read yet null is returned.
     */
    public ProofList getPO() {
        return po;
    }

    /** starts reading the input and modifies the InitConfig of this 
     * object with respect to the given modification
     * strategy. 
     */
    public void read(ModStrategy mod) throws ProofInputException {
        po = new ProofList(
	    new Proof[] { new Proof( name(),
					getPOTerm(),
					"HOLA!",
					initConfig.createTacletIndex(),
					initConfig.createBuiltInRuleIndex(),
					initConfig.getActivatedChoices() ) }, 
	     name());
    }

    public void readProblem(ModStrategy mod) throws ProofInputException {
        read(mod);
    }

    /* Serves as an utility-method for getPOTerm(). It takes a Node
    (from a proof), traverses the tree of nodes, and collects in v all
    nodes where symbolic modification has been performed. To reduce the
    number of collected elements only nodes with updates will be
    collected if the parameter onlyUpdates is true. */
    private void getSymExecNodes(Node n, boolean onlyUpdates, Vector v) {
        if(v == null) throw new IllegalStateException(
            "The second parameter of getSMNodes is not initialised");
        if(n == null) return;
        if(n.getActiveStatement()!=null) {
        /*This is the case if symbolic modification has been performed*/
            if(onlyUpdates) {
             String ruleName = n.getAppliedRuleApp().rule().name().toString();
             if(ruleName.startsWith("assignment_"))
                v.add(n);
            } else {
            v.add(n);
            }
        }
        for(IteratorOfNode ni=n.childrenIterator();ni.hasNext();) {
            getSymExecNodes (ni.next(), onlyUpdates, v);
        }
    }

/*
Serves the getPOTerm() method.
@param a gamma1,gamma2,..., gammaN => <p1> phi
@param b delta1,delta2,..., deltaM => <p2> psi
@return gamma1,gamma2,...,gammaN,delta1,delta2,...,deltaM => <p2'> gamma1,gamma2,...,gammaN
where p2' is the first assignment statement of p2.
*/
    private Term nonInterfCondition(Node a, Node b) throws ProofInputException {
        Sequent sa = a.sequent();
        Sequent sb = b.sequent();

        Term gamma = TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
//collect gammas
        for(IteratorOfConstrainedFormula icf = sa.antecedent().iterator();
            icf.hasNext();) {
            gamma = TermFactory.DEFAULT.createJunctorTerm(Op.AND, gamma,
                                        icf.next().formula());
            }

//Save gamma for the succedent of the result
// the result will be:  resAnte => resSucc
        Term resAnte = gamma;

//collect deltas
        for(IteratorOfConstrainedFormula icf = sb.antecedent().iterator();
            icf.hasNext();) {
            resAnte = TermFactory.DEFAULT.createJunctorTerm(Op.AND, resAnte,
                                        icf.next().formula());
            }
/* now
    gamma == gamma1 and gamma2 and... and gammaN
    resAnte == gamma and delta1 and delta2 and ... and deltaM
*/
        Term bSucc = sb.succedent().getFirst().formula();
        while (bSucc.isSimultaneousUpdateTerm())  {
            bSucc = bSucc.sub(bSucc.arity()-1); // skip update
        }
        if(bSucc.op() != Op.DIA)
            throw new ProofInputException("Diamondoperator expected at top of "+
                                           bSucc);

// the result will be:  resAnte => resSucc
//the following part looks for the first Assigment in the JavaBlock
//and constructs a new JavaBlock which contains only this assigment
//and puts it into a BoxOperator.
/*
        JavaBlock p2 = bSucc.javaBlock();
        SourceElement srcElem = p2.program().getFirstElement();
        while(srcElem!=null && !(srcElem instanceof Assignment))
            srcElem = srcElem.getFirstElement();
        if(srcElem == null)
            throw new RuntimeException("Assignment not found in JavaBlock.");
        JavaBlock p2prime = JavaBlock.createJavaBlock(
            new StatementBlock((Assignment)srcElem));
        Term resSucc = TermFactory.DEFAULT.createBoxTerm(p2prime,gamma);
*/

        JavaProgramElement srcElem = bSucc.javaBlock().program();
        FirstStatementExtractionVisitor v = 
            new FirstStatementExtractionVisitor(srcElem);
        v.start();
        JavaBlock p2prime = JavaBlock.createJavaBlock(
            (StatementBlock)v.result());
        Term resSucc = TermFactory.DEFAULT.createBoxTerm(p2prime,gamma);



        return TermFactory.DEFAULT.createJunctorTerm(Op.IMP, resAnte, resSucc);
    }



/* Creates the necessary term for the NonInterference proof by a crossproduct.
Important Subroutines are getSMNodes and combineTerms.
*/
    private Term getPOTerm() throws ProofInputException {

        Vector nodes1 = new Vector();
        Vector nodes2 = new Vector();
        /*collect all nodes where Symbolic Manipulation was performed*/
        getSymExecNodes (proof1.root(),false,nodes1);
        getSymExecNodes (proof2.root(),true, nodes2);


        Term result = TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);

        //create the crossproduct of the two lists
        for( Enumeration i=nodes1.elements(); i.hasMoreElements();) {
            Node a = (Node)i.nextElement();
            for( Enumeration j=nodes2.elements(); j.hasMoreElements();) {
                Node b = (Node)j.nextElement();
                result = TermFactory.DEFAULT.createJunctorTerm(Op.AND,
                    result, nonInterfCondition(a,b));
            }
        }
        //take the list of terms and create a single term using and conjunction.
        return result;
    }
    

    public boolean askUserForEnvironment() {
        return true;
    }

    /** reads the Java model that belongs to this input.
     */
    public String readJava() throws ProofInputException {
        return null;
    }

    /** returns the path to the Java model.
     */
    public String getJavaPath() throws ProofInputException {
        return null;
    }

    /** set the initial configuration used to read an input. It may become
     * modified during reading depending on the modification strategy used
     * for reading.
     */
    public void setConfig(InitConfig i) {
    }

    public void readSpecs() {
    }

    public void readActivatedChoices() throws ProofInputException {
    }

    public SetOfString getActivatedChoices() throws ProofInputException {
        return null;
    
    }
    
    /** reads the include section and returns an Includes object.  
     */
    public Includes readIncludes() throws ProofInputException {
        return new Includes();
    }
    
    /** returns the name of the proof obligation input.
     */
    public String name() {
        return "Non-Interference of ... and ...";
    }

    public Object getObjectOfContract() {
        return null;
    }

    public boolean fitsToContract(Contract ct) {
        return false;
    }

    public void startProtocol() {
	// do nothing
    }
    
    
    public class FirstStatementExtractionVisitor extends CreatingASTVisitor {

        private ProgramElement result=null;
        
        public FirstStatementExtractionVisitor(ProgramElement root) {
	    super(root);
        }

        /** starts the walker*/
        public void start() {	
	    stack.push(new ExtList());		
	    walk(root());
	    ExtList el=(ExtList)stack.peek();
	    int i=0;
	    while (!(el.get(i) instanceof ProgramElement)) {
	        i++;
	    }
	    result=(ProgramElement) ((ExtList)stack.peek()).get(i);
        }

        public ProgramElement result() { 	
	    return result;
        }

        public void performActionOnStatementBlock(StatementBlock x) {
	    addChild(new StatementBlock((Statement)x.getChildAt(0)));
	    changed();
        }

    }
    

}
