// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.proof.mgt;

import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.logic.op.Modality;
import de.uka.ilkd.key.rule.Taclet;
import java.util.LinkedList;
import java.util.List;
import de.uka.ilkd.key.casetool.ReprModelMethod;
import de.uka.ilkd.key.proof.init.ProofOblInput;
import de.uka.ilkd.key.proof.init.OCLPostPO;

public class OCLMethodContract extends AbstractMethodContract{
    
    private String pre;
    private String post;
    private String typeDescr;

    public OCLMethodContract(ReprModelMethod meth, String pre, String post, 
			     Modality modality, String typeDescr) {
	super(meth);
	this.pre=pre;
	this.post=post;
	this.modality = modality;
	this.typeDescr=typeDescr;
    }

    public OCLMethodContract(ReprModelMethod meth, PrePostPair ppp, 
			     Modality modality, String typeDescr) {
	this(meth, ppp.getPre(), ppp.getPost(), modality, typeDescr);
    }


    protected AbstractMethodLemmaFactory getMethodLemmaFactory() {
	return new OCLMethodLemmaFactory(meth, env.getInitConfig(), 
					 pre, post, typeDescr);
    }


    public String getPreText() {
	return pre;
    }

    public String getPostText() {
	return post;
    }

    public ProofOblInput getProofOblInput() {
	return new OCLPostPO("EnsuresPostCondition"
			     +ContractUtil.getPOSuffix(getModality()), 
			     getModelMethod(), 
			     getPreText(), getPostText(), null);
    }

    public String toString() {
	return "[OCL] context " + 
	    getModelMethod().getContainingClass().getClassName() +
	    "::" + getModelMethod().getCallSignature() + "\n" +
	    "pre: " + getPreText() + "\n" +
	    "post: " + getPostText() + "\n" +
	    "modifies: " + getModelMethod().getMyModifClause();
    }

    public boolean equals(Object cmp) {
	if (!(cmp instanceof OCLMethodContract)) return false;
	MethodContract mc = (MethodContract)cmp;
	return (getModelMethod().equals(mc.getModelMethod()) 
		&& getPreText().equals(mc.getPreText())
		&& getModality().equals(mc.getModality())
		&& getPostText().equals(mc.getPostText()));

    }

    public int hashCode() {
	int result = 0;
	result = 37 * result + getModelMethod().hashCode(); 
	result = 37 * result + getPreText().hashCode();
	result = 37 * result + getPostText().hashCode();
	result = 37 * result + getModality().hashCode();
	return result;
    }

}
