// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof.mgt;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.abstraction.Type;
import de.uka.ilkd.key.java.declaration.MethodDeclaration;
import de.uka.ilkd.key.java.declaration.TypeDeclaration;
import de.uka.ilkd.key.logic.op.*;
import java.util.*;

public class OCLSpecExtractor implements SpecExtractor{

    private Services serv;
    private String rootDir;

    public OCLSpecExtractor(Services serv, String rootDir) {
	this.serv=serv;
	this.rootDir=rootDir;
    }


    private boolean containsOCLSpec(String s) {
	return s.indexOf("@postconditions")>=0; //%%
    }

    public void addSpecsFromClass(KeYJavaType td, ContractSet result) {
	JavaInfo ji = serv.getJavaInfo();
	IteratorOfProgramMethod methIt 
	    = ji.getAllProgramMethods(td).iterator();
	while (methIt.hasNext()) {
	    ProgramMethod pm = methIt.next();
	    MethodDeclaration md = pm.getMethodDeclaration();		
	    Comment[] comments = md.getComments();
	    if (comments!=null && pm.getContainedInKeYJavaType()==td) {
		for(int i=0;  i<comments.length; i++) {
		    if (containsOCLSpec(comments[i].getText())) {
			JavaModelMethod jmm = new JavaModelMethod(pm, rootDir);
			PrePostPair prepost = ContractUtil.getPrePost(jmm);
			MethodContract mc = new OCLMethodContract
			    (jmm, prepost, Op.DIA, "ensuresPost");
			result.add(mc);
		    }
		}
	    }
	}
    }


    public ContractSet getSpecsFromAllClasses() {
	JavaInfo ji = serv.getJavaInfo();
	ContractSet result = new ContractSet();
	Iterator it = ji.getAllKeYJavaTypes().iterator();
	while (it.hasNext()) {
	    addSpecsFromClass((KeYJavaType) it.next(), result);
	}	
	return result;
    }

    public ContractSet getSpecsFromLibraries() {
	JavaInfo ji = serv.getJavaInfo();
	ContractSet result = new ContractSet();
	Iterator it = ji.getAllKeYJavaTypes().iterator();
	while (it.hasNext()) {
	    KeYJavaType kjt = (KeYJavaType) it.next();
	    Type td = kjt.getJavaType();
	    if (td instanceof TypeDeclaration &&
		((TypeDeclaration)td).isLibraryClass()) {
		addSpecsFromClass(kjt, result);
	    }
	}	
	return result;
    }


}
