// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.rule;

import java.util.*;

import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.*;

/**
 * checks if terms and sequents put into this class contain only bound
 * variables that occur only once in these terms
 */
public class BoundUniquenessChecker {


    private HashSet boundVars=new HashSet();
    private ListOfTerm terms=SLListOfTerm.EMPTY_LIST;

    public BoundUniquenessChecker() {
    }

    public BoundUniquenessChecker(Term t, Sequent seq) {
	addTerm(t);
	addSequent(seq);
    }


    public BoundUniquenessChecker(Sequent seq) {
	addSequent(seq);
    }

    public void addTerm(Term t) {
	terms=terms.append(t);	
    }

    public void addSequent(Sequent seq) {
	IteratorOfConstrainedFormula it=seq.iterator();
	while (it.hasNext()) {
	    terms=terms.append(it.next().formula());	
	}
    }

    //recursive helper
    private boolean correct(Term t) {
	/* Note that a term can bound a variable in several
	 * subterms. */

	for (int i=0; i<t.arity(); i++) {
	    for (int j=0; j<t.varsBoundHere(i).size(); j++) {
		QuantifiableVariable qv
		    = t.varsBoundHere(i).getQuantifiableVariable(j);
		if (boundVars.contains(qv)) {
		    return false;
		}
	    }
	}

	for (int i = 0; i < t.arity(); ++i) {
	    for (int j = 0; j < t.varsBoundHere(i).size(); ++j) {
		QuantifiableVariable qv
		    = t.varsBoundHere(i).getQuantifiableVariable(j);
		boundVars.add(qv);
	    }
	}

	for (int i = 0; i < t.arity(); ++i) {
	    if (!correct(t.sub(i))) {
		return false;
	    }
	}

	return true;
    }


    /**
     * returns true iff there is no quantifiable variable in the terms and 
     * sequent
     * that have been put to this checker before bound twice.
     */
    public boolean correct() {
	IteratorOfTerm it=terms.iterator();
	while (it.hasNext()) {
	    Term t=it.next();
	    if (!correct(t)) {
		return false;
	    }	    
	}
	return true;
    }

}
