// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/** class contains a pair of SchemaVariables, the first part has to be 
 * match a QuantifiableVariable, the second one a
 * Term in order to model a pair of the not-free-in relation of
 * an Taclet.*/

package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.Constraint;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.ProgramMethod;
import de.uka.ilkd.key.logic.op.SVSubstitute;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.sort.Sort;

public class ContainsQueryCondition extends VariableConditionAdapter {

    private SchemaVariable sv;

    public ContainsQueryCondition(SchemaVariable sv) {
	this.sv = sv;
    }

    public SchemaVariable schemaVariable() {
	return sv;
    }

    public boolean check(SchemaVariable var, SVSubstitute term, 
			 SVInstantiations matchCond, 
			 Services services) {
	if (var != schemaVariable()) {
	    return true;  // not responsible
	}
	if (!(term instanceof Term)) {
	    return false;
	}
	return containsQuery((Term) term);	
    }

    public static boolean containsQuery(Term term) {	
	if (term.isRigid()) {
	    return false; // sufficient condition for optimisation
	}
	if (term.op() instanceof ProgramMethod) {
	    return true; // base case
	}
	for (int i=0; i<term.arity(); i++) {
	    if (term.sub(i).sort()==Sort.FORMULA) { // this improves
						    // performance
						     // but makes the
						     // condition only
						     // true for
						     // immediate
						     // subterms of
						     // formulas
		return false;
	    }
	    if (containsQuery(term.sub(i))) { //descent recursively
		return true;
	    }
	}
	return false;
    }

    public String toString() {
	return "(containsquery "+schemaVariable()+")";
    }

    public boolean equals(Object o) {
	if (!(o instanceof ContainsQueryCondition)) {
	    return false;
	}
	ContainsQueryCondition cqc=(ContainsQueryCondition)o;
	return (cqc.schemaVariable()==schemaVariable());
    }
    
    public int hashCode(){
    	int result = 17;
    	result = 37 * result + schemaVariable().hashCode();
    	return result;
    }

}

 
