// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/** class contains a pair of variables where the first variable is in
 * the scope of the second. The first variable can represent a logic
 * variable (i.e. is a VariableSV, a TermSV or a LogicVariable), the
 * second is a variable that can be bound, i.e. a QuantifiableVariable
 */
package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.op.LogicVariableRepresentative;
import de.uka.ilkd.key.logic.op.QuantifiableVariable;
import java.io.Serializable;
public class InScopePair implements Serializable {

    private LogicVariableRepresentative first;
    private QuantifiableVariable second;

    /**
     * constructs a pair of two SchemaVariables
     */
    public InScopePair(LogicVariableRepresentative first, 
		       QuantifiableVariable second) {
	this.first=first;
	this.second=second;
    }

    /** returns the first SchemaVariable of the pair. */
    public LogicVariableRepresentative first() {
	return first;
    }

    /** returns the second SchemaVariable of the pair. */
    public QuantifiableVariable second() {
	return second;
    }

    public String toString() {
	return "("+first()+","+second()+")";
    }

    public boolean equals(Object o) {
	InScopePair s=(InScopePair)o;
	return (first()==s.first() && second()==s.second());
    }

    public int hashCode () {
        return ((first()==null) ? 0 : first ().hashCode ()) * 23 
	    + ((second()==null) ? 0 : second().hashCode ());
    }
}

 
