// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/** this class is used to parse in Taclet from a file that are used by tests */
package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.proof.*;
import java.io.*;
import de.uka.ilkd.key.gui.ProofSettings;
import de.uka.ilkd.key.gui.AbbrevMap;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.recoderext.*;
import de.uka.ilkd.key.logic.ldt.LDT;
import de.uka.ilkd.key.logic.ldt.*;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.proof.init.*;

public class TacletForTests {

    private TacletForTests() {}

    public static final String testRules = System.getProperty("key.home")+
        File.separator+"system"+File.separator+"proofExamples"+
        File.separator+"testrules.key";
    public static String standardFile = testRules;
    public static NamespaceSet nss = new NamespaceSet();
    public static AbbrevMap scm = new AbbrevMap();
    public static TacletIndex rules= null;
    public static Proof[] problems;
    public static ProblemInitializer pi=null;
    public static Services services;
    public static Recoder2KeY r2k;
    public static File lastFile=null;

    
    public static void parse(File file) {
	try {	    
	    if (!file.equals(lastFile)) {
		ProblemInitializer.BASE_CONFIG = null;
		pi = ProblemInitializer.
		    problemInitializerForTests(new KeYFileForTests("Test", file)); 
		problems = pi.setUpTest().getProofs();
		nss      = pi.initConfig().namespaces(); 
		rules    = pi.initConfig().createTacletIndex();
		services = pi.initConfig().getServices();
		lastFile = file;
	    }
	} catch (Exception e) {
	    System.err.println("Exception occured while parsing "+file+"\n");
	    e.printStackTrace();
	    System.exit(-1);
	}

    }

    public static Services services() {
	if (services == null) parse();
	return services;
    }

    public static JavaInfo javaInfo() {
	return services ().getJavaInfo ();
    }

    public static JavaInfo getJavaInfo() {
	return javaInfo();
    }

    public static void setStandardFile(String filename) {
	standardFile=filename;
    }

    public static Proof[] problems() {
	return problems;
    }

    public static void parse(String filename) {
	parse(new File(filename));
    }

    public static void parse() {
	parse(standardFile);
    }
    
    public static NoPosTacletApp getTaclet(String filename, String name) {
	parse(filename);
	return rules.lookup(new Name(name));
    }

    public static NoPosTacletApp getTaclet(String name) {
	return rules.lookup(new Name(name));
    }
    
    public static AbbrevMap getAbbrevs(){
        return scm;
    }

    public static Namespace getSorts() {
	return nss.sorts();
    }

    public static TacletIndex getRules() {
	return rules;
    }


    public static Namespace getHeuristics() {
	return nss.heuristics();
    }

    public static Namespace getFunctions() {
	return nss.functions();
    }


    public static Namespace getVariables() {
	return nss.variables();
    }

    public static NamespaceSet getNamespaces() {
	return nss;
    }

    public static Function funcLookup(String name) {
	return (Function)getFunctions().lookup(new Name(name));
    }

    public static SchemaVariable svLookup(String name) {
	return (SchemaVariable)getVariables().lookup(new Name(name));
    }

    public static Sort sortLookup(String name) {
	return (Sort)getSorts().lookup(new Name(name));
    }

    private static Term parseTerm(String termstr, Services services) {
	if (termstr.equals("")) return null;
	try {	    
	    StringReader br   = new StringReader(termstr);
	    TermParser parser = new TermParser(new KeYLexer(br), 
					       "No file. " +
					       "TacletForTests.parseTerm("
					       +termstr+")",
					       TermFactory.DEFAULT, 
					       new Recoder2KeY
						   (services, nss),
					       services, nss,
					       TacletForTests.getAbbrevs());
	    return parser.term();
	} catch (Exception e) {
	    System.err.println("Exception during parsing!");
	    e.printStackTrace();
	    return null;
	}
	
    }

    public static Term parseTerm(String termstr, NamespaceSet set) {
	if (termstr.equals("")) return null;
	try {	    
	    StringReader br=new StringReader(termstr);
	    TermParser parser = new TermParser(new KeYLexer(br), 
					       "No file. " +
					       "TacletForTests.parseTerm("
					       +termstr+")",
					       TermFactory.DEFAULT, 
					       new Recoder2KeY(services(), set),
					       services(),
					       set,
					       new de.uka.ilkd.key.gui.AbbrevMap());
	    return parser.term();
	} catch (Exception e) {
	    System.err.println("Exception during parsing!");
	    e.printStackTrace();
	    return null;
	}
	
    }

    public static Term parseTerm(String termstr) {
	return parseTerm(termstr, services());
    }

    public static ProgramElement parsePrg(String prgString) {
	r2k = new Recoder2KeY(services(), new NamespaceSet());
	return r2k.readBlockWithEmptyContext(prgString).program();
    }

    public static ProgramElement parsePrg(String prgString, 
					  Services ownServices) {
	r2k = new Recoder2KeY(ownServices, new NamespaceSet());
	return r2k.readBlockWithEmptyContext(prgString).program();
    }

    public static ProgramElement parseExpr(String type, String prgString) {
	r2k=new Recoder2KeY(new Services(), new NamespaceSet());

	ProgramElement pe=r2k.readBlockWithEmptyContext("{"+type
	       					+" x="+prgString+";}").program();
        LocalVariableDeclaration ld
	    =(LocalVariableDeclaration)((StatementBlock)pe).getStatementAt(0);

	return ld.getVariableSpecifications().
	    getVariableSpecification(0).getInitializer();

    }


    public static Proof parseTermAsInitialProof(String termstr) {
	ConstrainedFormula cf=new ConstrainedFormula(parseTerm(termstr));
	Semisequent succ=Semisequent.EMPTY_SEMISEQUENT.insert(0,cf).semisequent();
	Sequent s=Sequent.createSuccSequent(succ);
	Proof proof = new Proof();
	proof.setRoot(new Node(proof, s));
	return proof;
    }

}
