// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.SimultaneousUpdateOp;
import de.uka.ilkd.key.logic.ListOfTerm;
import de.uka.ilkd.key.logic.SLListOfTerm;
import de.uka.ilkd.key.logic.IteratorOfTerm;
import de.uka.ilkd.key.util.Debug;

/** this class represents a pair of an update operator and the term
 * that is used to update the variable(attribute) of update operator
 */

public class UpdatePair {

    /** the update terms */
    private final Term update;
    
    public UpdatePair(Term update) {

	Debug.assertTrue(update.op() instanceof SimultaneousUpdateOp, 
			 "updatepair:: simultaneous update expected");
	
	this.update = update; 
    }

    public int arity() {
	return update.arity()-1;
    }
    
    public Term sub(int i) {	
	return update.sub(i);
    }
    
    
    private boolean equalEntries(UpdatePair cmp) {
	for (int i = 0; i<arity(); i++) {
	    if (!sub(i).equals(cmp.sub(i))) {
		return false;
	    }
	}
	return true;
    }

    public boolean equals(Object o) {
	if (!(o instanceof UpdatePair)) {
	    return false;
	}
	UpdatePair cmp = (UpdatePair) o;	
	return cmp.arity()==arity() && equalEntries(cmp);
    }
    
    public int hashCode(){
    	int result = 17;
    	result = 37 * result + arity();
    	for(int i = 0; i<arity(); i++){
    		result = 37 * result + sub(i).hashCode();
    	}
    	return result;
    }

    public String toString() {
	StringBuffer result = new StringBuffer("{");
	for (int i = 0; i<arity(); i+=2) {
 	    result.append(""+sub(i));
 	    result.append(":=");
 	    result.append(""+sub(i+1));

	    if (i < arity() + 2) {  	
		result.append(",");
	    }
 	}
	result.append("}");
	return result.toString();
    }
}
