// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.rule;


import de.uka.ilkd.key.proof.TacletInstantiationsTableModel;
import de.uka.ilkd.key.proof.ProofSaver;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.expression.operator.CopyAssignment;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.logic.op.SortedSchemaVariable;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.JavaBlock;
import de.uka.ilkd.key.java.visitor.ProgramReplaceVisitor;
import de.uka.ilkd.key.java.ContextStatementBlock;
import de.uka.ilkd.key.rule.inst.SVInstantiations;


public class VarNameDeliverer {

    private VarNameDeliverer() {}

    static boolean off;
    
    public static void setEnabled(boolean enabled) {
        off = !enabled;
    }


    // precondition: sv.sort()==ProgramSVSort.VARIABLE
    public static String getNameProposalForProgramVariable(
                                                    SortedSchemaVariable sv,
                                                    TacletApp app,
						    Services services) {
	if (off) return TacletInstantiationsTableModel.
            getNameProposalForSchemaVariable(sv);
        String proposal;
	boolean found = false;
	try {
	    IteratorOfTacletGoalTemplate templs =
	        app.taclet().goalTemplates().iterator();
            RewriteTacletGoalTemplate rwgt =null;
	    String name = "";
	    while (templs.hasNext() && !found) {	
                rwgt = (RewriteTacletGoalTemplate) templs.next();
	        Term t = findProgramInTerm(rwgt.replaceWith());
	        ContextStatementBlock c = 
                    (ContextStatementBlock) t.executableJavaBlock().program();
	        de.uka.ilkd.key.java.StatementBlock b = c.getMidBody();
		
		
		if (b.getChildAt(0) instanceof LocalVariableDeclaration) {
	            VariableSpecification v = 
                	((LocalVariableDeclaration) b.getChildAt(0)).
			getVariables().getVariableSpecification(0);
			
	            if (v.hasInitializer()) {
			ProgramElement rhs = instantiateExpression(
			    v.getInitializer(), app.instantiations(), services);
			name = ProofSaver.printProgramElement(rhs);
			break;
		    } else if (b.getChildAt(1) instanceof CopyAssignment) {
	        	CopyAssignment p2 = (CopyAssignment) b.getChildAt(1);
                	Expression lhs = p2.getExpressionAt(0);
                	if (lhs.equals(sv)) {
			     SortedSchemaVariable rhs = 
			         (SortedSchemaVariable) p2.getExpressionAt(1);
                             name=app.instantiations().getInstantiation(rhs).
			          toString();
                	     break;
                	}
		    }
		}
                
	    }
	    if ("".equals(name)) throw new Exception();
	    proposal = "["+name+"]";
	} catch(Exception e) { // bad style, but this is really non-critical
//	    System.err.println(e);
//	    e.printStackTrace();
	    proposal = TacletInstantiationsTableModel.
                getNameProposalForSchemaVariable(sv);
	}
	return proposal;
    }
    
    
    public static String getNameProposalForSchemaVariable(Expression e) {
	if (off) return "_var" + TacletInstantiationsTableModel.
	    globalVariableNameCnt++;
	return "["+ProofSaver.printProgramElement(e)+"]";
    }



    static Term findProgramInTerm(Term t) {
        if (t.javaBlock()!=JavaBlock.EMPTY_JAVABLOCK) return t;
	for(int i=0; i < t.arity(); i++) {
	    Term t1= findProgramInTerm(t.sub(i));
	    if (t1!=null) return t1;
	}
	return null;
    }
    
    
    
    static ProgramElement instantiateExpression(ProgramElement e, 
                                                SVInstantiations svInst,
						Services services) {
        ProgramReplaceVisitor trans = 
	    new ProgramReplaceVisitor(e, services, svInst, false);
	trans.start();
	return trans.result();
    }

}
