// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.rule;


import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.Constraint;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.SVSubstitute;
import de.uka.ilkd.key.java.Services;


/** 
 * This class represents additional generic conditions on the
 * instantiation of schema variables. 
 */

public interface VariableCondition {

    /**
     * checks if the condition for a correct instantiation is fulfilled
     * @param var the template Variable to be instantiated
     * @param instantiation the SVSubstitute (e.g. Term, ProgramElement) to be mapped to var
     * @param matchCond the MatchCondition with the current matching state and in particular 
     *    the SVInstantiations that are already known to be needed 
     * @param services the program information object
     * @return modified match results if the condition can be satisfied,
     * or <code>null</code> otherwise
     */
    MatchConditions check(SchemaVariable var, SVSubstitute term, 
		     MatchConditions matchCond, Services services);

}
