// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.Expression;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.ArrayOfExpression;
import de.uka.ilkd.key.java.statement.MethodBodyStatement;
import de.uka.ilkd.key.java.statement.MethodFrame;
import de.uka.ilkd.key.java.reference.ExecutionContext;
import de.uka.ilkd.key.java.reference.MethodReference;
import de.uka.ilkd.key.java.reference.VariableReference;
import de.uka.ilkd.key.java.reference.TypeRef;
import de.uka.ilkd.key.java.reference.TypeReference;
import de.uka.ilkd.key.java.reference.ReferencePrefix;
import de.uka.ilkd.key.java.reference.SuperReference;
import de.uka.ilkd.key.java.declaration.Modifier;
import de.uka.ilkd.key.java.declaration.MethodDeclaration;
import de.uka.ilkd.key.java.declaration.LocalVariableDeclaration;
import de.uka.ilkd.key.java.declaration.VariableSpecification;
import de.uka.ilkd.key.java.declaration.ParameterDeclaration;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.ArrayOfStatement;
import de.uka.ilkd.key.java.visitor.ProgVarReplaceVisitor;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.SortedSchemaVariable;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.op.ProgramMethod;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.logic.*;

import java.util.HashMap;

public class ExpandMethodBody extends ProgramMetaConstruct {

    private final SchemaVariable runtimeSV;
    static int counter = 0;

    public ExpandMethodBody(SchemaVariable sv, SchemaVariable runtimeSV) {
	super(new Name("expand-method-body"), (SortedSchemaVariable)sv);
	this.runtimeSV = runtimeSV;
    }

    public ExpandMethodBody(SchemaVariable sv) {
	this(sv, null);
    }

    /** 
     * Replaces the MethodBodyStatement shortcut with the full body,
     * performs prefix adjustments in the body (execution context).
     * @param services the Services with all necessary information 
     * about the java programs
     * @param svInst the instantiations esp. of the inner and outer label 
     * @return the transformed program
     */
    public ProgramElement symbolicExecution(ProgramElement pe,
					    Services services,
					    SVInstantiations svInst) {

	MethodBodyStatement mbs = (MethodBodyStatement) pe;
	//        MethodReference mr = mbs.getMethodReference();

        ProgramMethod pm = mbs.getProgramMethod();
	//mr.method(services, mbs.getBodySource());

	MethodDeclaration methDecl = pm.getMethodDeclaration();
	int params = methDecl.getParameterDeclarationCount();


	StatementBlock result = (StatementBlock) mbs.getBody(services);	
	ReferencePrefix newCalled = mbs.getDesignatedContext();
	TypeReference classContext = new TypeRef(mbs.getBodySource());

	result = prettyNewObjectNames(result, methDecl, classContext);

	// at this point all arguments should be program variables
	ArrayOfExpression argsAsParam = mbs.getArguments();

	HashMap map = new HashMap();
	for (int i = 0; i < argsAsParam.size(); i++) {
	    map.put(methDecl.getParameterDeclarationAt(i).
		    getVariableSpecification().getProgramVariable(), 
		    argsAsParam.getExpression(i));
	}
	ProgVarReplaceVisitor paramRepl = 
	    new ProgVarReplaceVisitor(result, map); 
	paramRepl.start();	
	result = (StatementBlock) paramRepl.result();

        return 
	    new MethodFrame(mbs.getResultVariable(),
			    new ExecutionContext(classContext, newCalled),
			    (StatementBlock)result); 
    }


    private String getReferencePrefixName(ReferencePrefix rp) {
	if (rp instanceof ProgramVariable) {
	    String s=((ProgramVariable)rp).name().toString();
	    if (s.lastIndexOf("::")>0) {
		s=s.substring(s.lastIndexOf("::")+1);
	    }
	    return s;
	} else if (rp instanceof VariableReference) {
	    return getReferencePrefixName(rp.getReferencePrefix())+"_"
		+((VariableReference)rp).getProgramElementName();
	}   
	return "this";       
    }
    
    private ProgramElementName getNewCalledName(MethodBodyStatement mbs) {
	ReferencePrefix rp=mbs.getDesignatedContext();
	return new ProgramElementName(mbs.getBodySource().getSort()+"::"
				      +getReferencePrefixName(rp));
    }




    /** replaces __NEW__ (formerly newObject) identifiers in instance init.
      * code with something like newMyClass1. Not that these live long, 
      * but still...
      */
    private StatementBlock prettyNewObjectNames(StatementBlock b,
                                                MethodDeclaration methDecl,
                                                final TypeReference 
                                                    classContext) {
        if (!methDecl.getName().toString().equals(
            de.uka.ilkd.key.java.recoderext.
                CreateObjectBuilder.IMPLICIT_OBJECT_CREATE)) return b;

        final String dummyVarName = de.uka.ilkd.key.java.recoderext.
            CreateObjectBuilder.NEW_OBJECT_VAR_NAME;
        final int n = counter++;

        ProgVarReplaceVisitor repl = new ProgVarReplaceVisitor(b) {

            ProgramVariable newPV = null;

            public void performActionOnProgramVariable(ProgramVariable pv) {
	        if (pv.name().toString().equals(dummyVarName)) {
                    if (newPV == null) newPV = new ProgramVariable(
                        new ProgramElementName(
                            "<new"+classContext.getName()+n+">"),
	                pv.getKeYJavaType());
	            addChild(newPV);
	            changed();
	        } else {
	            doDefaultAction(pv);
	        }
            }

        };
        repl.start();
        return (StatementBlock) repl.result();
    }


}
