// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.expression.literal.*;
import java.math.BigInteger;
import de.uka.ilkd.key.logic.op.AbstractMetaOperator;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.logic.Name;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.*;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.*;

import de.uka.ilkd.key.gui.*;

import java.util.*;

/** Is used for OCL simplification.
 * Extracts all subtypes of the given class from the 
 * current UML model.
 */
public class MetaAllSubtypes extends AbstractMetaOperator {

    private int arity;

    private TermFactory origTf = TermFactory.DEFAULT;

    public MetaAllSubtypes() {
	super(new Name("#allSubtypes"), 0);
    }

    /**
     * checks whether the top level structure of the given @link Term
     * is syntactically valid, given the assumption that the top level
     * operator of the term is the same as this Operator. The
     * assumption that the top level operator and the term are equal
     * is NOT checked.  
     * @return true iff the top level structure of
     * the @link Term is valid.
     */
    public boolean validTopLevel(Term term) {
	return true;
	//return term.arity()==arity();
    }


    /** calculates the resulting term. */
    public Term calculate(Term term, SVInstantiations svInst, Services services) {

	Main main = Main.getInstance();
	KeYMediator mediator = main.mediator();
	Namespace namespace = mediator.func_ns();

	TermSymbol nil = (Function)namespace.lookup(new Name("$nil"));
	TermSymbol cons = (Function)namespace.lookup(new Name("$cons"));
	TermSymbol aBEComposite = (Function)namespace.lookup(new Name("ABEComposite"));
	TermSymbol group = (Function)namespace.lookup(new Name("Group"));
	TermSymbol folder = (Function)namespace.lookup(new Name("Folder"));
	
	Term nilTerm = origTf.createFunctionTerm(nil);
	Term aBECompositeTerm = origTf.createFunctionTerm(aBEComposite);
	Term groupTerm = origTf.createFunctionTerm(group);
	Term folderTerm = origTf.createFunctionTerm(folder);
	Term listTerm1 = origTf.createFunctionTerm(cons, aBECompositeTerm, nilTerm);
	Term listTerm2 = origTf.createFunctionTerm(cons, folderTerm, listTerm1);
	Term listTerm3 = origTf.createFunctionTerm(cons, groupTerm, listTerm2);

	return listTerm3;	
    }
}
