// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.

package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.Field;
import de.uka.ilkd.key.java.abstraction.ListOfField;
import de.uka.ilkd.key.java.abstraction.SLListOfField;
import de.uka.ilkd.key.java.abstraction.IteratorOfField;
import de.uka.ilkd.key.java.abstraction.Type;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.java.abstraction.ClassType;
import de.uka.ilkd.key.java.abstraction.ArrayType;
import de.uka.ilkd.key.java.abstraction.NullType;
import de.uka.ilkd.key.java.abstraction.ArrayType;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.abstraction.SLListOfType;
import de.uka.ilkd.key.java.abstraction.SLListOfField;
import de.uka.ilkd.key.java.declaration.LocalVariableDeclaration;
import de.uka.ilkd.key.java.expression.ArrayInitializer;
import de.uka.ilkd.key.java.expression.literal.BooleanLiteral;
import de.uka.ilkd.key.java.expression.literal.IntLiteral;
import de.uka.ilkd.key.java.expression.operator.LessThan;
import de.uka.ilkd.key.java.expression.operator.LogicalOr;
import de.uka.ilkd.key.java.expression.operator.New;
import de.uka.ilkd.key.java.expression.operator.NewArray;
import de.uka.ilkd.key.java.expression.operator.PostIncrement;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.logic.ProgramElementName;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.logic.op.ProgramMethod;


import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.util.ExtList;
import java.util.LinkedList;

/**
 * Split an array creation expression with explicit array initializer,
 * creating a creation expression with dimension expression and a list
 * of assignments (-> Java language specification, 15.10)
 *
 * This meta construct delivers the creation expression
 */
public class ProcessIsTransient  extends ProgramMetaConstruct{

    private final SchemaVariable newObjectSV;
    private String createArrayName = new String("<createArray>");

    public ProcessIsTransient(SchemaVariable newObjectSV,
			     ProgramElement newExpr) {	
	super("process-isTransient", newExpr); 
	this.newObjectSV = newObjectSV;
    }

    public ProgramElement symbolicExecution(ProgramElement pe, 
					    Services services,
					    SVInstantiations svInst) {
	JavaInfo javaInfo = services.getJavaInfo ();

	Expression resultVar = (Expression)
	    svInst.getInstantiation ( newObjectSV );

	MethodReference mref = null;
	if( pe instanceof MethodReference) {
	    mref = (MethodReference)pe;
	    if(!mref.getName().toString().startsWith("jvmIsTransient"))
		return pe;
	} 

	LinkedList bodyStmt = new LinkedList();

	KeYJavaType integerType = javaInfo.getPrimitiveKeYJavaType("int");
	KeYJavaType objectType = javaInfo.getObjectKeYJavaType();
	ProgramMethod pm = javaInfo.getKeYProgModelInfo().getProgramMethod
	     (javaInfo.getKeYJavaTypeByClassName("javacard.framework.JCSystem"), 
	      "<jvmIsTransient>", 
	      SLListOfType.EMPTY_LIST.prepend(objectType), 
	      javaInfo.getKeYJavaTypeByClassName("javacard.framework.JCSystem"));
 	MethodReference mr = new MethodReference
	    (new ArrayOfExpression(mref.getArgumentAt(0)), 
	     new ProgramElementName("<jvmIsTransient>"), 
	     new TypeRef(javaInfo.getKeYJavaTypeByClassName
			 ("javacard.framework.JCSystem"))); 

	bodyStmt.addLast(assign(resultVar, mr));

 	return new StatementBlock
	    ((Statement[])bodyStmt.toArray(new Statement[bodyStmt.size()]));
    }
    

}
