// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.rule.metaconstruct.arith;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.expression.literal.*;
import java.math.BigInteger;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.gui.ProofSettings;
import de.uka.ilkd.key.logic.op.AbstractMetaOperator;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.logic.Name;
/** this class implements the interface for
 * MetaAdderators. MetaAdderators are used to do complex term
 * transformation when applying a taclet. Often these transformation
 * caanot be described with the taclet scheme (or trying to do so would
 * result in a huge number of rules)
 */
public class MetaMod extends AbstractMetaOperator {

    private int arity;

    private TermFactory termFactory = TermFactory.DEFAULT;

    public MetaMod() {
	super(new Name("#mod"), 2);
    }


    /**
     * checks whether the top level structure of the given @link Term
     * is syntactically valid, given the assumption that the top level
     * operator of the term is the same as this Operator. The
     * assumption that the top level operator and the term are equal
     * is NOT checked.  
     * @return true iff the top level structure of
     * the @link Term is valid.
     */
    public boolean validTopLevel(Term term) {
	// a meta operator accepts almost everything
	return term.op() instanceof MetaMod && term.arity()==arity();
    }


    /** calculates the resulting term. */
    public Term calculate(Term term, SVInstantiations svInst, Services services) {
	Term arg1 = term.sub(0);
	Term arg2 = term.sub(1);
	BigInteger bigIntArg1=null;
	BigInteger bigIntArg2=null;

	bigIntArg1 = new
	    BigInteger(convertToDecimalString(arg1, services));
	bigIntArg2 = new
	    BigInteger(convertToDecimalString(arg2, services));
	if (bigIntArg2.compareTo(new BigInteger("0"))==0) {
	    return termFactory.createFunctionTerm(new Function(new Name("undef"),
							       ProofSettings.THE_SETTINGS.
							       getLDTSettings().getNaturalNumberLDT().targetSort(), 
							       new Sort[0]));
	}	
	BigInteger bigIntResult = bigIntArg1.remainder(bigIntArg2);

	if (bigIntArg1.compareTo(BigInteger.ZERO)<0)
	    if (bigIntArg2.compareTo(BigInteger.ZERO)>0)
		bigIntResult = bigIntResult.add(bigIntArg2);
	    else
		bigIntResult = bigIntResult.subtract(bigIntArg2);
	
	    
	
	IntLiteral lit = new IntLiteral(bigIntResult.toString());
	return services.getTypeConverter().convertToLogicElement(lit);

    }

}
