// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.strategy;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Named;
import de.uka.ilkd.key.logic.Namespace;
import de.uka.ilkd.key.proof.Proof;

/**
 * Interface for creating Strategy instances.
 */
public abstract class StrategyFactory implements Named {
    
    /**
     * Get the selectable strategies.
     * @return namespace containing factories for user-selectable strategies
     */
    public static Namespace getStrategyNamespace () {
        final Namespace strategyNamespace = new Namespace ();
        
        strategyNamespace.add (new SimpleFOLStrategy.Factory ());
        strategyNamespace.add (new SimpleJavaCardDLNoSpecsStrategy.Factory ());
        strategyNamespace.add (new SimpleJavaCardDLWithoutLoopsStrategy.Factory ());        
	strategyNamespace.add (new SimpleJavaCardDLWithoutLoopsAndMethodBodiesStrategy.Factory ());	
	strategyNamespace.add (new SimpleJavaCardDLUseMethodContractsStrategy.Factory ());
        
        return strategyNamespace;
    }
    
    /**
     * Get a factory for the default strategy.
     * @return factory for the default strategy.
     */
    public static StrategyFactory getDefaultStrategy () {
        return new SimpleJavaCardDLNoSpecsStrategy.Factory ();
    }
    
    /**
     * Create the strategy with the specified name.
     * If there is no strategy with the specified name, this method behaves as
     * if the name of the default strategy was given.
     * @param proof the proof for which the strategy is to be created
     * @param name the name of the strategy
     * @return the strategy
     */
    public static Strategy create ( Proof proof, Name name ) {
        Namespace strategyNamespace = getStrategyNamespace ();
        
        StrategyFactory factory =
            (StrategyFactory) strategyNamespace.lookup(name);
        if ( factory == null )
            factory = StrategyFactory.getDefaultStrategy ();
        return factory.create ( proof );
    }
    
    /**
     * Create strategy for a proof.
     * @param proof
     * @return
     */
    public abstract Strategy create(Proof proof);
}
