// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.strategy.feature;

import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.rule.TacletApp;
import de.uka.ilkd.key.util.Debug;


/**
 * A binary feature that returns zero iff the constraint of the given taclet app
 * is stronger than the original constraint of the formula to which the rule is
 * applied (i.e. replacewith-clauses are changed into add-clauses)
 */
public class ConstraintStrengthenFeature extends BinaryTacletAppFeature {

    public final static Feature INSTANCE = new ConstraintStrengthenFeature (); 
    
    protected boolean filter (TacletApp app, PosInOccurrence pos, Goal goal) {
        Debug.assertFalse ( pos == null,
                            "Feature is only applicable to rules with find" );
        
        return !pos.constrainedFormula ().constraint ().isAsStrongAs
                                          ( app.constraint () );
    }

}
