// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.strategy.feature;

import de.uka.ilkd.key.logic.Constraint;
import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.proof.ConstraintTableModel;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.rule.TacletApp;
import de.uka.ilkd.key.util.Debug;


/**
 * This feature is similar to <code>ConstraintStrengthenFeature</code>, but
 * instead of just comparing the constraint resulting from a rule application
 * with the original constraint of the find-formula of the application, the new
 * constraint is compared with the joint of the original constraint and the user
 * constraint. I.e. the feature returns zero iff the new constraint is stronger
 * than <code>userConstraint.join(formulaConstraint)</code>.
 */
public class ConstraintStrengthenFeatureUC extends BinaryTacletAppFeature {

    private final ConstraintTableModel userConstraint;
    
    private ConstraintStrengthenFeatureUC (ConstraintTableModel userConstraint) {
        this.userConstraint = userConstraint;
    }

    public static ConstraintStrengthenFeatureUC create (Proof proof) {
        return new ConstraintStrengthenFeatureUC ( proof.getUserConstraint () );
    }
    
    protected boolean filter (TacletApp app, PosInOccurrence pos, Goal goal) {
        Debug.assertFalse ( pos == null,
                            "Feature is only applicable to rules with find" );

        final Constraint oriConstraint =
            pos.constrainedFormula ().constraint ()
            				.join ( userConstraint.getConstraint () );
        return !oriConstraint.isAsStrongAs ( app.constraint () );
    }

}
